unit JabberSrv;

interface

uses
  ActiveX, Classes, ComObj, StdVcl, SysUtils, Forms, SyncObjs,
  JabberCOM_TLB, JabberChatServer_TLB, ObjectPooler_osi,Windows;

type
{ Custom exceptions }
  EJabberServer = class(Exception);
  TCustomJabberSession = class;
  TServer=class;
  TCustomMessageType= set of (mtGroupChat, mtChat); //, mtCommunicationError, mtAuthenticationError);
  MessageLink = ^TMessageLinkedList;
  UserLink = ^TUserData;

  TMessageLinkedList = record
     Time: TDateTime;
     UserName: WideString;
     Text: WideString;
     MessageType: TCustomMessageType;
     GroupDisplayName: String;
     Next: MessageLink;
     Last: MessageLink;
  end;

  TUserData = record
     SessionUserName: WideString;
     GrabID: Integer;
     ObjectIndex: Integer;
     LastPresenceTime: TTimeStamp;
  end;

  { TCustomJabberSession }
  TCustomJabberSession = class (TJabberSession)
  private
    Owner:TComponent;
    FUsedAuthenticationType: JabberAuthType;
    FLastAccessTime: TTimeStamp;
    FLastErrorCode: Integer;
    FLastErrorMessage: WideString;
    FChatUsers: TList;
    FOutboxMessages: MessageLink;
    FInboxMessages: MessageLink;
    FCriticalSection: TCriticalSection;
    FRosterProcessingEnd:Boolean;
{$IFDEF debug}
    FDebugLog: WideString;
{$ENDIF}
    { private methods}  
    function EscapeCharacterData(const CDATA:String):String;
    function NeedToSendPresence(const UserName:String):Boolean;
    { private event handlers}
    procedure ActionOnAuthenticationError(AOwner: TObject; var txt: OleVariant);
    procedure ActionOnConnect(AOwner: TObject);
    procedure ActionOnCommunicationError(AOwner: TObject; var txt: OleVariant);
    procedure ActionOnReceiveMessage(AOwner: TObject; var Msg: OleVariant; var Tag: OleVariant);
    procedure ActionOnRosterEnd(AOwner: TObject);
    procedure ActionOnRegFields(Sender: TObject; var RegIQ: OleVariant);
    procedure ActionOnSubscriptionRequest(Sender: TObject; var FromJID: OleVariant; var SubType: OleVariant; var Status: OleVariant);
{$IFDEF debug}
    procedure ActionOnDebugXML(AOwner: TObject; Direction: Integer; var Text: OleVariant);
    procedure WriteDebugXML;
{$ENDIF}
  protected
    { protected methods}
    function getInboxMessages: WideString;
    function getRosterData: WideString;
    function getStatusForUser(const UserName: WideString): WideString;
    procedure setOutboxMessages(const ToUser, JabberServer, MessageText, GroupDisplayName: WideString; const MessageType: TCustomMessageType);
    procedure sendMessages;
    { Constructor/Destructor }
    constructor Create(AOwner:TComponent);
    destructor Destroy; override;
  end;

 { TPooledJabber }
  TPooledJabber = class
  private
    Owner:TComponent;
  protected
    { Public properties }
    JabberSessionObj:TCustomJabberSession;
    { Constructor/Destructor }
    constructor Create(AOwner:TComponent);
    destructor Destroy; override;
  end; { TPooledJabber }

  { TCleanTaskThread }
  TCleanTaskThread=class(TThread)
  private
    { Private fields }
    FOwner:TServer;
    hTerminateSemaphore:THandle;
  protected
    { Protected methods }
    procedure Execute;override;
    { Constructor/Destructor }
    constructor Create(AOwner:TServer);
    destructor Destroy;override;
  end; { TCleanTaskThread }

  { TServer }
  TServer = class
  private
    { Private variables }
    IniFilename:String;
    RefCount:Integer;
    JabberPooler:TObjectPooler_osi;
    FJabberUsers:TThreadList;
    FCleanTaskThread:TCleanTaskThread;
    FTimeZoneInformation: TTimeZoneInformation;
    CleanTaskInterval:Integer;
    SessionLifeSpan:Integer;
    SendPresenceInterval:Integer;
    JabberServer:String;
    JabberGroupChatServer:String;
    JabberPort:Integer;
    JabberUseSSL:String;
    { Private event handlers }
    procedure PooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
    procedure PooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
  protected
    { Constructor/Destructor }
    constructor Create;
    destructor Destroy; override;
    { Protected methods }
    function AutostartServer:Boolean;
    procedure SaveAutostartSettings(const Autostart:Boolean);
    function StartJabberPooler(const PoolSize, QueueLimit:Integer):Boolean;
    procedure StopJabberPooler;
    procedure RunCleanTask;
    procedure StopCleanTask;
    procedure getDataForJabberUser(const SessionUserName: String; out GrabID, ObjectIndex:Integer);
    procedure setDataForJabberUser(const SessionUserName: String; const GrabID, ObjectIndex:Integer);
    procedure removeDataForJabberUser(const SessionUserName: String);
  end; { TServer }
  
  { TJabberServer }
  TJabberServer = class(TTypedComObject, IJabberServer)
  protected
    function  Send(MaxWait: Integer; const FromUser: WideString; const ToUser: WideString; 
                   var MessageText: WideString; const MessageType: WideString; 
                   const GroupChatDisplayName: WideString): HResult; stdcall;
    function  Connect(MaxWait: Integer; const UserName: WideString; const Password: WideString; 
                      const Resource: WideString; out OutMessage: WideString): HResult; stdcall;
    function  Disconnect(const UserName: WideString; out OutValue: WideString): HResult; stdcall;
    function  GetLastMessages(MaxWait: Integer; const UserName: WideString; 
                              const StatusForUser: WideString; out MessagesXML: WideString): HResult; stdcall;
    function  GetRosterData(MaxWait: Integer; const UserName: WideString; out RosterXML: WideString): HResult; stdcall;
  end; { TJabberServer}

implementation

uses ComServ;

const
  COM_ERROR_BASE = -$200;
  COM_ERROR_NO_SESSION_FOR_USER = -$401;
  COM_ERROR_COMMUNICATION = -$402;
  COM_ERROR_AUTHENTICATION = -$403;
  JABBERERROR = COM_ERROR_BASE;
{$IFDEF debug}
  DebugDirPath='c:\';
{$ENDIF}

var
  AServer:TServer;
  ExceptionList:TStringList;

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                         TCustomJabberSession                                 }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TCustomJabberSession.Create(AOwner:TComponent);
begin { Create }
  inherited Create(AOwner);
  Owner:=AOwner;
  OnConnect:=ActionOnConnect;
  OnMessage:=ActionOnReceiveMessage;
  OnCommError:=ActionOnCommunicationError;
  OnAuthError:=ActionOnAuthenticationError;
  OnRosterEnd:=ActionOnRosterEnd;
  OnRegFields:=ActionOnRegFields;
  OnSubscriptionRequest:=ActionOnSubscriptionRequest;
{$IFDEF debug}
  OnXML:=ActionOnDebugXML;
{$ENDIF}
  FCriticalSection:=TCriticalSection.Create;
  FChatUsers:=TList.Create;
end; { Create }

destructor TCustomJabberSession.Destroy;
var
  ptr:MessageLink;
  I:Integer;
begin { Destroy }
   FCriticalSection.Enter;
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.Destroy ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
{$ENDIF}
   try
     while (FOutboxMessages <> nil) do begin
       ptr:=FOutboxMessages;
       FOutboxMessages:=FOutboxMessages^.Next;
       dispose(ptr);
     end;
     while (FInboxMessages <> nil) do begin
       ptr:=FInboxMessages;
       FInboxMessages:=FInboxMessages^.Next;
       dispose(ptr);
     end;
   finally
     FCriticalSection.Leave;
     for I:=0 to FChatUsers.Count-1 do begin
       dispose(FChatUsers.Items[I]);
       FChatUsers.Items[I]:=nil;
     end;
     FreeAndNil(FChatUsers);
     FCriticalSection.Free;
   end;
  inherited Destroy;
end; { Destroy }

{==============================================================================}
{                        private methods                                       }
{==============================================================================}
function TCustomJabberSession.EscapeCharacterData(const CDATA:string):string;
var
  Len, I:Integer;
begin { EscapeCharacterData }
  { This should be used as little as possible because of the extra iterations. }
  Result:='';
  Len:=Length(CDATA);
  for I:=1 to Len do
    case (CDATA[I]) of
      #34: Result:=Result+'&#34;';   // "
      #38: Result:=Result+'&#38;';   // &
      #39: Result:=Result+'&#39;';   // '
      #60: Result:=Result+'&#60;';   // <
      #62: Result:=Result+'&#62;';   // >
      #96: Result:=Result+'&#96;';   // `
      #1..#8,
      #11..#12,
      #14..#31,
      #127..#255:Result:=Result+'~'; // Valid XML characters are: Char    ::=    #x9 | #xA | #xD | [#x20-#xD7FF] | [
    else
      Result:=Result+CDATA[I];
    end;
end; { EscapeCharacterData }

function TCustomJabberSession.NeedToSendPresence(const UserName: string): Boolean;
var
    AUser: UserLink;
    I: Integer;
    NeedAdd: Boolean;
    RunTime: TTimeStamp;
    TimeDifference: Double;
begin { NeedToSendPresence }
  try
    Result:=false;
    NeedAdd:=true;
    RunTime:=DateTimeToTimeStamp(Now);
    for I:=0 to FChatUsers.Count-1 do begin
      AUser:=FChatUsers.Items[I];
      if ((AUser<>nil) and (AUser^.SessionUserName=UserName)) then begin
        TimeDifference:=(RunTime.Date-AUser^.LastPresenceTime.Date)*86400000 + (RunTime.Time-AUser^.LastPresenceTime.Time);
        if (TimeDifference > AServer.SendPresenceInterval*1000) then begin
          Result:=true;
          AUser^.LastPresenceTime:=DateTimeToTimeStamp(Now);
        end;
        NeedAdd:=false;
        break;
      end;
    end;
    if (NeedAdd=true) then begin
      Result:=true;
      new(AUser);
      AUser^.SessionUserName:=UserName;
      AUser^.LastPresenceTime:=DateTimeToTimeStamp(Now);
      FChatUsers.Add(AUser);
    end;
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.NeedToSendPresence ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    FDebugLog:=FDebugLog+#9'ToUser '+UserName+#13#10;
    if (Result) then
      FDebugLog:=FDebugLog+'NeedToSendPresence return true '+#13#10
    else
      FDebugLog:=FDebugLog+'NeedToSendPresence return false '+#13#10;
{$ENDIF}
  except
    on E:Exception do
      raise EJabberServer.Create('Exception (TCustomJabberSession.NeedToSendPresence): '+E.Message);
  end;
end; { NeedToSendPresence }
{==============================================================================}
{                        private event handlers                                }
{==============================================================================}
{$IFDEF debug}
procedure TCustomJabberSession.ActionOnDebugXML(AOwner: TObject; Direction: Integer; var Text: OleVariant);
begin
  if (Direction=0) then
    FDebugLog:=FDebugLog+'Coming in from the server'#13#10'START'#13#10+ Text+ #13#10'END'#13#10
  else
    FDebugLog:=FDebugLog+'Being sent from the client to the server'#13#10'START'#13#10+ Text+ #13#10'END'#13#10
end;

procedure TCustomJabberSession.WriteDebugXML;
var
 S:WideString;
 D:TDateTime;
begin
  FCriticalSection.Enter;
  try
    with TStringList.Create do
      try
      finally
        D:=Now;
        S:='Creation time:'+FormatDateTime('mm/dd/yyyy hh:mm:ss',D)+#13#10+FDebugLog;
        Text:=S;
        SaveToFile(DebugDirPath+'\'+FormatDateTime('yyyymmddhhmmss',D)+'_'+self.Username+'.txt');
        Free;
      end;
  finally
    FCriticalSection.Leave;
  end;
end;
{$ENDIF}

procedure TCustomJabberSession.ActionOnReceiveMessage(AOwner: TObject; var Msg: OleVariant; var Tag: OleVariant);
var
  ptr: MessageLink;
begin { ActionOnRecieveMessage }
  new (ptr);
  ptr^.Time:=Now;
  ptr^.UserName:=Msg.FromJID;
  ptr^.Text:=Msg.Body;
  ptr^.MessageType:=[mtChat];
  ptr^.GroupDisplayName:='';
  ptr^.Next:=nil;
  ptr^.Last:=ptr;
  FCriticalSection.Enter;
  try
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnReceiveMessage ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    FDebugLog:=FDebugLog+#9'From:'+Msg.FromJID+#13#10;
    FDebugLog:=FDebugLog+#9'Body:'+Msg.Body+#13#10;
{$ENDIF}
    if (FInboxMessages = nil) then
      FInboxMessages:=ptr
    else begin
      FInboxMessages^.Last^.Next:=ptr;
      FInboxMessages^.Last:=ptr;
    end;
  finally
    FCriticalSection.Leave;
  end;
end; {ActionOnRecieveMessage }

procedure TCustomJabberSession.ActionOnAuthenticationError(AOwner: TObject; var txt: OleVariant) ;
begin { ActionOnAuthenticationError }
    if (FUsedAuthenticationType = jatNoAuth) then begin
      DoConnect(false, jatAuto);   //one more try to connect, this time as existing user. Without creating of new account.
      FCriticalSection.Enter;
      try
        FUsedAuthenticationType:=jatAuto;
      finally
        FCriticalSection.Leave;
      end;
    end
    else begin
      FCriticalSection.Enter;
      try
{$IFDEF debug}
        FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnAuthenticationError ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
        FDebugLog:=FDebugLog+#9'ErrorText:'+txt+#13#10;
{$ENDIF}
        FLastErrorCode:=COM_ERROR_AUTHENTICATION;
        FLastErrorMessage:=txt;
      finally
        FCriticalSection.Leave;
      end;
    end;
end; { ActionOnAuthenticationError }

procedure TCustomJabberSession.ActionOnRosterEnd(AOwner: TObject);
begin { ActionOnRosterEnd }
  FCriticalSection.Enter;
  try
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnRosterEnd ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
{$ENDIF}
    FRosterProcessingEnd:=true;
  finally
    FCriticalSection.Leave;
  end;
end; { ActionOnRosterEnd }

procedure TCustomJabberSession.ActionOnRegFields(Sender: TObject; var RegIQ: OleVariant);
var RIQ: IJabberIQ;
begin { ActionOnRegFields }
{$IFDEF debug}
  FCriticalSection.Enter;
  try
    FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnRegFields ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
  finally
    FCriticalSection.Leave;
  end;
{$ENDIF}
  RIQ:=CreateIQ;
  RIQ.iqType:='result';
  RIQ.FromJID:=self.MyJID;
  RIQ.SetField('username',self.Username);
  RIQ.SetField('password',self.Password);
  SendRegister(RIQ);
end; { ActionOnRegFields }

procedure TCustomJabberSession.ActionOnSubscriptionRequest(Sender: TObject; var FromJID: OleVariant; var SubType: OleVariant; var Status: OleVariant);
begin { ActionOnSubscriptionRequest }
{$IFDEF debug}
  FCriticalSection.Enter;
  try
    FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnSubscriptionRequest ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    FDebugLog:=FDebugLog+#9'Type:'+SubType+#13#10;
    FDebugLog:=FDebugLog+#9'From:'+FromJID+#13#10;
    FDebugLog:=FDebugLog+#9'Status:'+Status+#13#10;
  finally
    FCriticalSection.Leave;
  end;
{$ENDIF}
  if (AnsiUpperCase(SubType)='SUBSCRIBE') then
    SendPresence(FromJID, jptSubscribed, 'Available', nil);
end; { ActionOnSubscriptionRequest }

procedure TCustomJabberSession.ActionOnCommunicationError(AOwner: TObject; var txt: OleVariant) ;
begin { ActionOnComunicationError }
  FCriticalSection.Enter;
  try
    FLastErrorCode:=COM_ERROR_COMMUNICATION;
    FLastErrorMessage:=txt;
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnCommunicationError ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    FDebugLog:=FDebugLog+#9'ErrorText:'+txt+#13#10;
  //  WriteDebugXML;
{$ENDIF}
  finally
    FCriticalSection.Leave;
  end;
end; { ActionOnComunicationError }

procedure TCustomJabberSession.ActionOnConnect(AOwner: TObject);
begin { ActionOnConnect }
  FCriticalSection.Enter;
  try
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.ActionOnConnect ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
{$ENDIF}
    FLastErrorCode:=0;
    FLastErrorMessage:='';
  finally
    FCriticalSection.Leave;
  end;
  SendXML('<presence type="available"><priority>1</priority><show>chat</show></presence>');
  sendMessages();
  Roster.Fetch;
//  SendMyPresence;
{
  !!Don't use SendMyPresence, because presence "unavailable"  will be send to the server
  here is XML sent to the server: <presence type="unavailable"><priority>1</priority><show>chat</show></presence>
  Use SendXML instead.
}
end; { ActionOnConnect }
{
procedure TCustomJabberSession.ActionOnRosterStart(AOwner: TObject);
begin { ActionOnRosterStart
  FRosterProcessingStart:=true;
end; { ActionOnRosterStart
}
{==============================================================================}
{                          protected methods                                   }
{==============================================================================}
procedure TCustomJabberSession.setOutboxMessages(const ToUser, JabberServer, MessageText, GroupDisplayName: WideString; const MessageType: TCustomMessageType);
var
  ptr: MessageLink;
begin { setOutboxMessages }
  new (ptr);
  ptr^.Time:=Now;
  ptr^.UserName:=ToUser+'@'+JabberServer;
  ptr^.Text:=MessageText;
  ptr^.MessageType:=MessageType;
  ptr^.GroupDisplayName:=GroupDisplayName;
  ptr^.Next:=nil;
  ptr^.Last:=ptr;
  FCriticalSection.Enter;
  try
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.setOutboxMessages ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    FDebugLog:=FDebugLog+#9'ToUser:'+ToUser+#13#10;
    FDebugLog:=FDebugLog+#9'MessageText:'+MessageText+#13#10;
{$ENDIF}
    if (FOutboxMessages = nil) then
      FOutboxMessages:=ptr
    else begin
      FOutboxMessages^.Last^.Next:=ptr;
      FOutboxMessages^.Last:=ptr;
    end;
  finally
    FCriticalSection.Leave;
  end;
end; { setOutboxMessages }

function TCustomJabberSession.getInboxMessages: WideString;
var
  ptr: MessageLink;
begin { getInboxMessages }
  (*
  if (Active) then
    Result:='<mm session="active">'
  else
    Result:='<mm session="inactive">';
  *)
  Result:='';
  FCriticalSection.Enter;
  try
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.getInboxMessages ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
{$ENDIF}
    while (FInboxMessages <> nil) do begin
      (*
      if (mtCommunicationError in FInboxMessages.MessageType) then
        ErrorType:='COMMUNICATION_ERROR'
      else if (mtAuthenticationError in FInboxMessages.MessageType) then
        ErrorType:='AUTHENTICATION_ERROR';
      *)
      Result:=Result+'<m time="' +FormatDateTime('hh:mm:ss AM/PM',FInboxMessages.Time+AServer.FTimeZoneInformation.Bias/60/24)+ '" from="' +EscapeCharacterData(FInboxMessages.UserName)+ '" to="'+EscapeCharacterData(self.UserName)+'" text="'+ EscapeCharacterData(FInboxMessages.Text) +'"/>'; // due to problems with XML parsing in FLASH we send message in attribute "text"  :(( <![CDATA['+FInboxMessages.Text+']]></m>';
      ptr:=FInboxMessages;
      FInboxMessages:=FInboxMessages^.Next;
      dispose(ptr);
    end;
{$IFDEF debug}
  FDebugLog:=FDebugLog+#9'MessagesXML:'+Result+#13#10;
{$ENDIF}
  finally
    FCriticalSection.Leave;
  end;
  //Result:=Result+'</mm>';
end; { getInboxMessages }

function TCustomJabberSession.getRosterData: WideString;
var
  I: Integer;
begin { getRosterData }
  Result:='';
  FCriticalSection.Enter;
  try
    if (FRosterProcessingEnd) then
      for I:=0 to self.Roster.Count-1 do
        Result:=Result+'<r nick="' +EscapeCharacterData(self.Roster.Items[I].NickName)+ '" jid="' +EscapeCharacterData(self.Roster.Items[I].JID)+ '" usid="' +EscapeCharacterData(self.Roster.Items[I].UserID)+ '"/>';
{$IFDEF debug}
  FDebugLog:=FDebugLog+'TCustomJabberSession.getRosterData ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
  Result:=Result+'<debug><![CDATA['#13#10+FDebugLog+#13#10'</debug>';
  FDebugLog:=FDebugLog+#9'RosterXML:'+Result+#13#10;
{$ENDIF}
  finally
    FCriticalSection.Leave;
  end;
end; { getRosterData }

function TCustomJabberSession.getStatusForUser(const UserName: WideString): WideString;
var
 UserJID: String;
 Presence: IJabberPres;
 RosterItem: IJabberRosterItem;
begin { getStatusForUser }
  Result:='Offline';
  UserJID:=UserName +'@'+ AServer.JabberServer;
  Presence:=GetPres(UserJID,'',true);
  if (Presence=nil) then
    if (self.FRosterProcessingEnd) then begin
      RosterItem:=Roster.FindUserID(UserJID);
      if (RosterItem=nil) then begin
        RosterItem:=Roster.Add(UserJID,UserJID);
        RosterItem.Ask:='subscribe';
        Result:='Awaiting Authorization from '+UserName;
        RosterItem.Update;
      end
      else  //if RosterItem <> nil
        if (RosterItem.Subscription=jstNone) then begin
          Result:='Awaiting Authorization from '+UserName;
          if (UpperCase(RosterItem.Ask)<>'SUBSCRIBE') then
            SendXML('<presence to="' +UserJID+ '" type="subscribe"><priority>0</priority></presence>');
        end;
    end
    else
      Result:='Unknown'
  else
    Result:=EscapeCharacterData(Presence.Status);
end; { getStatusForUser }

procedure TCustomJabberSession.sendMessages;
var
  ptr: MessageLink;
  JMsg:IJabberMsg;
  PresenceToUser:String;
begin { sendMessages }
  PresenceToUser:='';
  FCriticalSection.Enter;
  try
{$IFDEF debug}
    FDebugLog:=FDebugLog+'TCustomJabberSession.sendMessages ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
{$ENDIF}
    FLastAccessTime:=DateTimeToTimeStamp(Now);
    if (Active) then begin
      while (FOutboxMessages <> nil) do begin
        JMsg:=CreateMsg();
        PresenceToUser:=FOutboxMessages.UserName;
        if (mtGroupChat in FOutboxMessages.MessageType) then begin
          JMsg.MsgType:=jmtGroupChat;
          if (FOutboxMessages.GroupDisplayName <> '') then
            PresenceToUser:=PresenceToUser +'/'+ FOutboxMessages.GroupDisplayName
          else
            PresenceToUser:=PresenceToUser +'/'+ self.Username;
        end else
          JMsg.MsgType:=jmtChat;
        JMsg.ToJID:=FOutboxMessages.UserName;
        JMsg.Body:=FOutboxMessages.Text;
        if (NeedToSendPresence(PresenceToUser)) then  begin
{$IFDEF debug}
          FDebugLog:=FDebugLog+#9'SendPesence to user '+PresenceToUser+#13#10;
{$ENDIF}
          SendPresence(PresenceToUser, jptNormal, 'Available', nil);
        end;
{$IFDEF debug}
        FDebugLog:=FDebugLog+#9'SendMessage to user '+JMsg.ToJID+#13#10;
        FDebugLog:=FDebugLog+#9'SendMessage text:'+JMsg.Body+#13#10;
{$ENDIF}
        SendMessage(JMsg);
        ptr:=FOutboxMessages;
        FOutboxMessages:=FOutboxMessages^.Next;
        dispose(ptr);
      end;
    end;
  finally
    FCriticalSection.Leave;
  end;
end; { sendMessages }

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                TPooledJabber                                 }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TPooledJabber.Create(AOwner:TComponent);
begin { Create }
  inherited Create;
  Owner:=AOwner;
  JabberSessionObj:=TCustomJabberSession.Create(AOwner);
end; { Create }

destructor TPooledJabber.Destroy;
begin { Destroy }
  JabberSessionObj.Free;
  inherited Destroy;
end; { Destroy }

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                   TServer                                    }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TServer.Create;
begin { Create }
  inherited Create;
  { Init variables }
  IniFilename:=ExtractFilePath(Application.ExeName)+'JabberServer.ini';
  RefCount:=0;
  JabberPooler:=nil;
  FCleanTaskThread:=nil;
  FJabberUsers:=TThreadList.Create;
  GetTimeZoneInformation(FTimeZoneInformation);
end; { Create }

destructor TServer.Destroy;
var
 I:Integer;
begin { Destroy }
  StopCleanTask;
  JabberPooler.Free;
  JabberPooler:=nil;
  with FJabberUsers.LockList do
  try
    for I:=0 to Count-1 do
      if (Items[I] <> nil) then begin
        dispose(Items[I]);
        Items[I]:=nil;
      end;
  finally
    FJabberUsers.UnLockList;
  end;
  FreeAndNil(FJabberUsers);
  inherited Destroy;
end; { Destroy }
{==============================================================================}
{                            Private Event handlers                            }
{==============================================================================}
procedure TServer.PooledObjectCreate(Sender:TObjectPooler_osi; var PooledObject:TObject);
var
  NewPooledJabber:TPooledJabber;
begin { PooledObjectCreate }
  NewPooledJabber:=TPooledJabber.Create(Sender); //Assigning Sender as the owner of the TPooledJabber will ensure that if anything happens to the object pooler, all of the XSLT stuff will be destroyed.
  PooledObject:=NewPooledJabber;
end; { PooledObjectCreate }

procedure TServer.PooledObjectDestroy(Sender:TObjectPooler_osi; var PooledObject:TObject);
begin { PooledObjectDestroy }
  TPooledJabber(PooledObject).Free;
  PooledObject:=nil;
end; { PooledObjectDestroy }

{==============================================================================}
{                               Protected methods                              }
{==============================================================================}
function TServer.AutostartServer:Boolean;
var
  IniList:TStringList;
  AutoStart:String;
  PoolSize, QueueLimit:Integer;
begin { AutostartServer }
  Result:=False;
  { Load .ini settings }
  IniList:=TStringList.Create;
  Try
    IniList.LoadFromFile(IniFilename);
    AutoStart:=Trim(UpperCase(IniList.Values['autostart']));
    PoolSize:=StrToInt(Trim(IniList.Values['poolsize']));
    QueueLimit:=StrToInt(Trim(IniList.Values['queuelimit']));
    JabberServer:=Trim(IniList.Values['server']);
    JabberGroupChatServer:=Trim(IniList.Values['group_chat_server']);
    JabberPort:=StrToInt(Trim(IniList.Values['port']));
    JabberUseSSL:=Trim(IniList.Values['ssl']);
    CleanTaskInterval:=StrToInt(Trim(IniList.Values['cleantaskinterval']));
    SessionLifeSpan:=StrToInt(Trim(IniList.Values['sessionlifespan']));
    SendPresenceInterval:=StrToInt(Trim(IniList.Values['sendpresenceinterval']));

    If (AutoStart<>'TRUE') or (PoolSize<1) or (Queuelimit<-1)  Then
      Exit;
    { Start the server }
    if (StartJabberPooler(PoolSize,QueueLimit)) then begin
      RunCleanTask;
      Result:=true; end
    else
      Result:=false;
  Finally
    IniList.Free;
  End;
end; { AutostartServer }

procedure TServer.SaveAutostartSettings(const Autostart:Boolean);
var
  IniList:TStringList;
begin { SaveAutostartSettings }
  IniList:=TStringList.Create;
  Try
    If not(AutoStart) Then
      Begin
        { Turn off AutoStart, but keep the rest of the settings }
        IniList.LoadFromFile(IniFilename);
        IniList.Values['autostart']:='false';
      End
    Else
      Begin
        IniList.Add('autostart=true');
        IniList.Add('poolsize='+IntToStr(JabberPooler.PoolSize));
        IniList.Add('queuelimit='+IntToStr(JabberPooler.QueueLimit));
        IniList.Add('server='+JabberServer);
        IniList.Add('group_chat_server='+JabberGroupChatServer);
        IniList.Add('port='+IntToStr(JabberPort));
        IniList.Add('ssl='+JabberUseSSL);
        IniList.Add('cleantaskinterval='+IntToStr(CleanTaskInterval));
        IniList.Add('sessionlifespan='+IntToStr(SessionLifeSpan));
        IniList.Add('sendpresenceinterval='+IntToStr(SendPresenceInterval));
      End;
    IniList.SaveToFile(IniFilename);
  Finally
    IniList.Free;
  End;
end; { SaveAutostartSettings }

function TServer.StartJabberPooler(const PoolSize, QueueLimit:Integer):Boolean;
begin { StartJabberPooler }
  Result:=False;
  If not(Assigned(JabberPooler)) Then
    Begin
      JabberPooler:=TObjectPooler_osi.Create(nil);
      Try
        JabberPooler.PoolSize:=PoolSize;
        JabberPooler.QueueLimit:=QueueLimit;
        JabberPooler.OnPooledObjectCreate:=PooledObjectCreate;
        JabberPooler.OnPooledObjectDestroy:=PooledObjectDestroy;
        JabberPooler.Enabled:=True;
        SaveAutostartSettings(True);
        Result:=True;
      Except
        JabberPooler.Free;
        JabberPooler:=nil;
        raise;
      End;
    End;
  //Don't need reference counting because the server should get started once and left alone.
  //Inc(RefCount);
end; { StartJabberPooler }

procedure TServer.StopJabberPooler;
begin
  //Don't need reference counting because the server should get started once and left alone.
  //If RefCount=1 Then
    //Begin
      JabberPooler.Free;
      JabberPooler:=nil;
    //End;
  //Dec(RefCount);
end;

procedure TServer.RunCleanTask;
begin { RunCleanTask }
  if (FCleanTaskThread = nil) then
    FCleanTaskThread:=TCleanTaskThread.Create(self);
end; { RunCleanTask }

procedure TServer.StopCleanTask;
begin { StopCleanTask }
  if (FCleanTaskThread <> nil) then begin
    FCleanTaskThread.Terminate;
    ReleaseSemaphore(FCleanTaskThread.hTerminateSemaphore,1,nil);
    FCleanTaskThread:=nil;
  end;
end; { StopCleanTask }

procedure TServer.getDataForJabberUser(const SessionUserName: String; out GrabID, ObjectIndex:Integer);
var
    I: Integer;
begin { getDataForJabberUser }
  try
    with FJabberUsers.LockList do
    try
      for I:=0 to Count-1 do
        if ((Items[I]<>nil) and (UserLink(Items[I])^.SessionUserName=SessionUserName)) then begin
          GrabID:=UserLink(Items[I])^.GrabID;
          ObjectIndex:=UserLink(Items[I])^.ObjectIndex;
          break;
        end;
    finally
      FJabberUsers.UnlockList;
    end;
  except
    on E:Exception do
      raise EJabberServer.Create('Exception (TServer.getDataForJabberUser): '+E.Message);
  end;
end; { getDataForJabberUser }

procedure TServer.setDataForJabberUser(const SessionUserName: String; const GrabID, ObjectIndex:Integer);
var
  AUser: UserLink;
begin { setDataForJabberUser }
  new (AUser);
  AUser^.SessionUserName:=SessionUserName;
  AUser^.GrabID:=GrabID;
  AUser^.ObjectIndex:=ObjectIndex;
  FJabberUsers.Add(AUser);
end; { setDataForJabberUser }

procedure TServer.removeDataForJabberUser(const SessionUserName: String);
var
  I: Integer;
begin { removeDataForJabberUser }
  with FJabberUsers.LockList do
  try
    I:=0;
    while I<Count do
      if ((Items[I]<>nil) and (UserLink(Items[I])^.SessionUserName=SessionUserName)) then begin
        dispose(Items[I]);
        Items[I]:=nil;
        Delete(I);
        break;
      end
      else
        Inc(I);
  finally
    FJabberUsers.UnlockList;
  end;
end; { removeDataForJabberUser }


{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                             TCleanTaskThread                                 }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TCleanTaskThread.Create(AOwner:TServer);
begin { Create }
  inherited Create(false);
  FOwner:=AOwner;
  hTerminateSemaphore:=CreateSemaphore(nil,0,1,nil);
  FreeOnTerminate:=True;
end; { Create }

destructor TCleanTaskThread.Destroy;
begin { Destroy }
  CloseHandle(hTerminateSemaphore);
  inherited Destroy;
end; { Destroy }

{==============================================================================}
{                               Protected methods                              }
{==============================================================================}
procedure TCleanTaskThread.Execute;
var
  I: Integer;
  TimeDifference: Double;
  RunTime:TTimeStamp;
  ObjRef:TPooledObjectRef;
begin
  repeat
   if (WaitForSingleObject(hTerminateSemaphore,FOwner.CleanTaskInterval*1000)=WAIT_TIMEOUT) then begin
     try
       RunTime:=DateTimeToTimeStamp(Now);
       with FOwner.FJabberUsers.LockList do
         try
           I:=0;
           while I<Count do
             if (Items[I]<>nil) then begin
               ObjRef.GrabID:=UserLink(Items[I])^.GrabID;
               ObjRef.ObjectIndex:=UserLink(Items[I])^.ObjectIndex;
               try
                 with TPooledJabber(FOwner.JabberPooler.Objects[ObjRef]) do begin
                   JabberSessionObj.FCriticalSection.Enter;
                   try
                     TimeDifference:=(RunTime.Date-JabberSessionObj.FLastAccessTime.Date)*86400000 + (RunTime.Time-JabberSessionObj.FLastAccessTime.Time);
                   finally
                     JabberSessionObj.FCriticalSection.Leave;
                   end;
                   if (TimeDifference > FOwner.SessionLifeSpan*1000) then begin
                     if (JabberSessionObj.Active) then
                       JabberSessionObj.DoDisconnect(false);
                     FOwner.JabberPooler.ReleaseObject(ObjRef);
                     dispose(Items[I]);
                     Items[I]:=nil;
                     Delete(I);
                   end
                   else
                     Inc(I);
                 end;
               except
               end;
             end
             else
               Inc(I);
         finally
          FOwner.FJabberUsers.UnlockList;
         end;
     except
       on E:Exception do
         raise EJabberServer.Create('Exception (TCleanTaskThread.Execute): '+E.Message);
     end;
   end;
  until (Terminated);
end;

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                             TJabberServer                                    }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                          protected methods                                   }
{==============================================================================}
function TJabberServer.Send(MaxWait: Integer; const FromUser: WideString; const ToUser: WideString;
                            var MessageText: WideString; const MessageType: WideString;
                            const GroupChatDisplayName: WideString): HResult; stdcall;
var
  ObjRef:TPooledObjectRef;
  GrabID,ObjectIndex,LastErrorCode:Integer;
  PooledJabber:TPooledJabber;
begin { Send }
  GrabID:=-1;
  ObjectIndex:=-1;
  Result:=0;
  try
    AServer.getDataForJabberUser(FromUser,GrabID,ObjectIndex);
    if ((GrabID=-1) and (ObjectIndex=-1)) then begin
      MessageText:='No Session for user '+ FromUser +'. Please connect first.';
      Result:=COM_ERROR_NO_SESSION_FOR_USER; end
    else begin
      ObjRef.GrabID:=GrabID;
      ObjRef.ObjectIndex:=ObjectIndex;
      PooledJabber:=TPooledJabber(AServer.JabberPooler.Objects[ObjRef]);
      if not (PooledJabber.JabberSessionObj.Active) then begin
        PooledJabber.JabberSessionObj.FCriticalSection.Enter;
        try
{$IFDEF debug}
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+'TJabberServer.Send ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+#9'Session is inactive'+#13#10;
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+#9'text:'+MessageText+#13#10;
{$ENDIF}
          LastErrorCode:=PooledJabber.JabberSessionObj.FLastErrorCode;
        finally
          PooledJabber.JabberSessionObj.FCriticalSection.Leave;
        end;
        if (LastErrorCode <> 0) then begin
          PooledJabber.JabberSessionObj.FCriticalSection.Enter;
          try
            MessageText:=PooledJabber.JabberSessionObj.FLastErrorMessage;
            Result:=PooledJabber.JabberSessionObj.FLastErrorCode;
            Exit; //cancel processing
{$IFDEF debug}
           PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+#9'TJabberServer.Send there is error:'+IntToStr(PooledJabber.JabberSessionObj.FLastErrorCode)+#13#10;
           PooledJabber.JabberSessionObj.WriteDebugXML;
{$ENDIF}
          finally
            PooledJabber.JabberSessionObj.FCriticalSection.Leave;
          end;
        end
        else // if LastErrorCode=0  and session is Inactive then try to connect;
          PooledJabber.JabberSessionObj.DoConnect(false, jatAuto);
      end;
      if (MessageType='CHAT') then
        PooledJabber.JabberSessionObj.setOutboxMessages(ToUser, AServer.JabberServer, MessageText, '', [mtChat])
      else //  MessageType='GROUP_CHAT' 
        PooledJabber.JabberSessionObj.setOutboxMessages(ToUser, AServer.JabberGroupChatServer, MessageText, GroupChatDisplayName, [mtGroupChat]);
      PooledJabber.JabberSessionObj.sendMessages;
    end;
  except
    on E:Exception do begin
        MessageText:='Exception (TJabberServer.Send):  '+E.Message;
        Result:=JABBERERROR;
    end;
  end;
end; { Send }

function TJabberServer.GetLastMessages(MaxWait: Integer; const UserName: WideString;
                                       const StatusForUser: WideString;
                                       out MessagesXML: WideString): HResult; stdcall;
var
  ObjRef:TPooledObjectRef;
  PooledJabber:TPooledJabber;
  GrabID,ObjectIndex,LastErrorCode:Integer;
begin { GetLastMessages }
  try
    GrabID:=-1;
    ObjectIndex:=-1;
    Result:=0;
    AServer.getDataForJabberUser(UserName, GrabID, ObjectIndex);
    if ((GrabID=-1) and (ObjectIndex=-1)) then begin
      MessagesXML:='No Session for user '+ UserName +'. Please connect first.';
      Result:=COM_ERROR_NO_SESSION_FOR_USER; end
    else begin
      ObjRef.GrabID:=GrabID;
      ObjRef.ObjectIndex:=ObjectIndex;
      PooledJabber:=TPooledJabber(AServer.JabberPooler.Objects[ObjRef]);
      if not (PooledJabber.JabberSessionObj.Active) then begin
        PooledJabber.JabberSessionObj.FCriticalSection.Enter;
        try
{$IFDEF debug}
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+'TJabberServer.GetLastMessages ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+#9'Session is inactive'+#13#10;
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+#9'UserName:'+UserName+#13#10;
{$ENDIF}
          LastErrorCode:=PooledJabber.JabberSessionObj.FLastErrorCode;
        finally
          PooledJabber.JabberSessionObj.FCriticalSection.Leave;
        end;
        if (LastErrorCode <> 0) then begin
          PooledJabber.JabberSessionObj.FCriticalSection.Enter;
          try
            MessagesXML:=PooledJabber.JabberSessionObj.FLastErrorMessage;
            Result:=PooledJabber.JabberSessionObj.FLastErrorCode;
{$IFDEF debug}
    PooledJabber.JabberSessionObj.FDebugLog:=PooledJabber.JabberSessionObj.FDebugLog+#9'TJabberServer.GetLastMessages there is error:'+IntToStr(PooledJabber.JabberSessionObj.FLastErrorCode)+#13#10;
    PooledJabber.JabberSessionObj.WriteDebugXML;
{$ENDIF}
          finally
            PooledJabber.JabberSessionObj.FCriticalSection.Leave;
          end;
        end
        else // if LastErrorCode = 0 and Session is Inactive
          MessagesXML:='<mm session="inactive">' +PooledJabber.JabberSessionObj.getInboxMessages+ '</mm>';
      end
      else // if Session is Active
        if (StatusForUser=EmptyStr) then
          MessagesXML:='<mm session="active">' +PooledJabber.JabberSessionObj.getInboxMessages+ '</mm>'
        else
          MessagesXML:='<mm session="active" userstatus="' +PooledJabber.JabberSessionObj.getStatusForUser(StatusForUser)+ '">' +PooledJabber.JabberSessionObj.getInboxMessages+ '</mm>';
    end;
  except
      on E:Exception do
      begin
        MessagesXML:='Exception (TJabberServer.GetLastMessages):  '+E.Message;
        Result:=JABBERERROR;
      end;
  end;
end; { GetLastMessages }

function  TJabberServer.Connect(MaxWait: Integer; const UserName: WideString; const Password: WideString;
                                const Resource: WideString; out OutMessage: WideString): HResult; stdcall;
var
  ObjRef:TPooledObjectRef;
  GrabID, ObjectIndex, I:Integer;
  PooledJabber:TPooledJabber;
  ptr:MessageLink;
  NeedToAddUser:Boolean;
begin { Connect }
  try
    GrabID:=-1;
    ObjectIndex:=-1;
    Result:=0;
    NeedToAddUser:=false;
    AServer.getDataForJabberUser(UserName, GrabID, ObjectIndex);
    if ((GrabID=-1) and (ObjectIndex=-1)) then  begin
      ObjRef:=AServer.JabberPooler.GrabObject(MaxWait);
      if ObjRef.GrabID<0 Then
        raise EJabberServer.Create('connect: '+AServer.JabberPooler.ParseError(ObjRef.GrabID));
      PooledJabber:=TPooledJabber(AServer.JabberPooler.Objects[ObjRef]);
      NeedToAddUser:=true;
      PooledJabber.JabberSessionObj.UserName:=UserName;
      PooledJabber.JabberSessionObj.Password:=Password;
      PooledJabber.JabberSessionObj.Server:=AServer.JabberServer;
      PooledJabber.JabberSessionObj.Port:=AServer.JabberPort;
      PooledJabber.JabberSessionObj.Resource:=Resource;
      PooledJabber.JabberSessionObj.Priority:=1;
      PooledJabber.JabberSessionObj.ShowType:=jshowChat;
      PooledJabber.JabberSessionObj.Status:='Available';
      PooledJabber.JabberSessionObj.Available:=true;
      if (AServer.JabberUseSSL='Y') then
        PooledJabber.JabberSessionObj.UseSSL:=true
      else
        PooledJabber.JabberSessionObj.UseSSL:=false;
      //reset initial values for TCustomJabberSession Object. If the object was previously used it goes back from OSI_Pooler with old values.
      with TCustomJabberSession(PooledJabber.JabberSessionObj) do begin
        for I:=0 to FChatUsers.Count-1 do
          if (FChatUsers.Items[I] <> nil) then begin
            dispose(FChatUsers.Items[I]);
            FChatUsers.Items[I]:=nil;
          end;
        FChatUsers.Clear;
        FCriticalSection.Enter;
        try
          FRosterProcessingEnd:=false;
          FUsedAuthenticationType:=jatNoAuth;
          while (FOutboxMessages <> nil) do begin
            ptr:=FOutboxMessages;
            FOutboxMessages:=FOutboxMessages^.Next;
            dispose(ptr);
          end;
          while (FInboxMessages <> nil) do begin
            ptr:=FInboxMessages;
            FInboxMessages:=FInboxMessages^.Next;
            dispose(ptr);
          end;
        finally
          FCriticalSection.Leave;
        end;
      end;
      PooledJabber.JabberSessionObj.DoConnect(true, jatAuto); //jatNoAuth);
      OutMessage:='CONNECTION_NEW';
    end
    else begin
      ObjRef.GrabID:=GrabID;
      ObjRef.ObjectIndex:=ObjectIndex;
      PooledJabber:=TPooledJabber(AServer.JabberPooler.Objects[ObjRef]);
      if not (PooledJabber.JabberSessionObj.Active) then begin
        with PooledJabber.JabberSessionObj.FChatUsers do   //set "LastPresenceTime" to 0 for each chat user for this session. This way presence ("Available") will be send to user next time we send message.
          for I:=0 to Count-1 do
            if (Items[I] <> nil) then begin
              UserLink(Items[I])^.LastPresenceTime.Date:=0;
              UserLink(Items[I])^.LastPresenceTime.Time:=0;
            end;
        PooledJabber.JabberSessionObj.DoConnect(false, jatAuto);
        OutMessage:='CONNECTION_POOL_RECONNECT';
      end else
        OutMessage:='CONNECTION_POOL';
    end;
    // always goes here
    with TCustomJabberSession(PooledJabber.JabberSessionObj) do begin
      FCriticalSection.Enter;
      try
        FLastAccessTime:=DateTimeToTimeStamp(Now);
        FLastErrorCode:=0;
        FLastErrorMessage:='';
{$IFDEF debug}
        FDebugLog:='TJabberServer.Connect ['+ FormatDateTime('mm/dd/yyyy hh:mm:ss',Now) +']'+#13#10;
{$ENDIF}
      finally
        FCriticalSection.Leave;
      end;
    end;
    if (NeedToAddUser) then
      AServer.setDataForJabberUser(UserName, ObjRef.GrabID, ObjRef.ObjectIndex);
  except
      on E:Exception do begin
        OutMessage:='Exception (TJabberServer.Connect):  '+E.Message;
        Result:=JABBERERROR;
      end;
  end;  
end; { Connect }

function  TJabberServer.GetRosterData(MaxWait: Integer; const UserName: WideString; out RosterXML: WideString): HResult; stdcall;
var
  ObjRef:TPooledObjectRef;
  PooledJabber:TPooledJabber;
  GrabID,ObjectIndex,LastErrorCode:Integer;
begin  { GetRosterData }
  try
    GrabID:=-1;
    ObjectIndex:=-1;
    Result:=0;
    AServer.getDataForJabberUser(UserName, GrabID, ObjectIndex);
    if ((GrabID=-1) and (ObjectIndex=-1)) then begin
      RosterXML:='No Session for user '+ UserName +'. Please connect first.';
      Result:=COM_ERROR_NO_SESSION_FOR_USER; end
    else begin
      ObjRef.GrabID:=GrabID;
      ObjRef.ObjectIndex:=ObjectIndex;
      PooledJabber:=TPooledJabber(AServer.JabberPooler.Objects[ObjRef]);
      if not (PooledJabber.JabberSessionObj.Active) then begin
        PooledJabber.JabberSessionObj.FCriticalSection.Enter;
        try
          LastErrorCode:=PooledJabber.JabberSessionObj.FLastErrorCode;
{$IFDEF debug}
          LastErrorCode:=0;  //Don't exit if we are in debug mode;
{$ENDIF}
        finally
          PooledJabber.JabberSessionObj.FCriticalSection.Leave;
        end;
        if (LastErrorCode <> 0) then begin
          PooledJabber.JabberSessionObj.FCriticalSection.Enter;
          try
            RosterXML:=PooledJabber.JabberSessionObj.FLastErrorMessage;
            Result:=PooledJabber.JabberSessionObj.FLastErrorCode;
          finally
            PooledJabber.JabberSessionObj.FCriticalSection.Leave;
          end;
        end
        else // if LastErrorCode = 0 and Session is Inactive
          RosterXML:='<rr session="inactive">' +PooledJabber.JabberSessionObj.getRosterData+ '</rr>';
      end
      else // if session is Active
        RosterXML:='<rr session="active">' +PooledJabber.JabberSessionObj.getRosterData+ '</rr>';
    end;
  except
    on E:Exception do begin
      RosterXML:='Exception (TJabberServer.GetRosterData):  '+E.Message;
      Result:=JABBERERROR;
    end;
  end;
end; { GetRosterData }

function  TJabberServer.Disconnect(const UserName: WideString; out OutValue: WideString): HResult; stdcall;
var
  ObjRef:TPooledObjectRef;
  GrabID,ObjectIndex:Integer;
begin { Disconnect }
  try
    Result:=0;
    GrabID:=-1;
    ObjectIndex:=-1;
    AServer.getDataForJabberUser(UserName, GrabID, ObjectIndex);
    if ((GrabID<>-1) and (ObjectIndex<>-1)) then begin
      ObjRef.GrabID:=GrabID;
      ObjRef.ObjectIndex:=ObjectIndex;
{$IFDEF debug}
    TPooledJabber(AServer.JabberPooler.Objects[ObjRef]).JabberSessionObj.FDebugLog:=TPooledJabber(AServer.JabberPooler.Objects[ObjRef]).JabberSessionObj.FDebugLog+'Disconnect'+#13#10;
    TPooledJabber(AServer.JabberPooler.Objects[ObjRef]).JabberSessionObj.WriteDebugXML;
{$ENDIF}
      if (TPooledJabber(AServer.JabberPooler.Objects[ObjRef]).JabberSessionObj.Active) then
        TPooledJabber(AServer.JabberPooler.Objects[ObjRef]).JabberSessionObj.DoDisconnect(false);
      AServer.JabberPooler.ReleaseObject(ObjRef);
      AServer.removeDataForJabberUser(UserName);
    end;
  except
    on E:Exception do begin
        OutValue:='Exception (TJabberServer.Disconnect): '+e.Message;
        Result:=JABBERERROR;
    end;
  end;
end; { Disconnect }

initialization
{==============================================================================}
{                                Initialization                                }
{==============================================================================}
  Try
    { Create TServer object }
    AServer:=TServer.Create;
    CoInitializeEx(nil,COINIT_MULTITHREADED); //COM gets initialized by the the OmniHTTPd and IIS web servers, so this isn't called in the ISAPIs.  However, this is needed to keep the CreateOleObject calls from failing during autostart.
    If not(AServer.AutostartServer) Then
      raise EJabberServer.Create('Server refused to autostart.');
    { Class factory call }
    TTypedComObjectFactory.Create(ComServer, TJabberServer, CLASS_JabberServer, ciMultiInstance, tmFree);
  Except
    On E:Exception Do
      Begin
        ExceptionList:=TStringList.Create;
        Try
          ExceptionList.Add(DateTimeToStr(Now));
          ExceptionList.Add('Unable to autostart JabberServer: '+E.Message);
          ExceptionList.SaveToFile(ExtractFilePath(Application.ExeName)+'jabberserver_startup_failure.txt');
        Finally
          ExceptionList.Free;
        End;
        raise;
      End;
  End;

finalization
{==============================================================================}
{                                 Finalization                                 }
{==============================================================================}
  AServer.Free;
  CoUninitialize; //See note above (after CoInitializeEx() call).
end.
end.
