unit WebSessionsServer;

interface

uses
  Windows, ActiveX, Classes, ComObj, WebSesServer_TLB, SysUtils, WebSessions,
  Forms, StdVcl;

type
  { Custom exceptions }
  EWebSessionsServer = class(Exception);

  { TServer }
  TServer=class
  private
    { Private variables }
    DatFilename:String;
    IniFilename:String;
    RunJanitorImp:Boolean;
    PersistSessions:Boolean;
    WebSessions:TWebSessions;
    function BooleanToString(Value:Boolean):String;
    function StringToBoolean(const Value:String):Boolean;
  protected
    { Protected methods }
    function AutostartServer:Boolean;
    procedure ReadWebSessions;
    procedure SaveAutostartSettings(const Autostart:Boolean);
    procedure StartJanitorImp(SleepInterval:Cardinal);
    function StartWebSessions(Lifespan,MaxLifespan:Integer;TouchSessions,RunJanitor:Boolean;JanitorInterval,DeleteLag:Integer;TrackStatistics:Boolean):Boolean;
    procedure StopWebSessions;
    procedure WriteWebSessions;
  public
    { Constructor/Destructor }
    constructor Create;
    destructor Destroy;override;
  end; { TServer }

  { TWebSessionsServer }
  TWebSessionsServer = class(TTypedComObject, IWebSessionsServer)
  protected
    { Declare IWebSessionsServer methods here }
    function InvalidateWebSession(const SessionString: WideString;
      out SessionExists: WordBool; out ErrorMsg: WideString): HResult;
      stdcall;
    function NewWebSession(const SessionLookupKey, SessionData,
      SessionPrivileges, SessionDataFilters: WideString; out SessionString,
      ErrorMsg: WideString): HResult; stdcall;
    function NewWebSessionEx(UserID: Integer;
      const Username, UserFirstName, UserLastName: WideString;
      SuperUser: WordBool; const UserPrivileges, UserDataFilters,
      UserMiscData, Application, SessionGroup, SessionClass: WideString;
      Lifespan, MaxLifespan, TouchOnValidate: Integer; out SessionString,
      ErrorMsg: WideString): HResult; stdcall;
    function ResetStatistics(out ErrorMsg: WideString): HResult; stdcall;
    function StartServer(Lifespan, MaxLifespan: Integer; TouchSessions,
      RunJanitor: WordBool; JanitorInterval, DeleteLag: Integer;
      TrackStatistics: WordBool; out ErrorMsg: WideString): HResult;
      stdcall;
    function Statistics(out StatisticsString, ErrorMsg: WideString): HResult;
      stdcall;
    function StopServer(out ErrorMsg: WideString): HResult; stdcall;
    function UpdateSessionData(const SessionString, SessionData: WideString;
      out SessionExists: WordBool; out ErrorMsg: WideString): HResult;
      stdcall;
    function UpdateWebSession(const SessionString, UserPrivileges,
      UserDataFilters, UserMiscData: WideString; Lifespan,
      TouchOnValidate: Integer; out SessionExists: WordBool;
      out ErrorMsg: WideString): HResult; stdcall;
    function UserSessions(UserID: Integer; const Application: WideString;
      out SessionStrings, ErrorMsg: WideString): HResult; stdcall;
    function WebSession(var SessionString: WideString; out SessionData,
      SessionPrivileges, SessionDataFilters,
      ErrorMsg: WideString): HResult; stdcall;
    function WebSessionEx(var SessionString: WideString; out UserID: Integer;
      out Username, UserFirstName, UserLastName: WideString;
      out SuperUser: WordBool; out UserPrivileges, UserDataFilters,
      UserMiscData, Application, SessionGroup, SessionClass: WideString;
      out Lifespan: Integer; out TouchOnValidate: WordBool;
      out CreateDateTime, ExpireDateTime, MaxExpireDateTime: Double;
      out SessionExists: WordBool; out ErrorMsg: WideString): HResult;
      stdcall;
    function WebSessionInfo(out OutValue: WideString): HResult; stdcall;
    function WebSessionLookup(const SessionLookupKey: WideString;
      out SessionString, ErrorMsg: WideString): HResult; stdcall;
  end; { TWebSessionsServer }

implementation

uses
  ComServ, SyncObjs, Hash;

const
  ERROR_RESULT=-$200;
  SUCCESS_RESULT=$0;

var
  GlobalCriticalSection:TCriticalSection;
  Server:TServer;

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                                  TServer                                     }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                            Constructor/Destructor                            }
{==============================================================================}
constructor TServer.Create;
begin { Create }
  inherited;
  DatFilename:=ExtractFilePath(Application.ExeName)+'WebSessionsServer.dat';
  IniFilename:=ExtractFilePath(Application.ExeName)+'WebSessionsServer.ini';
end; { Create }

destructor TServer.Destroy;
begin { Destroy }
  If (PersistSessions) Then
    WriteWebSessions;
  WebSessions.Free;
  inherited;
end; { Destroy }

{==============================================================================}
{                                Private methods                               }
{==============================================================================}
function TServer.BooleanToString(Value:Boolean):String;
begin { BooleanToString }
  If (Value) Then
    Result:='true'
  Else
    Result:='false';
end; { BooleanToString }

function TServer.StringToBoolean(const Value:String):Boolean;
begin { StringToBoolean }
  Result:=(Trim(UpperCase(Value))='TRUE');
end; { StringToBoolean }

{==============================================================================}
{                               Protected methods                              }
{==============================================================================}
function TServer.AutostartServer:Boolean;
var
  AutoStart:String;
  TrackStatistics,TouchSessions:Boolean;
  Lifespan,MaxLifespan,JanitorInterval,DeleteLag:Integer;
begin { AutostartServer }
  Result:=False;
  { Load .ini settings }
  With TStringList.Create Do
    Try
      LoadFromFile(IniFilename);
      AutoStart:=Trim(UpperCase(Values['autostart']));
      TrackStatistics:=StringToBoolean(Values['trackstatistics']);
      TouchSessions:=StringToBoolean(Values['touchsessions']);
      RunJanitorImp:=StringToBoolean(Values['runjanitor']);
      Lifespan:=StrToInt(Trim(Values['lifespan']));
      MaxLifespan:=StrToInt(Trim(Values['maxlifespan']));
      JanitorInterval:=StrToInt(Trim(Values['janitorinterval']));
      PersistSessions:=StringToBoolean(Values['persistsessions']);
      DeleteLag:=StrToInt(Trim(Values['deletelag']));
      { Start the server }
      If (AutoStart='TRUE') Then
        Begin
          Result:=StartWebSessions(Lifespan,MaxLifespan,TouchSessions,RunJanitorImp,JanitorInterval,DeleteLag,TrackStatistics);
          If (PersistSessions) Then
            ReadWebSessions;
        End;
    Finally
      Free;
    End;
end; { AutostartServer }

procedure TServer.ReadWebSessions;
var
  FS:TFileStream;
begin { ReadWebSessions }
  If (FileExists(DatFilename)) Then
    Begin
      FS:=TFileStream.Create(DatFilename,fmOpenRead or fmShareDenyWrite);
      Try
        WebSessions.ReadSessions(FS);
      Finally
        FS.Free;
      End;
    End;
end; { ReadWebSessions }

procedure TServer.SaveAutostartSettings(const Autostart:Boolean);
begin { SaveAutostartSettings }
  With TStringList.Create Do
    Try
      If not(AutoStart) Then
        Begin
          { Turn off AutoStart, but keep the rest of the settings }
          LoadFromFile(IniFilename);
          Values['autostart']:='false';
        End
      Else
        Begin
          Add('autostart=true');
          Add('trackstatistics='+BooleanToString(WebSessions.TrackStatistics));
          Add('touchsessions='+BooleanToString(WebSessions.DefaultTouchSessionOnValidate));
          Add('runjanitor='+BooleanToString(RunJanitorImp));
          Add('lifespan='+IntToStr(WebSessions.DefaultSessionLifespan));
          Add('maxlifespan='+IntToStr(WebSessions.DefaultMaxSessionLifespan));
          Add('janitorinterval='+IntToStr(WebSessions.JanitorSleepInterval));
          Add('persistsessions='+BooleanToString(PersistSessions));
          Add('deletelag='+IntToStr(WebSessions.DeleteLag));
        End;
      SaveToFile(IniFilename);
    Finally
      Free;
    End;
end; { SaveAutostartSettings }

procedure TServer.StartJanitorImp(SleepInterval:Cardinal);
begin { StartJanitorImp }
  If (SleepInterval>0) Then
    WebSessions.JanitorSleepInterval:=SleepInterval;
  WebSessions.JanitorImpStart;
end; { StartJanitorImp }

function TServer.StartWebSessions(Lifespan,MaxLifespan:Integer;TouchSessions,RunJanitor:Boolean;JanitorInterval,DeleteLag:Integer;TrackStatistics:Boolean):Boolean;
begin { StartWebSessions }
  Result:=False;
  If (Assigned(WebSessions)) then
    Exit;
  WebSessions:=TWebSessions.Create(nil);
  Try
    WebSessions.DefaultSessionLifespan:=Lifespan;
    WebSessions.DefaultMaxSessionLifespan:=MaxLifespan;
    WebSessions.DefaultTouchSessionOnValidate:=TouchSessions;
    WebSessions.TrackStatistics:=TrackStatistics;
    WebSessions.DeleteLag:=DeleteLag;
    RunJanitorImp:=RunJanitor;
    If (RunJanitor) Then
      StartJanitorImp(JanitorInterval);
    SaveAutostartSettings(True);
    Result:=True;
  Except
    WebSessions.Free;
    WebSessions:=nil;
    Raise;
  End;
end; { StartWebSessions }

procedure TServer.StopWebSessions;
begin { StopWebSessions }
  WebSessions.Free;
  WebSessions:=nil;
end; { StopWebSessions }

procedure TServer.WriteWebSessions;
var
  FS:TFileStream;
begin { WriteWebSessions }
  FS:=TFileStream.Create(DatFilename,fmCreate or fmShareDenyWrite);
  Try
    WebSessions.WriteSessions(FS);
  Finally
    FS.Free;
  End;
end; { WriteWebSessions }

{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{                            TWebSessionsServer                                }
{++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
{==============================================================================}
{                               Protected methods                              }
{==============================================================================}
function TWebSessionsServer.InvalidateWebSession(
  const SessionString: WideString; out SessionExists: WordBool;
  out ErrorMsg: WideString): HResult;
begin { InvalidateWebSession }
  Try
    Result:=SUCCESS_RESULT;
    SessionExists:=Server.WebSessions.InvalidateWebSession(SessionString);
  Except
    On e:Exception Do
      Begin
        SessionExists:=False;
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { InvalidateWebSession }

{ DEPRECATED by NewWebSessionEx }
function TWebSessionsServer.NewWebSession(const SessionLookupKey,
  SessionData, SessionPrivileges, SessionDataFilters: WideString;
  out SessionString, ErrorMsg: WideString): HResult;
begin { NewWebSession }
  Try
    Result:=SUCCESS_RESULT;
    SessionString:=Server.WebSessions.NewWebSession(0,'','','',False,SessionPrivileges,SessionDataFilters,SessionData,'','','',0,0,0).SessionString; //SessionLookupKey is ignored entirely
  Except
    On e:Exception Do
      Begin
        SessionString:='';
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { NewWebSession }

{ DEPRECATES NewWebSession }
function TWebSessionsServer.NewWebSessionEx(UserID: Integer;
  const Username, UserFirstName, UserLastName: WideString;
  SuperUser: WordBool; const UserPrivileges, UserDataFilters, UserMiscData,
  Application, SessionGroup, SessionClass: WideString; Lifespan,
  MaxLifespan, TouchOnValidate: Integer; out SessionString,
  ErrorMsg: WideString): HResult;
begin { NewWebSessionEx }
  Try
    Result:=SUCCESS_RESULT;
    SessionString:=Server.WebSessions.NewWebSession(UserID,Username,UserFirstName,UserLastName,SuperUser,UserPrivileges,UserDataFilters,UserMiscData,Application,SessionGroup,SessionClass,Lifespan,MaxLifespan,TouchOnValidate).SessionString;
  Except
    On e:Exception Do
      Begin
        SessionString:='';
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { NewWebSessionEx }

function TWebSessionsServer.ResetStatistics(
  out ErrorMsg: WideString): HResult;
begin { ResetStatistics }
  Try
    Result:=SUCCESS_RESULT;
    Server.WebSessions.ResetStatistics;
  Except
    On e:Exception Do
      Begin
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { ResetStatistics }

function TWebSessionsServer.StartServer(Lifespan, MaxLifespan: Integer;
  TouchSessions, RunJanitor: WordBool; JanitorInterval, DeleteLag: Integer;
  TrackStatistics: WordBool; out ErrorMsg: WideString): HResult;
begin { StartServer }
  Try
    Result:=SUCCESS_RESULT;
    GlobalCriticalSection.Enter;
    Try
      Server.StartWebSessions(Lifespan,MaxLifespan,TouchSessions,RunJanitor,JanitorInterval,DeleteLag,TrackStatistics);
    Finally
      GlobalCriticalSection.Leave;
    End;
  Except
    On e:Exception Do
      Begin
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { StartServer }

function TWebSessionsServer.Statistics(out StatisticsString,
  ErrorMsg: WideString): HResult;
var
  Stats:TWebSessionStatistics;
begin { Statistics }
  Try
    Result:=SUCCESS_RESULT;
    GlobalCriticalSection.Enter;
    Try
      Stats:=Server.WebSessions.Statistics;
    Finally
      GlobalCriticalSection.Leave;
    End;
    With Stats Do
      Begin
        StatisticsString:='CurrentDateTime='+FormatDateTime('c',Now);
        StatisticsString:=StatisticsString+#13#10'SessionCount='+IntToStr(SessionCount);
        StatisticsString:=StatisticsString+#13#10'JanitorRuns='+IntToStr(JanitorRuns);
        StatisticsString:=StatisticsString+#13#10'JanitorLastRun='+FormatDateTime('c',JanitorLastRun);
        StatisticsString:=StatisticsString+#13#10'JanitorSessionsDeletedLastRun='+IntToStr(JanitorSessionsDeletedLastRun);
        StatisticsString:=StatisticsString+#13#10'JanitorTotalSessionsDeleted='+IntToStr(JanitorTotalSessionsDeleted);
        StatisticsString:=StatisticsString+#13#10'LastJanitorTime='+FloatToStrF(LastJanitorTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'TotalJanitorTime='+FloatToStrF(TotalJanitorTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'AverageJanitorTime='+FloatToStrF(AverageJanitorTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'JanitorTimePerSessionDeleted='+FloatToStrF(JanitorTimePerSessionDeleted,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'Retrievals='+IntToStr(Retrievals);
        StatisticsString:=StatisticsString+#13#10'RetrievalSuccesses='+IntToStr(RetrievalSuccesses);
        StatisticsString:=StatisticsString+#13#10'RetrievalFailures='+IntToStr(RetrievalFailures);
        StatisticsString:=StatisticsString+#13#10'LastRetrievalSuccess='+FormatDateTime('c',LastRetrievalSuccess);
        StatisticsString:=StatisticsString+#13#10'LastRetrievalFailure='+FormatDateTime('c',LastRetrievalFailure);
        StatisticsString:=StatisticsString+#13#10'LastRetrievalTime='+FloatToStrF(LastRetrievalTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'TotalRetrievalTime='+FloatToStrF(TotalRetrievalTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'AverageRetrievalTime='+FloatToStrF(AverageRetrievalTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'Inserts='+IntToStr(Inserts);
        StatisticsString:=StatisticsString+#13#10'LastInsert='+FormatDateTime('c',LastInsert);
        StatisticsString:=StatisticsString+#13#10'LastInsertTime='+FloatToStrF(LastInsertTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'TotalInsertTime='+FloatToStrF(TotalInsertTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'AverageInsertTime='+FloatToStrF(AverageInsertTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'Restorations='+IntToStr(Restorations);
        StatisticsString:=StatisticsString+#13#10'LastRestoration='+FormatDateTime('c',LastRestoration);
        StatisticsString:=StatisticsString+#13#10'LastRestorationTime='+FloatToStrF(LastRestorationTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'RestorationSuccesses='+IntToStr(RestorationSuccesses);
        StatisticsString:=StatisticsString+#13#10'RestorationFailures='+IntToStr(RestorationFailures);
        StatisticsString:=StatisticsString+#13#10'TotalRestorationTime='+FloatToStrF(TotalRestorationTime,ffFixed,15,3)+' ms.';
        StatisticsString:=StatisticsString+#13#10'AverageRestorationTime='+FloatToStrF(AverageRestorationTime,ffFixed,15,3)+' ms.';
      End;
  Except
    On e:Exception Do
      Begin
        StatisticsString:='';
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { Statistics }

function TWebSessionsServer.StopServer(out ErrorMsg: WideString): HResult;
begin { StopServer }
  Try
    Result:=SUCCESS_RESULT;
    GlobalCriticalSection.Enter;
    Try
      Server.StopWebSessions;
      Server.SaveAutostartSettings(False);
    Finally
      GlobalCriticalSection.Leave;
    End;
  Except
    On e:Exception Do
      Begin
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { StopServer }

{ DEPRECATED by UpdateWebSession }
function TWebSessionsServer.UpdateSessionData(const SessionString,
  SessionData: WideString; out SessionExists: WordBool;
  out ErrorMsg: WideString): HResult;
var
  S:TWebSession;
begin { UpdateSessionData }
  Try
    Result:=SUCCESS_RESULT;
    S:=Server.WebSessions.WebSession(SessionString);
    If (S=nil) Then
      SessionExists:=False
    Else
      Begin
        SessionExists:=True;
        S.UserMiscData:=SessionData;
      End;
  Except
    On e:Exception Do
      Begin
        SessionExists:=False;
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { UpdateSessionData }

{ DEPRECATES UpdateSessionData }
function TWebSessionsServer.UpdateWebSession(const SessionString,
  UserPrivileges, UserDataFilters, UserMiscData: WideString; Lifespan,
  TouchOnValidate: Integer; out SessionExists: WordBool;
  out ErrorMsg: WideString): HResult;
var
  S:TWebSession;
begin { UpdateWebSession }
  Try
    Result:=SUCCESS_RESULT;
    S:=Server.WebSessions.WebSession(SessionString);
    If (S=nil) Then
      SessionExists:=False
    Else
      Begin
        SessionExists:=True;
        If (UserPrivileges<>'') Then
          S.UserPrivileges:=UserPrivileges;
        If (UserDataFilters<>'') Then
          S.UserDataFilters:=UserDataFilters;
        If (UserMiscData<>'') Then
          S.UserMiscData:=UserMiscData;
        If (Lifespan>0) Then
          S.Lifespan:=Lifespan;
        If (TouchOnValidate<0) Then
          S.TouchOnValidate:=False
        Else
          If (TouchOnValidate>0) Then
            S.TouchOnValidate:=True;
      End;
  Except
    On e:Exception Do
      Begin
        SessionExists:=False;
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { UpdateWebSession }

{ DEPRECATES WebSessionLookup }
function TWebSessionsServer.UserSessions(UserID: Integer;
  const Application: WideString; out SessionStrings,
  ErrorMsg: WideString): HResult;
var
  UserSessions:TList;
  i:Integer;
begin { UserSessions }
  Try
    Result:=SUCCESS_RESULT;
    SessionStrings:='';
    UserSessions:=Server.WebSessions.UserSessions(Application,UserID);
    If (UserSessions<>nil) Then
      For i:=0 to UserSessions.Count-1 Do
        SessionStrings:=SessionStrings+TWebSession(UserSessions[i]).SessionString+#13#10;
  Except
    On e:Exception Do
      Begin
        SessionStrings:='';
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { UserSessions }

{ DEPRECATED by WebSessionEx }
function TWebSessionsServer.WebSession(var SessionString: WideString;
  out SessionData, SessionPrivileges, SessionDataFilters,
  ErrorMsg: WideString): HResult;
var
  S:TWebSession;
begin { WebSession }
  Try
    Result:=SUCCESS_RESULT;
    S:=Server.WebSessions.WebSession(SessionString);
    If (S=nil) Then
      Begin
        SessionString:='';
        SessionData:='';
        SessionPrivileges:='';
        SessionDataFilters:='';
      End
    Else
      Begin
        SessionString:=S.SessionString;
        SessionData:=S.UserMiscData;
        SessionPrivileges:=S.UserPrivileges;
        SessionDataFilters:=S.UserDataFilters;
      End;
  Except
    On e:Exception Do
      Begin
        SessionString:='';
        SessionData:='';
        SessionPrivileges:='';
        SessionDataFilters:='';
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { WebSession }

{ DEPRECATES WebSession }
function TWebSessionsServer.WebSessionEx(var SessionString: WideString;
  out UserID: Integer; out Username, UserFirstName,
  UserLastName: WideString; out SuperUser: WordBool; out UserPrivileges,
  UserDataFilters, UserMiscData, Application, SessionGroup,
  SessionClass: WideString; out Lifespan: Integer;
  out TouchOnValidate: WordBool; out CreateDateTime, ExpireDateTime,
  MaxExpireDateTime: Double; out SessionExists: WordBool;
  out ErrorMsg: WideString): HResult;
var
  S:TWebSession;
begin { WebSessionEx }
  Try
    Result:=SUCCESS_RESULT;
    S:=Server.WebSessions.WebSession(SessionString);
    If (S=nil) Then
      Begin
        SessionExists:=False;
        SessionString:='';
        UserID:=0;
        Username:='';
        UserFirstName:='';
        UserLastName:='';
        SuperUser:=False;
        UserPrivileges:='';
        UserDataFilters:='';
        UserMiscData:='';
        Application:='';
        SessionGroup:='';
        SessionClass:='';
        Lifespan:=0;
        TouchOnValidate:=False;
        CreateDateTime:=0;
        ExpireDateTime:=0;
        MaxExpireDateTime:=0;
      End
    Else
      Begin
        SessionExists:=True;
        SessionString:=S.SessionString;
        UserID:=S.UserID;
        Username:=S.Username;
        UserFirstName:=S.UserFirstName;
        UserLastName:=S.UserLastName;
        SuperUser:=S.SuperUser;
        UserPrivileges:=S.UserPrivileges;
        UserDataFilters:=S.UserDataFilters;
        UserMiscData:=S.UserMiscData;
        Application:=S.Application;
        SessionGroup:=S.SessionGroup;
        SessionClass:=S.SessionClass;
        Lifespan:=S.Lifespan;
        TouchOnValidate:=S.TouchOnValidate;
        CreateDateTime:=S.CreateDateTime;
        ExpireDateTime:=S.ExpireDateTime;
        MaxExpireDateTime:=S.MaxExpireDateTime;
      End;
  Except
    On e:Exception Do
      Begin
        SessionString:='';
        UserID:=0;
        Username:='';
        UserFirstName:='';
        UserLastName:='';
        SuperUser:=False;
        UserPrivileges:='';
        UserDataFilters:='';
        UserMiscData:='';
        Application:='';
        SessionGroup:='';
        SessionClass:='';
        Lifespan:=0;
        TouchOnValidate:=False;
        CreateDateTime:=0;
        ExpireDateTime:=0;
        MaxExpireDateTime:=0;
        SessionExists:=False;
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { WebSessionEx }

{ DEPRECATE? }
function TWebSessionsServer.WebSessionInfo(
  out OutValue: WideString): HResult;
var
  InfoText:String;
  SessionArray:TObjectArray;
  i:Integer;
begin { WebSessionInfo }
  SessionArray:=nil;
  Try
    Result:=SUCCESS_RESULT;
    InfoText:='';
    GlobalCriticalSection.Enter;
    Try
      SessionArray:=Server.WebSessions.SessionsArray;
      For i:=0 to High(SessionArray) Do
        With TWebSession(SessionArray[i]) Do
          InfoText:=InfoText+SessionString+#127+UserMiscData+#127+FormatDateTime('yyyymmddhhnnss',ExpireDateTime)+#127+FormatDateTime('yyyymmddhhnnss',MaxExpireDateTime)+#127;
      SessionArray:=nil;
    Finally
      GlobalCriticalSection.Leave;
    End;
    OutValue:=InfoText;
  Except
    On e:Exception Do
      Begin
        OutValue:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { WebSessionInfo }

{ DEPRECATED by UserSessions }
function TWebSessionsServer.WebSessionLookup(
  const SessionLookupKey: WideString; out SessionString,
  ErrorMsg: WideString): HResult;
const
  SetOfDigits=['0','1','2','3','4','5','6','7','8','9'];
var
  LookupKey,Application:String;
  i,IDPos,KeyLen,UserID:Integer;
  UserSessions:TList;
begin { WebSessionLookup }
  Try
    Result:=SUCCESS_RESULT;
    LookupKey:=SessionLookupKey;
    KeyLen:=Length(LookupKey);
    IDPos:=0;
    i:=1;
    While (i<=KeyLen) and (IDPos=0) Do
      Begin
        If (LookupKey[i] in SetOfDigits) Then
          IDPos:=i;
        Inc(i);
      End;
    If (IDPos=0) Then
      Raise EWebSessionsServer.Create('TWebSessionsServer.WebSessionLookup: Unable to parse Application and UserID from SessionLookupKey');
    Application:=Copy(LookupKey,1,IDPos-1);
    Try
      UserID:=StrToInt(Copy(LookupKey,IDPos,KeyLen));
    Except
      Raise EWebSessionsServer.Create('TWebSessionsServer.WebSessionLookup: String conversion error--invalid UserID parsed from SessionLookupKey');
    End;
    UserSessions:=Server.WebSessions.UserSessions(Application,UserID);
    If (UserSessions=nil) or (UserSessions.Count=0) Then
      SessionString:=''
    Else
      SessionString:=TWebSession(UserSessions[0]).SessionString;
  Except
    On e:Exception Do
      Begin
        SessionString:='';
        ErrorMsg:=e.Message;
        Result:=ERROR_RESULT;
      End;
  End;
end; { WebSessionLookup }

initialization
  Try
    { Create the critical section }
    GlobalCriticalSection:=TCriticalSection.Create;
    { Create TServer object }
    Server:=TServer.Create;
    If not(Server.AutostartServer) then
      Raise EWebSessionsServer.Create('Server refused to autostart.');
    { Class factory call }
    TTypedComObjectFactory.Create(ComServer, TWebSessionsServer, Class_WebSessionsServer,
      ciMultiInstance, tmFree);
  Except
    On E:Exception Do
      Begin
        With TStringList.Create Do
          Try
            Add(DateTimeToStr(Now));
            Add('Unable to autostart WebSessionsServer: '+E.Message);
            SaveToFile(ExtractFilePath(Application.ExeName)+'websesserver_startup_failure.txt');
          Finally
            Free;
          End;
        Raise;
      End;
  End;

finalization
  Server.Free;
  GlobalCriticalSection.Free;

end.
