unit ObjectManager;

interface

uses
  Windows, SysUtils, Classes, StringDictionary;


type
  TListCalendar=class(TPersistent)        
  private
    FFieldName: String;
    FCalendarType: Integer;
  published
    property FieldName: String read FFieldName write FFieldName;
    property CalendarType: Integer read FCalendarType write FCalendarType;
  end; 
  
  TParam = (pObjectID, pGrID, pUserID, pNone);

  TQueryItem=class(TPersistent)      
  private
    FSQL:TStrings;
    FParam: TParam;
    procedure SetSQL(Value:TStrings);
    procedure SetParam(Value: TParam);
  public
    constructor Create();
    destructor Destroy;override;
  published
    property SQL: TStrings read FSQL write SetSQL;
    property Param: TParam read FParam write SetParam;
  end;

  TOptionItem=class(TCollectionItem)
  private
    FName:String;
    //FSQL:TStrings;
    //FParam: TParam;
    //procedure SetSQL(Value:TStrings);
    //procedure SetParam(Value: TParam);
  public
    FQueryItem: TQueryItem;
    constructor Create(Collection:TCollection); override;
    destructor Destroy;override;
  published
    property Name: String read FName write FName;
    property QueryItem: TQueryItem read FQueryItem write FQueryItem;
    //property SQL: TStrings read FSQL write SetSQL;
    //property Param: TParam read FParam write SetParam;
  end;

  TOptionLists=class(TOwnedCollection)
  private
    function GetOptionList(Index:Integer):TOptionItem;
    procedure SetOptionList(Index:Integer;Value:TOptionItem);
  protected
    function GetAttr(Index:Integer):string;override;
    function GetAttrCount:Integer;override;
    function GetItemAttr(Index,ItemIndex:Integer):string;override;
  public
    { Constructor/Destructor }
    constructor Create(AOwner: TPersistent; ItemClass:TCollectionItemClass);
    property Items[Index:Integer]:TOptionItem read GetOptionList write SetOptionList;default;
  end;

  TPrivilegeObject = class(TCollectionItem)
  private
    FObjectName: String;
    FObjectAlias: String;
    FIgnoreGrID: Boolean;
    FSecurityOn: Boolean;
    FDBTableName: String;
    FDetailSQL: TQueryItem;
    FDetailUseFrame: Boolean;
    FOptionLists: TOptionLists;
    FOptionListsSearch: TOptionLists;
    FSecurityName: String;
    FListUsePages: Boolean;
    FListUseFrame: Boolean;
    FListCalendar: TListCalendar;
    FListOrder: String;
  protected
  public
    constructor Create(Collection:TCollection); override;
    destructor Destroy;override;
  published
    property Name: String read FObjectName write FObjectName;
    property Alias: String read FObjectAlias write FObjectAlias;
    property DBTableName: String read FDBTableName write FDBTableName;
    property DetailSQL: TQueryItem read FDetailSQL  write FDetailSQL;
    property DetailUseFrame: Boolean read FDetailUseFrame write FDetailUseFrame default true;
    property SecurityOn: Boolean read FSecurityOn write FSecurityOn  default False;
    property SecurityName: String read FSecurityName write FSecurityName;
    property OptionLists:TOptionLists read FOptionLists write FOptionLists;
    property OptionListsSearch:TOptionLists read FOptionListsSearch write FOptionListsSearch;
    property IgnoreGrID: Boolean read FIgnoreGrID write FIgnoreGrID default false;
    property ListUsePages: Boolean read FListUsePages write FListUsePages;
    property ListUseFrame: Boolean read FListUseFrame write FListUseFrame default true;
    property ListCalendar: TListCalendar read FListCalendar write FListCalendar;
    property ListOrder: String read FListOrder write FListOrder;
  end;

  TPrivilegeObjects=class(TOwnedCollection)
  private
    FDictionary:TStringDictionary;
    FDictionaryBuilt:Boolean;
    procedure BuildDictionary;
    function GetPrivilegeObject(Index:Integer):TPrivilegeObject;
    procedure SetPrivilegeObject(Index:Integer;Value:TPrivilegeObject);
  protected
    function GetAttr(Index:Integer):string;override;
    function GetAttrCount:Integer;override;
    function GetItemAttr(Index,ItemIndex:Integer):string;override;
  public
    { Constructor/Destructor }
    constructor Create(AOwner: TPersistent; ItemClass:TCollectionItemClass);
    destructor Destroy; override;
//    function  GetPrivilegeObjectByName(Name: String):TPrivilegeObject;
    function Add:TPrivilegeObject;
    function IndexOf(const Name:string):Integer;
    property Items[Index:Integer]:TPrivilegeObject read GetPrivilegeObject write SetPrivilegeObject;default;
  end;

  TObjectManager = class(TComponent)
  private
    { Private declarations }
    FPrivilegeObjects: TPrivilegeObjects;
    FListMaxRows: Integer;
    FMaxWait: Integer;
    FCompressMinSize: Integer;                
  protected
    { Protected declarations }
    function GetPrivilegeObject(Index: Integer): TPrivilegeObject;
    procedure SetPrivilegeObjects(const Value: TPrivilegeObjects);
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    property PrivilegeObject[Index:Integer]:TPrivilegeObject read GetPrivilegeObject;
  published
    { Published declarations }
    property PrivilegeObjects: TPrivilegeObjects read FPrivilegeObjects write SetPrivilegeObjects;
    property CompressMinSize: Integer read FCompressMinSize write FCompressMinSize default 4096;
    property MaxWait: Integer read FMaxWait write FMaxWait default 3000;
    property ListMaxRows: Integer read FListMaxRows write FListMaxRows default 50;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('OSI', [TObjectManager]);
end;

{ TTTSPrivilegeObjects }

function TPrivilegeObjects.Add: TPrivilegeObject;
begin
  Result:=TPrivilegeObject(inherited Add);
end;

procedure TPrivilegeObjects.BuildDictionary;
var
  i:Integer;
begin 
  { free the old dictionary and initialize the new one }
  FDictionary.Free;
  FDictionary:=TStringDictionary.Create(nil);
  FDictionary.NumBuckets:=32;
  { No objects will be stored in the dictionary, only integer indexes into the Items
    array of the collection.  Casting to TObject here and back to Integer in the
    IndexOf function is necessary because the dictionary expects a TObject.  Must
    add one to the value to be stored, because Delphi interprets nil pointers cast
    as Integers as 0, so if we stored an actual 0 we would get hits on that in the
    dictionary, and on every Name that didn't exist in the dictionary, as well. }
  for i:=0 to Count-1 do                         //todo
    FDictionary.Add(Items[i].Name,TObject(i+1));
  FDictionaryBuilt:=True;
end;

constructor TPrivilegeObjects.Create(AOwner: TPersistent;
  ItemClass: TCollectionItemClass);
begin
  inherited Create(AOwner, ItemClass);
  FDictionaryBuilt:=False;
end;

destructor TPrivilegeObjects.Destroy;
begin
  inherited;
end;

{ TPrivilegeObject }
constructor TPrivilegeObject.Create(Collection: TCollection);
begin
  inherited;
  FDetailSQL:= TQueryItem.Create();
  FDetailUseFrame:=True;
  FObjectName:=EmptyStr;
  FObjectAlias:=EmptyStr;
  FIgnoreGrID:=False;
  FDBTableName:=EmptyStr;
  FSecurityName:=EmptyStr;
  FListUsePages:=True;
  FListUseFrame:=True;
  FListCalendar:= TListCalendar.Create;
  FListOrder:= EmptyStr;
  FOptionLists:= TOptionLists.Create(Collection, TOptionItem);
  FOptionListsSearch:= TOptionLists.Create(Collection, TOptionItem);
end;

destructor TPrivilegeObject.Destroy;
begin
  inherited;
  FListCalendar.Destroy;
  FOptionLists.Destroy;
  FOptionListsSearch.Destroy;
end;

{ TObjectManager }
constructor TObjectManager.Create(AOwner: TComponent);
begin
  inherited;
  FPrivilegeObjects:=TPrivilegeObjects.Create(Self, TPrivilegeObject);
end;

destructor TObjectManager.Destroy;
begin
  inherited;

end;

function TObjectManager.GetPrivilegeObject(
  Index: Integer): TPrivilegeObject;
begin
  Result:=FPrivilegeObjects[Index];
end;


procedure TObjectManager.SetPrivilegeObjects(const Value: TPrivilegeObjects);
begin
   FPrivilegeObjects.Assign(Value);
end; 

function TPrivilegeObjects.GetAttr(Index: Integer): string;
begin
  case (Index) of
    0:Result:='Name';
    1:Result:='Security';
    2:Result:='Security Name';
  else
    Result:='';
  end;
end;

function TPrivilegeObjects.GetAttrCount: Integer;
begin
  { Returns the number of 'custom' attributes associated with collection items.}
  Result:=3;
end;     

function TPrivilegeObjects.GetItemAttr(Index, ItemIndex: Integer): string;
begin
  case (Index) of
    0:Result:=Items[ItemIndex].Name;
    1:if (Items[ItemIndex].SecurityOn) then
        Result:='True'
      else
        Result:='False';
   2:Result:=Items[ItemIndex].SecurityName;
  else
    Result:='';
  end;
end;

function TPrivilegeObjects.GetPrivilegeObject(
  Index: Integer): TPrivilegeObject;
begin
  Result:=TPrivilegeObject(inherited Items[Index]);
end;

function TPrivilegeObjects.IndexOf(const Name: string): Integer;
begin
  if not(FDictionaryBuilt) then BuildDictionary;
  Result:=Integer(FDictionary.Values[Name])-1;
end;

procedure TPrivilegeObjects.SetPrivilegeObject(Index: Integer;
  Value: TPrivilegeObject);
begin
  Items[Index].Assign(Value);
end;

{ TOptionLists }

constructor TOptionLists.Create(AOwner: TPersistent;
  ItemClass: TCollectionItemClass);
begin
  inherited Create(AOwner, ItemClass);
end;

function TOptionLists.GetOptionList(Index: Integer): TOptionItem;
begin
  Result:=TOptionItem(inherited Items[Index]);
end;

procedure TOptionLists.SetOptionList(Index: Integer; Value: TOptionItem);
begin
  Items[Index].Assign(Value);
end;


function TOptionLists.GetAttr(Index: Integer): string;
begin
  case (Index) of
    0:Result:='Name';
    1:Result:='Parameter';
  else
    Result:='';
  end;
end;

function TOptionLists.GetAttrCount: Integer;
begin
  Result:=2;
end;

function TOptionLists.GetItemAttr(Index, ItemIndex: Integer): string;
begin
  case (Index) of
    0:Result:=Items[ItemIndex].Name;
    1:case (Items[ItemIndex].FQueryItem.Param) of
       pUserID:  Result:='UserID';
       pObjectID:  Result:='ObjectID';
       pGrID:  Result:='GrID';
      end;
  else
    Result:='';
  end;
end; 

{ TOptionItem }

constructor TOptionItem.Create;
begin
  inherited;
  FQueryItem:=TQueryItem.Create;
  //FSQL:=TStringList.Create;
end;

destructor TOptionItem.Destroy;
begin
  //FSQL.Free;
  inherited;
end;

procedure TQueryItem.SetParam(Value: TParam);
begin
  if Value <> FParam then
  begin
    FParam := Value;
    //Changed(False);
  end;
end;


procedure TQueryItem.SetSQL(Value: TStrings);
begin
  FSQL.Assign(Value);
end;

{ TQueryItem }

constructor TQueryItem.Create;
begin
   FSQL:=TStringList.Create;
end;

destructor TQueryItem.Destroy;
begin
  FSQL.Free;
  inherited;
end;



end.
