unit WebModule;

interface

uses
  Windows, SysUtils, Classes, HTTPApp, Math, LogWriter_osi, ObjectManager, DzURL,
  IdHTTP, IdIntercept, IdSSLIntercept, IdSSLOpenSSL, IdTCPClient, IdBaseComponent, IdComponent, IdTCPConnection;
                  
type
                          
  TWM = class(TWebModule)
    IdHTTPS: TIdHTTP;
    IdConnectionInterceptOpenSSL: TIdConnectionInterceptOpenSSL;
    MainPageProducer: TPageProducer;
    ObjectManager: TObjectManager;
    procedure WebModuleBeforeDispatch(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModuleCreate(Sender: TObject);
    procedure WMWebActionItemDefaultAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModuleAfterDispatch(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure WebModuleDestroy(Sender: TObject);
    procedure WMWebActionItemLoginAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemLogoutAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemListAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemDetailAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemUpdateAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemInsertAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemDeleteAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemWorkedHourAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemPostWorkedHourAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemSendAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemApproveAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemResetPasswordAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemJabberAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);   
    procedure WMWebActionItemMonthWorkedHourAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemFileAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WMWebActionItemMailLoginAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure MainPageProducerHTMLTag(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings;
      var ReplaceText: String);
    procedure WMWebActionItemOMAtestAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
   private
    { Private declarations }

    BrID: String;
    COMResult:HResult;
    CorePath:String;
    ErrorEmail:String;
    GrID: String;
    GroupPeriod:Integer;
    GroupRoleNavigation:WideString;
    GroupName:String;
    GroupHourFormat:String;
    GroupStartDate:TDateTime;
    RoID: String;
    PrID: String;
    PO: TPrivilegeObject;
    MainDocsPath:String;
    OutHTML:WideString;
    OutHTMLCalendar:WideString;
    OutValue:WideString;
    LogWriter: TLogWriter;
    SessionID: WideString;
    SessionData:TStrings;
    SessionNewMsgs: Integer;
    SessionCheckedOutTodo:String;
    UploadPath: String;
    UsID: String;
    UserName:String;
    UserLogin:String;
    UserDate:TDateTime;
    ThreadID: String;
    ThreadRequestNum: LongWord;
    TravelProfileServer: String;
    TravelProfileURL: String;
    ExtractFieldsXSLT:String;
    GenerateMenuXSLT: String;
    GenerateCalendarXSLT: String;
    CoreSearchSQLXSLT:String;
    CoreUpdateSQLXSLT:String;
    CoreDeleteSQLXSLT:String;
    CoreInsertSQLXSLT:String;
    CoreMultipleSQLXSLT:String;
    OutHelpersDetailXSLT: String;
    OutHelpersListXSLT: String;

    function Calendars(const Path: String; var StartDate: TDateTime; var Period: Integer):String;
    function CompressResponse: Boolean;
    procedure CheckCOMError(COMResult:HResult; var ErrorMsg:WideString);
    function GetUserIDsCommaList(SQLWhere: String): String;
    function GetRequestValue(const FieldName:String):String;
    procedure GetLastPeriod(out StartDate, EndDate: TDateTime);
    procedure GetPayrollPeriod(out StartDate, EndDate: TDateTime);
    procedure GetPageNumber(out PageNumber: Integer);
    function DoLogin(out OutValue:WideString):Boolean;
    //procedure DoLogout;
    procedure DoList(var XML:String; var PageNumber: Integer; const UsePages:Boolean);
    procedure DoInsert(XML, InsertGrID: String; out ObjID, OutValue:WideString);
    procedure DoUpdate(XML, UpdateGrID: String; out RowCount:Integer; out OutValue:WideString);
//    function  DoDetail(const ObjectID: String; const ObjectName:String; out OutValue:WideString):WideString;
    procedure DoDelete(XML, DeleteGrID:String; out RowCount:Integer; out OutValue:WideString);
    procedure DoMultipleUpdate(XML:String;const UsedGrID: String; out OutValue:WideString);
    function ErrorMsg(const Location:String;const ErrorText:String):WideString;
    function  ExtractObjectName(S:String):String;
    procedure JabberServerConnect(const JabberLogin, JabberPassword: String; out OutValue: WideString);
    function LoadFile(FileName:String):WideString;
    procedure MarkAsReadMsg(const DetailXML:String);
    function MonthCalendar(iMonth, iDay, iYear:Integer; dtFocusStartDate, dtFocusEndDate:TDateTime):WideString;
    function Msg(const Text, Ico:String):WideString;
    procedure PostTravelProfile(TravelProfileID: String);
    procedure SendMessage(FromUser, ToUsers, Subject, Content:String; Urgent: Boolean; out OutValue:WideString);
    procedure SendEmail(FromUserEmail, ToUsers, CCToUsers, Subject, Content: String; const IsHTML: Boolean;  out OutValue: WideString);
    procedure SendTemplateEmail(const MailTemplateName, ToUsers: String;  out OutValue:WideString);
    procedure UpdateChatLog(const MessageText: String);
    function GenerateBaseXML(const TableName: String; const XMLValues: String; const IgnoreGrID: Boolean) : String;
    function XMLEscape(XML:String): String;
    procedure UpdateSessionData(VarName,NewValue: String); 
  public
    { Public declarations }
  published
    { Published declarations }
  protected
    { Protected declarations }
  end;

ESendEMail = class(Exception);
EDuplicateLogin = class(Exception);
ENoDataFound = class(Exception);
EDuplicateEntryFound = class(Exception);
EChildRecordFound = class(Exception);
ENoRowsUpdated = class(Exception);
ENoJabberSessionFound = class(Exception);
EJabberCommunicationError = class(Exception);
EJabberAuthenticationError = class(Exception);

const
  APP_NAME = 'TTS';
  COMPRESS_MIN_SIZE = 4096;
  CONFIG_DIR_ENV_VAR = 'ATTTINIDIR';
  CONFIG_FILE_NAME = 'attttimetrackingsystem.ini';
  DOCEXT_HTM='.htm';
  DOCEXT_SQL='.sql';
  DOCEXT_XSL='.xsl';
  DOCEXT_XML='.xml';
  DOCEXT_TXT='.txt';
  EMAIL_AGENT_ID=1;
  LIST_PAGE_SIZE=50; 
  MAIL_AGENT = 'Time Tracking System Mailer--Powered by @ttitude';
  MAX_WAIT = 2000;
  MONTH_LEN: array[1..12] of Integer = (31,28,31,30,31,30,31,31,30,31,30,31);
  OBJ_ACCOUNT = 'account';
  OBJ_LOGIN_USER = 'login_user';
  OBJ_MAIL_TMPL = 'mail_template';
  OBJ_MESSAGE = 'message';
  OBJ_EXP_STAT = 'exp_statement';
  OBJ_TRAVEL_PROFILE = 'travel_profile';
  OBJ_TODOLOG = 'to_do_log';
  OBJ_W4INFO = 'w4info';
  OBJ_WORKED_HOUR = 'worked_hour';
  ROLES: array[0..6] of String = ('', 'Employee','Supervisor','Manager','Sub-supervisor', 'Observer', 'HR');
  ROLE_ID_USER = 1;
  ROLE_ID_SPVR = 2;
  ROLE_ID_MAN = 3;
  ROLE_ID_SUBSPVR = 4;
  ROLE_ID_OBS = 5;
  ROLE_ID_HR = 6;
  NON_SECURITY_ACTIONS: array[0..4] of String = ('home', 'login', 'logout', 'mail', 'resetpassword');

var
  WM: TWM;


implementation

{$R *.DFM}

uses
  ComObj, AtttResortsServer_TLB, MailServer_TLB, WebSesServer_TLB,
  XSLTServer_TLB, zServer_TLB, JabberChatServer_TLB,
  MsMultipartParser;
var
  { COM interface pointers }
  ARS:IAtttitudeResortsServer;
  EMS:IEmailServer;
  WSS:IWebSessionsServer;
  XSLS:IXSLTransformServer;
  ZS:IzlibServer;
  JS:IJabberServer;


{==============================================================================}
{                              WebModule Event Handlers                        }
{==============================================================================}

procedure TWM.WebModuleCreate(Sender: TObject);
var
  B: array[0..255] of Char;
  ConfigSettings: TStringList;
  ConfigDir: string;
  ResortsServerMachine: String;
  EMailServerMachine: String;
  WebSesServerMachine: String;
  XSLTServerMachine: String;
  zServerMachine: String;
  JabberServerMachine: String;
begin
  ConfigSettings := TStringList.Create;
  try
    if (GetEnvironmentVariable(CONFIG_DIR_ENV_VAR, B, 256)=0) then
       raise Exception.Create('Unable to resolve initialization settings location');
     ConfigDir := AddSlash(B);
     if not(FileExists(ConfigDir + CONFIG_FILE_NAME)) then
       raise Exception.Create('Unable to load configuration settings');
     ConfigSettings.LoadFromFile(ConfigDir + CONFIG_FILE_NAME);

     MainDocsPath:=AddSlash(ConfigSettings.Values['mainpath']);
     UploadPath:= AddSlash(ConfigSettings.Values['uploadpath']);
     CorePath:=AddSlash(ConfigSettings.Values['corepath']);
     ErrorEmail:=ConfigSettings.Values['erroremail'];
     TravelProfileServer:= ConfigSettings.Values['travel_profileserver'];
     TravelPRofileURL:=ConfigSettings.Values['travel_profileurl'];
     LogWriter := TLogWriter.Create(ConfigSettings.Values['log_file_dir'], ConfigSettings.Values['log_file_prefix']);
     ResortsServerMachine := ConfigSettings.Values['ResortsServerMachine'];
     EMailServerMachine := ConfigSettings.Values['EMailServerMachine'];
     WebSesServerMachine := ConfigSettings.Values['WebSesServerMachine'];
     XSLTServerMachine := ConfigSettings.Values['XSLTServerMachine'];
     zServerMachine := ConfigSettings.Values['zServerMachine'];
     JabberServerMachine := ConfigSettings.Values['JabberServerMachine'];
  finally
     ConfigSettings.Free;
  end;
  MainPageProducer.HTMLFile:= MainDocsPath+'main'+DOCEXT_HTM;

  ThreadID:=IntToStr(GetCurrentThreadID);
  ThreadRequestNum := 0;

  with TStringList.Create do
  begin
    try
      LoadFromFile(MainDocsPath+'core_search_sql'+DOCEXT_XSL);
      CoreSearchSQLXSLT :=Text;
      LoadFromFile(MainDocsPath+'core_update_sql'+DOCEXT_XSL);
      CoreUpdateSQLXSLT :=Text;
      LoadFromFile(CorePath+'core_insert_sql'+DOCEXT_XSL);
      CoreInsertSQLXSLT :=Text;
      LoadFromFile(CorePath+'core_delete_sql'+DOCEXT_XSL);
      CoreDeleteSQLXSLT :=Text;
      LoadFromFile(MainDocsPath+'multipleupdate_sql'+DOCEXT_XSL);
      CoreMultipleSQLXSLT :=Text;
      LoadFromFile(MainDocsPath+'extractfields'+DOCEXT_XSL);
      ExtractFieldsXSLT :=Text;
      LoadFromFile(MainDocsPath+'generateMenu'+DOCEXT_XSL);
      GenerateMenuXSLT:=Text;
      LoadFromFile(MainDocsPath+'generateCalendar'+DOCEXT_XSL);
      GenerateCalendarXSLT :=Text;
      LoadFromFile(MainDocsPath+'out_helpers_detail'+DOCEXT_XSL);
      OutHelpersDetailXSLT:= Text;
      LoadFromFile(MainDocsPath+'out_helpers_list'+DOCEXT_XSL);
      OutHelpersListXSLT := Text;
    finally
      Free;
    end;
  end;
  SessionData:=TStringList.Create;
  ThreadRequestNum := 0;

  if (ResortsServerMachine='') then
    raise Exception.Create('Unable to get ResortsServer machine.');
  if (EMailServerMachine='') then
    raise Exception.Create('Unable to get EMailServer machine.');
  if (WebSesServerMachine='') then
    raise Exception.Create('Unable to get WebSesServer machine.');
  if (XSLTServerMachine='') then
    raise Exception.Create('Unable to get XSLTServer machine.');
  if (zServerMachine='') then
    raise Exception.Create('Unable to get zServer machine.');
  if (JabberServerMachine='') then
    raise Exception.Create('Unable to get JabberServer machine.');
    if not(Assigned(ARS)) then
      ARS:=CoAtttitudeResortsServer.CreateRemote(ResortsServerMachine);
    if not(Assigned(EMS)) then
      EMS:=CoEmailServer.CreateRemote(EMailServerMachine);
    if not(Assigned(WSS)) then
      WSS:=CoWebSessionsServer.CreateRemote(WebSesServerMachine);
    if not(Assigned(XSLS)) then
      XSLS:=CoXSLTransformServer.CreateRemote(XSLTServerMachine);
    if not(Assigned(ZS)) then
      ZS:=CozlibServer.CreateRemote(zServerMachine);
    if not(Assigned(JS)) then
      JS:=CoJabberServer.CreateRemote(JabberServerMachine);
end;

procedure TWM.WebModuleBeforeDispatch(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  function IsValidSession(): Boolean;
  var
    COMResult: HResult;
    CustomData,CustomNavigation,CustomDataFilters,OutValue:WideString;
    tmpDate:String;
    begin { IsValidSession }
      Result:=False;
      { Validate the web session in the COM server }
      COMResult:=WSS.WebSession(SessionID,CustomData,CustomNavigation,CustomDataFilters,OutValue);

      CheckCOMError(COMResult,OutValue);
      if SessionID<>'' then
      begin
        SessionData.Text:=CustomData;
        GrID := SessionData.Values['grid'];
        BrID := SessionData.Values['brid'];
        UsID := SessionData.Values['login_user_id'];
        RoID := SessionData.Values['role_id'];
        PrID := SessionData.Values['privilege_role_id'];
        GroupName := SessionData.Values['group_name'];
         if SessionData.Values['email_error']<>EmptyStr then
          ErrorEmail:=SessionData.Values['email_error'];

        SessionNewMsgs := StrToInt(SessionData.Values['newmsg']);
        SessionCheckedOutTodo := SessionData.Values['checked_out_to_do'];
        UserLogin:= SessionData.Values['user_name'];
        if SessionData.Values['first_name'] = EmptyStr then
          UserName := SessionData.Values['last_name']
        else if SessionData.Values['last_name'] = EmptyStr then
          UserName := SessionData.Values['first_name']
        else
          UserName := SessionData.Values['first_name']+' '+SessionData.Values['last_name'];
        UserDate:= Math.Floor(Now-(StrToInt(SessionData.Values['useroffset'])/24));
        GroupPeriod:=StrToInt(SessionData.Values['period_in_week'])*7;
        GroupHourFormat:=SessionData.Values['hour_format'];
        GroupRoleNavigation:=CustomNavigation;
        tmpDate:= SessionData.Values['start_date'];
        GroupStartDate:=EncodeDate(StrToInt(Copy(tmpDate,1,4)),StrToInt(Copy(tmpDate,5,2)),StrToInt(Copy(tmpDate,7,2)));
        Result:=True;
      end;
    end; { IsValidSession}

    function IsValidRights(): Boolean;
    var sActionObject, XSLT: String;
    begin   {IsValidRights}
      Result:=False;
      try
      if (Request.PathInfo='/home')  or (Request.PathInfo='/omatest')   or (Request.PathInfo='/login')  or (Request.PathInfo='/logout') or (Request.PathInfo='/mail') or (Request.PathInfo='/resetpassword') then
        Result:=True
      else if GroupRoleNavigation=EmptySTr then
        Result:=False
      else
      begin
        if(Request.PathInfo<>'/detail') and (Request.PathInfo<>'/search') and (Request.PathInfo<>'/list') and (Request.PathInfo<>'/update') and (Request.PathInfo<>'/insert')  and (Request.PathInfo<>'/delete') and (Request.PathInfo<>'/post') and (Request.PathInfo<>'/approve') and (Request.PathInfo<>'/send') then
          sActionObject:= Copy(Request.PathInfo,2, Length(Request.PathInfo)-1)
        else
        begin
          sActionObject:=GetRequestValue('o');
        end;
        PO:= ObjectManager.PrivilegeObjects[ObjectManager.PrivilegeObjects.IndexOf(sActionObject)];
        if PO.IgnoreGrID then GrID:='-1';
        //if Pos('@',sActionObject)<>0 then sActionObject:=Copy(sActionObject, 1, Pos('@', sActionObject)-1);
        {todo XSLT:='<?xml version="1.0" encoding="UTF-8"?><xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">'+
          '<xsl:output method="xml" omit-xml-declaration="yes"/><xsl:template match="/*">'+
          '<xsl:value-of select="((/d/g[@id='+RoID+']/i/i[@o='''+sObject+''']/@o) or (/d/g[@id='+RoID+']/i[@o='''+sObject+''']/@o)  or (/d/g[@id='+RoID+']/i/i/i[@o='''+sObject+''']/@o) )"/></xsl:template></xsl:stylesheet>';
        COMResult:=XSLS.Transform(MAX_WAIT,GroupNavigation,XSLT,OutValue);
        CheckCOMError(COMResult,OutValue);
        Result:=(OutValue='true'); }
        XSLT:='<?xml version="1.0" encoding="UTF-8"?><xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">'+
          '<xsl:output method="xml" omit-xml-declaration="yes"/><xsl:template match="/d"><xsl:value-of select="./descendant::*/@o=''' + PO.SecurityName + '''"/></xsl:template></xsl:stylesheet>';  // or (/d/i/i[@o='''+sObject+''']/@o)  or (/d/i/i/i[@o='''+sObject+''']/@o)
        COMResult:=XSLS.Transform(MAX_WAIT, GroupRoleNavigation, XSLT, OutValue);
        CheckCOMError(COMResult,OutValue);
        Result:=(OutValue='true');
      end;
      except
        on E:Exception do
          Response.Content := E.Message;
      end;
    end;   {IsValidRights}
  begin

  Inc(ThreadRequestNum, 1); //ThreadRequestNum is used to to uniquely identify the request for things like logging, etc.
  if not(Request.MethodType in [mtGet, mtPost]) Then
    Exit;
  SessionID:=Request.QueryFields.Values['s'];
  SessionData.Clear;
  GrID:='-1';
  RoID:='-1';
  PrID:='-1';
  BrID:='-1';
  UsID:='-1';
  PO:= nil;
  UserName:=EmptyStr;
  UserLogin:=EmptyStr;
  UserDate:= 0;
  GroupName:=EmptyStr;
  GroupPeriod:=0;
  GroupHourFormat:=EmptyStr;
  OutHTML:=EmptyStr;
  OutHTMLCalendar:=EmptyStr;
  ErrorEmail:=EmptyStr;
  GroupRoleNavigation:=EmptyStr;
  SessionNewMsgs:=0;
  SessionCheckedOutTodo:=EmptyStr;
  Response.ContentType:='text/html';
 { Validate both SessionID and SessionData }
  try
    if not (IsValidSession) and not ((Request.PathInfo='/home') or (Request.PathInfo='/login') or (Request.PathInfo='/logout') or (Request.PathInfo='/resetpassword')) then
    begin
      OutHTML:='<div id="e">Login is required to access this page. Use the Login Section  to continue. <br/><br/><form method="post" name="frmlogin" onsubmit="return validlogin(this);" action="login"><table width="250" cellpadding="0" cellspacing="0" border="0">'+
									 '   <tr><td valign="top" align="left" width="100">group name:</td><td align=right><input name="groupname" type="text" value=""/></td></tr>'+
									 '   <tr><td valign="top" align="left" width="150">user name:</td><td align=right><input name="username" type="text" value=""/></td></tr>'+
									 '   <tr><td valign="top" align="left">password:</td><td align=right><input type="password" name="password" value=""/><input type="hidden" name="useroffset"/></td></tr>'+
									 '   <tr><td><input name="checkboxsave" id="checkboxsave" type="checkbox" class="check" value="y"/><label for="checkboxsave">Save</label></td><td align="right"><button type="submit">Login</button></td></tr>'+
							     '  </table></form></div>';
      Handled:=True;
   end
   else if not(IsValidRights) then  //Validate privileges
    begin
      OutHTML:=Msg('Sorry, you do not have permission to view this page!', 'stop');
      Handled:=True;
    end;
  except
    on E:Exception do
      OutHTML := ErrorMsg('beforedispatch',  E.Message);
  end;
end;

procedure TWM.WebModuleAfterDispatch(Sender: TObject; Request: TWebRequest;
  Response: TWebResponse; var Handled: Boolean);
var
  SaveContent, HTML: WideString;
  UseFrame: Boolean;
begin { AfterDispatch }
  try
    UseFrame:=True;
    if (Response.ContentType<>'text/html') or (Request.QueryFields.Values['p']<>EmptyStr) then
      UseFrame:=False
    else if PO <> nil then
      if  (Request.PathInfo='/list') then
        UseFrame:=PO.ListUseFrame
      else if Request.PathInfo='/detail' then
        useFrame:=PO.DetailUseFrame;
    if UseFrame then //generate html with html frame
      HTML:=MainPageProducer.Content
    else
    begin
      HTML:=OutHTML;
      HTML:=StringReplace(HTML,'<#login_user_id>',UsID,[rfReplaceAll]);
      HTML:=StringReplace(HTML,'<#tts_group_name>',GroupName,[rfReplaceAll]);
      HTML:=StringReplace(HTML,'<#tts_jabber_login>',XMLEscape(UserLogin + '_' + GroupName),[rfReplaceAll]);
      HTML:=StringReplace(HTML,'<#s>',SessionID,[rfReplaceAll]);
    end;
    SaveContent:=HTML;
    Response.Content:= SaveContent;
    try
        CompressResponse;
    except
        on E:Exception do
        if (SaveContent<>'') Then // (Response.Content='')
        begin
          Response.Content:=SaveContent;
          Response.ContentEncoding:='';
        end
        else
        begin
          Response.Content:=ErrorMsg('compressresponse', E.Message);
          Response.ContentEncoding:='';
        end;
    end;
  except
    on E:Exception do
       Response.Content := ErrorMsg('afterdispatch',  E.Message);
  end;
  Handled:=True;
end; { AfterDispatch }

procedure TWM.MainPageProducerHTMLTag(Sender: TObject; Tag: TTag;
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var LoginAction:String;
begin
  if TagString='s' then  ReplaceText:= SessionID
  else if TagString='login_user_id' then  ReplaceText:= UsID
  else if (TagString='tts_user_name') and (SessionID<>EmptyStr) then  ReplaceText:= UserName + ', ' + ROLES[StrToInt(RoID)]
  else if TagString='tts_jabber_login' then ReplaceText:= XMLEscape(UserLogin + '_' + GroupName)
  else if (TagString='tts_newmsg') and (SessionNewMsgs>=1) then
      ReplaceText:= '<a href="list?o=message&sti=8&s='+SessionID+'"><img src="/tts/i/i/ico_blink.gif" border="0" style="vertical-align: baseline;"/>&nbsp;you have <font color="#FB583A">' + IntToStr(SessionNewMsgs) + '</font> new message(s)</a>'
  else if (TagString='tts_checked_out_todo') and (SessionCheckedOutTodo <> EmptyStr) then
      ReplaceText:= '<a href="detail?o=to_do&amp;oid=' + SessionCheckedOutTodo + '&sti=9&s='+SessionID+'"><img src="/tts/i/i/ico_blink.gif" border="0" style="vertical-align: baseline;"/>&nbsp;to-do #<font color="#FB583A">' + SessionCheckedOutTodo + '</font> is checked out </a>'
  else if TagString='menu' then
  begin
    //COMResult:=XSLS.Transform(MAX_WAIT,'<?xml version="1.0"?><menu sg="'+RoID+'" s="'+SessionID+'" sti="'+ Request.QueryFields.Values['sti'] +'" nm="' +IntToStr(SessionNewMsgs) + '">'+GroupNavigation+'</menu>',GenerateMenuXSLT,OutValue);
    COMResult:=XSLS.Transform(MAX_WAIT,'<?xml version="1.0"?><m s="'+SessionID+'" sti="'+ Request.QueryFields.Values['sti'] +'" nm="' +IntToStr(SessionNewMsgs) + '">'+GroupRoleNavigation+'</m>',GenerateMenuXSLT,OutValue);
    CheckCOMError(COMResult,OutValue);
    ReplaceText:= OutValue;
  end
  else if TagString='content' then
  begin
    if SessionID<>EmptyStr then
    begin
      OutHTML:=StringReplace(OutHTML,'<#s>', SessionID, [rfReplaceAll]);
      OutHTML:=StringReplace(OutHTML,'<#loginarea>' , '<span class="red">'+UserName+'</span><li/><a href="detail?o=account&sti=2&s='+SessionID+'">Account management</a>'+
			       	'<li/><a href="logout?s='+SessionID+'&cll='+Copy(Request.PathInfo,2,Length(Request.PathInfo)-1)+'">Logout</a>',[])
    end
    else
    begin
      if Request.PathInfo='/home' then LoginAction:='login'
      else LoginAction:='login?c=n';
      OutHTML:=StringReplace(OutHTML,'<#loginarea>' ,'<form method="post" name="frmlogin" onsubmit="return validlogin(this);" action="'+LoginAction+'"><table width="250" cellpadding="0" cellspacing="0" border="0">'+
									 '   <tr><td valign="top" align="left" width="100">group name:</td><td align=right><input name="groupname" type="text" value=""/></td></tr>'+
									 '   <tr><td valign="top" align="left" width="150">user name:</td><td align=right><input name="username" type="text" value=""/></td></tr>'+
									 '   <tr><td valign="top" align="left">password:</td><td align=right><input type="password" name="password" value=""/><input type="hidden" name="useroffset"/></td></tr>'+
									 '   <tr><td><input name="checkboxsave" id="checkboxsave" type="checkbox" class="check" value="y"/><label for="checkboxsave">Save</label></td><td align="right"><button type="submit">Login</button></td></tr>'+
							     '  </table></form> ',[]);
    end;
    ReplaceText:= OutHTML
  end
  else if TagString='calendar' then ReplaceText:= OutHTMLCalendar;
end;

procedure TWM.WebModuleDestroy(Sender: TObject);
begin { Destroy }
  SessionData.Free;
end; { Destroy }



{*******************************************}
{               ACTIONS                     }
{*******************************************}

procedure TWM.WMWebActionItemDefaultAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  Path: string;
begin { DeaultAction }
  Path:=Copy(Request.PathInfo,2,Length(Request.PathInfo))+DOCEXT_HTM;
  try
    if not FileExists(MainDocsPath+Path) then
      OutHTML:=Msg('The page you requested was not found.','question')
    else
      OutHTML:=LoadFile(Path);
  except
    on E: Exception do
      OutHTML := ErrorMsg('worked_hour', E.Message);
  end;    
  Handled:=True;
end; { DeaultAction }


procedure TWM.WMWebActionItemWorkedHourAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
   XML, Query, XSLT, UserID, ExpStatID, NotSubmitted, ObjectName, QuickEntryBar:String;
   DaysLeft: Double;
   PeriodStartDate,PeriodEndDate: TDateTime;
   Period: Integer;
   IsPrintVersion: Boolean;
begin { /worked_hour }
  DaysLeft:=0;
  PeriodStartDate:=UserDate;
  ObjectName:=OBJ_WORKED_HOUR;
  IsPrintVersion:=(Request.QueryFields.Values['p']<>EmptyStr);
  if Request.QueryFields.Values['o']<>EmptyStr then ObjectName:=Request.QueryFields.Values['o'];
  try
    GetPayrollPeriod(PeriodStartDate, PeriodEndDate);
    Period:= GroupPeriod-1;

    if IsPrintVersion then XSLT:=LoadFile('listworked_hour@print'+DOCEXT_XSL)
    else
    begin
      XSLT:=LoadFile('listworked_hour'+DOCEXT_XSL);
      OutHTMLCalendar:=Calendars('worked_hour?',PeriodStartDate, Period)  //month calendar
    end;

    UserID:='-1';
    ExpStatID:='-1';
    XML:=EmptyStr;

    if (RoID=IntToStr(ROLE_ID_MAN)) or (RoID=IntToStr(ROLE_ID_USER)) or (RoID=IntToStr(ROLE_ID_SPVR)) or (RoID=IntToStr(ROLE_ID_SUBSPVR)) then
    begin
      //users list - for managers,employee-csr, csr-s, supervisor and subsupervisors view
      if (Request.QueryFields.Values['oid']<>'') and (ObjectName=OBJ_WORKED_HOUR)  then UserID:=GetRequestValue('oid');
      Query:=LoadFile('listworked_hour@'+RoID +'lu' + DOCEXT_SQL);
      Query:=StringReplace(Query,'<#TRACK_DATE_1>',FormatDateTime('mm/dd/yyyy',PeriodStartDate),[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#TRACK_DATE_2>',FormatDateTime('mm/dd/yyyy',PeriodEndDate),[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#LOGIN_USER_ID>',UsID,[rfIgnoreCase]);
      COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
      CheckCOMError(COMResult,OutValue);
      XML:=XML+'<users><d>'+OutValue+'</d></users>';
    end;

    if (RoID=IntToStr(ROLE_ID_MAN)) or (RoID=IntToStr(ROLE_ID_SPVR)) or (RoID=IntToStr(ROLE_ID_SUBSPVR)) then
    begin
      //exp statements list - for managers, supervisor and subsupervisors view
      if (Request.QueryFields.Values['oid']<>'') and (ObjectName=OBJ_EXP_STAT)  then ExpStatID:=GetRequestValue('oid');
      Query:=LoadFile('listworked_hour@'+RoID +'es' + DOCEXT_SQL);
      Query:=StringReplace(Query,'<#TRACK_DATE_1>',FormatDateTime('mm/dd/yyyy',PeriodStartDate),[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#TRACK_DATE_2>',FormatDateTime('mm/dd/yyyy',PeriodEndDate),[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#LOGIN_USER_ID>',UsID,[rfIgnoreCase]);
      COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
      CheckCOMError(COMResult,OutValue);
      XML:=XML+'<exp_stat><d>'+OutValue+'</d></exp_stat>';
    end;

    if(UserID='-1') then UserID:=UsID;

    //list of worked hours - all roles
    if UserID<>'-1' then
    begin
      if (UsID=UserID) then   //employees only
      begin
        //project list
        Query:='select p.project_id, p.project_name, c.client_name'+
             ' from tts.project p, tts.login_user_project up, tts.client c'+
             ' where p.grid=:GRID'+
             ' and p.project_id = up.project_id'+
             ' and p.client_id = c.client_id'+
             ' and up.login_user_id=:LOGIN_USER_ID'+
             ' and up.active=''Y'''+
             ' and p.active=''Y'''+
             ' and c.active=''Y'''+
             ' order by c.client_name, p.project_name' +
             #127'LOGIN_USER_ID= '+ UserID;
        COMResult:=ARS.List(MAX_WAIT, StrToInt(GrID),-1,Query,0,OutValue);
        CheckCOMError(COMResult,OutValue);
        XML:=XML+'<project>'+OutValue+'</project>';

        //days left to submit  worked hours
        Query:='select case when (sum(decode(wh.submitted,''Y'',1,0))=count(*)) or (count(*)=0) then ''Y'' else ''N'' end case  '+
			       ' from tts.worked_hour wh, tts.project p where wh.login_user_id=:LOGIN_USER_ID'+
             ' and (wh.track_date between :TRACK_DATE$1 and :TRACK_DATE$2)'+
             ' and p.project_id=wh.project_id' +
             ' and p.active=''Y'''+
             ' and wh.grid=:GRID'+
             #127'LOGIN_USER_ID= '+ UsID +
             #127'TRACK_DATE$1='+ FormatDateTime('mm/dd/yyyy',PeriodStartDate)+
             #127'TRACK_DATE$2='+ FormatDateTime('mm/dd/yyyy',PeriodEndDate);
        COMResult:=ARS.Detail(MAX_WAIT,StrToInt(GrID),-1,Query,OutValue);
        CheckCOMError(COMResult,OutValue);
        NotSubmitted:=OutValue;
        DaysLeft:=PeriodStartDate+(GroupPeriod-1)-UserDate;
        Query:=LoadFile('listworked_hour'+DOCEXT_SQL)
     end
     else Query:=LoadFile('listworked_hour@'+RoID+DOCEXT_SQL);
      
      Query:=StringReplace(Query,'<#TRACK_DATE_1>',FormatDateTime('mm/dd/yyyy',PeriodStartDate),[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#TRACK_DATE_2>',FormatDateTime('mm/dd/yyyy',PeriodEndDate),[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#USER_ID>',UserID,[rfIgnoreCase]);
      Query:=StringReplace(Query,'<#LOGIN_USER_ID>',UsID,[rfIgnoreCase]);
      COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
      CheckCOMError(COMResult,OutValue);
      XML:=XML+'<hw>'+OutValue+'</hw>';
    end;

    if ExpStatID<>'-1' then
    begin
      Query:=LoadFile('detailexp_statement_wh'+DOCEXT_SQL);
      Query:=StringReplace(Query,'<#oid>',ExpStatID,[rfIgnoreCase]);
      COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID), -1, Query, OutValue);
      CheckCOMError(COMResult,OutValue);
      XML:=XML+'<exp_stat_detail>'+OutValue+'</exp_stat_detail>';
    end;

    Query:='select quick_entry_bar from tts.login_user where login_user_id=:LOGIN_USER_ID and grid=:GRID'#127'LOGIN_USER_ID= '+ UsID;
        COMResult:=ARS.Detail(MAX_WAIT,StrToInt(GrID),-1,Query,OutValue);
        CheckCOMError(COMResult,OutValue);
    QuickEntryBar:=OutValue;

    XML:='<d><i><s>'+SessionID+'</s><qeb>' + QuickEntryBar + '</qeb><o>' + ObjectName + '</o><oid>' +ExpStatID+ '</oid><notsub>'+NotSubmitted+'<d>'+ FloatToStr(DaysLeft)+'</d></notsub>'+
        '<sti>'+Request.QueryFields.Values['sti']+'</sti><lu>'+UserID+'</lu>'+
        '<sd>'+FormatDateTime('yyyymmdd', PeriodStartDate)+'</sd><ed>' + FormatDateTime('yyyymmdd', PeriodEndDate) + '</ed>'+
        '<pd>'+IntToStr(GroupPeriod)+'</pd>'+                         
        '<ro>'+RoID+'</ro><ui>'+UsID+'</ui><ch>'+Request.QueryFields.Values['ch']+'</ch>'+
        '<hf>'+GroupHourFormat+'</hf></i>'+XML+'</d>';
    COMResult:=XSLS.Transform(MAX_WAIT,XML,XSLT,OutValue);
    CheckCOMError(COMResult,OutValue);
    OutHTML:=OutValue;
  except
    on E: Exception do
      OutHTML := ErrorMsg('worked_hour', E.Message);
  end;
  Handled:=True;
end;{ /worked_hour }


procedure TWM.WMWebActionItemMonthWorkedHourAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
   XML, Query, XSLT, ProjectID:String;
   MonthStartDate, MonthEndDate: TDateTime;
   Period: Integer;
begin { /month_worked_hour }
  MonthStartDate:=1;
  Period:=-1;
  ProjectID:='-1';
  XML:='';
  try
    OutHTMLCalendar:= Calendars('worked_hour@month?', MonthStartDate, Period);
    MonthEndDate:= MonthStartDate+Period;

    if (Request.QueryFields.Values['oid']<>'') then ProjectID:=Request.QueryFields.Values['oid'];
    Query:=LoadFile('listworked_hour@month@' + RoID + DOCEXT_SQL);
    Query:=StringReplace(Query,'<#TRACK_DATE_1>',FormatDateTime('mm/dd/yyyy',MonthStartDate),[rfIgnoreCase]);
    Query:=StringReplace(Query,'<#TRACK_DATE_2>',FormatDateTime('mm/dd/yyyy',MonthEndDate),[rfIgnoreCase]);
    Query:=StringReplace(Query,'<#LOGIN_USER_ID>',UsID,[rfIgnoreCase]);
    COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
    CheckCOMError(COMResult,OutValue);
    XML:='<projects><d>'+OutValue+'</d></projects>';

    if ProjectID<>'-1' then
    begin
      if RoID=IntToStr(ROLE_ID_OBS) then
        Query:='select h.project_id,p.project_name,h.track_date,sum(h.total)       '+
        '  from tts.worked_hour h, tts.project p, tts.login_user ou, tts.client c  '+
        ' where                                                                    '+
        '   (TRACK_DATE between :TRACK_DATE$1 and :TRACK_DATE$2)                   '+
        '   and ((h.PROJECT_ID(+)=p.PROJECT_ID))                                   '+
        '   and h.grid =:GRID                                                      '+
        '   and h.project_id=:PROJECT_ID                                           '+
        '   and h.project_id=p.project_id                                          '+
        '   and p.active =''Y''                                                    '+
        '   and p.client_id=c.client_id                                            '+
        '   and c.active=''Y''                                                     '+
        '   and c.client_id=ou.client_id                                           '+
        '   and ou.login_user_id=:OBSERVER_ID                                      '+
        '   group by h.project_id,p.project_name,h.track_date'+
        #127'PROJECT_ID= '+ ProjectID +
        #127'TRACK_DATE$1='+ FormatDateTime('mm/dd/yyyy',MonthStartDate)+
        #127'TRACK_DATE$2='+ FormatDateTime('mm/dd/yyyy',MonthEndDate)+
        #127'OBSERVER_ID='+UsID
      else if RoID = IntToStr(ROLE_ID_MAN) then
        Query:='select h.project_id,p.project_name,h.track_date,sum(h.total)       '+
        '   from tts.worked_hour h, tts.project p                                  '+
        'where                                                                     '+
        ' (TRACK_DATE between :TRACK_DATE$1 and :TRACK_DATE$2)                     '+
        ' and ((h.PROJECT_ID(+)=p.PROJECT_ID))                                     '+
        ' and h.grid =:GRID                                                            '+
        ' and h.project_id=:PROJECT_ID                                             '+
        ' and h.project_id=p.project_id                                            '+
        ' and p.active =''Y''                                                      '+
        ' group by h.project_id,p.project_name,h.track_date                        '+
        #127'PROJECT_ID= '+ ProjectID +
        #127'TRACK_DATE$1='+ FormatDateTime('mm/dd/yyyy',MonthStartDate)+
        #127'TRACK_DATE$2='+ FormatDateTime('mm/dd/yyyy',MonthEndDate);
    COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
    CheckCOMError(COMResult,OutValue);
    XML:=XML+'<hw>'+OutValue+'</hw>';
   end;

   XML:='<d><i><s>'+SessionID+'</s><sti>'+Request.QueryFields.Values['sti']+'</sti>'+
      '<pi>'+ProjectID+'</pi>'+
      '<sd>'+FormatDateTime('yyyymmdd', MonthStartDate)+'</sd>'+
      '<ro>'+RoID+'</ro><ui>'+UsID+'</ui>'+
      '<hf>'+GroupHourFormat+'</hf></i>'+XML+'</d>';

   XSLT:=LoadFile('listworked_hour@month'+DOCEXT_XSL);
   COMResult:=XSLS.Transform(MAX_WAIT,XML,XSLT,OutValue);
   CheckCOMError(COMResult,OutValue);
   OutHTML:=OutValue;
 except
    on E: Exception do
      OutHTML := ErrorMsg('worked_hour@month', E.Message);
  end;
end;

procedure TWM.WMWebActionItemPostWorkedHourAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  XML, UserID, StartDate, EndDate, Query:String;
  OutParameters: WideString;
begin { /postworked_hour }
   try
    if RoID = IntToStr(ROLE_ID_USER) then UserID:=UsID
    else UserID:=Request.ContentFields.Values['LOGIN_USER_ID'];

    if Request.QueryFields.Values['action']='submit' then
    begin
      StartDate:=Request.ContentFields.Values['StartDate'];
      EndDate:=Request.ContentFields.Values['EndDate'];
      Query:='call tts.hproject_report(:USID, :START_DATE, :END_DATE ,:grid)'#127'START_DATE='+StartDate+#127'END_DATE='+EndDate+#127'USID='+UserID;
      COMResult:=ARS.Execute(MAX_WAIT, StrToInt(GrID), -1, Query ,False,'','','', OutParameters,OutValue );
      CheckCOMError(COMResult,OutValue);
    end;
    XML:=LoadFile('postworked_hour'+DOCEXT_XML);
    XML:=StringReplace(XML, '<#DATA>', Request.ContentFields.Values['xmlm'], [rfIgnoreCase]);
    XML:=StringReplace(XML, '[#LOGIN_USER_ID]', UserID, [rfReplaceAll]);
    XML:=StringReplace(XML, '[#action]', '<w c="SUBMITTED">Y</w>', [rfReplaceAll]);
    XML:=StringReplace(XML, '[#naction]', '<w c="SUBMITTED">N</w>', [rfReplaceAll]);
    DoMultipleUpdate(XML, GrID, OutValue) ;
    Response.SendRedirect(Request.Referer);
  except
    on E: EDuplicateEntryFound do
      OutHTML := Msg('<h2>Duplicate entry error!</h2>One of the following actions may have occurred: <br/>- The "Send for approval" button was pressed more than once. <br/>- After clicking on "Save. I''ll submit later" or "Send for approval",'+
                     ' the "Back" browser button was clicked and then clicked on "Save. I''ll submit later" or "Send for approval" button again.<br/>-"Send for approval" button was clicked immediately after "Save. I''ll submit later" before the hours were saved.'+
                     '<br/><br/>If none of the above actions occurred, please contact <a href="mailto:'+ErrorEmail+'">'+ErrorEmail+'</a> for further assistance.','stop');
   on E: Exception do
      OutHTML := ErrorMsg('postworked_hour', E.Message);
  end;
  Handled:=True;
end; { /postworked_hour }


procedure TWM.WMWebActionItemApproveAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  Query,LoginUserIDs,LoginUserNames, ExpStatementIDs, ExpStatementNames: String;
  PeriodStartDate, PeriodEndDate: TDateTIme;
  OutParameters: WideString;
begin { /approve}
  try
    PeriodStartDate:=EncodeDate(StrToInt(Request.ContentFields.Values['y']), StrToInt(Request.ContentFields.Values['m']), StrToInt(Request.ContentFields.Values['d']));
    PeriodEndDate:=PeriodStartDate+GroupPeriod-1;
    if (Request.QueryFields.Values['action']='process') and (RoID=IntToStr(ROLE_ID_MAN)) then
    begin
      Query:=' begin'+
             ' update tts.worked_hour_v wh set wh.processed=''Y'', wh.last_updated_by=:login_user_id   '+
             '  where wh.approved=''Y'' and wh.submitted=''Y'' and wh.grid=:GRID                 '+
             '  and wh.TRACK_DATE between :TRACK_DATE$1 and :TRACK_DATE$2;             '+
             ' update tts.exp_statement e set e.processed=''Y'', e.last_updated_by=:login_user_id'+
             '  where e.approved=''Y'' and e.submitted=''Y'' and e.grid=:GRID           '+
             '  and e.submitted_date between :TRACK_DATE$1 and :TRACK_DATE$2; '+
             ' end;'+
             #127'login_user_id='+UsID +
             #127'TRACK_DATE$1='+ FormatDateTime('mm/dd/yyyy',PeriodStartDate)+
             #127'TRACK_DATE$2='+ FormatDateTime('mm/dd/yyyy',PeriodEndDate);
      COMResult:=ARS.Update(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutValue);
      CheckCOMError(COMResult,OutValue);
 {
Query:='select t.login_user_id from tts.purchase_order t, tts.worked_hour h  where '+
       ' t.login_user_id in bla bla                    '+
' and sysdate>t.start_date and sysdate<t.end_date          '+
' and t.creation_date = (                                  '+
'     select max(s.creation_date)                          '+
'     from tts.purchase_order s                            '+
'     where s.login_user_id=t.login_user_id                '+
'     and sysdate>s.start_date                             '+
'     and sysdate<s.end_date)                              '+
' and h.login_user_id=t.login_user_id                      '+
' and h.processed_date between t.start_date and t.end_date  and t.grid=:grid and h.grid=:grid'+
' group by t.login_user_id, t.hours having (t.hours - sum(h.total)) <0 ';

    COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
    CheckComError(COMResult, OutValue);
    XML:='<l>'+OutValue+'</l>';
    COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
    CheckCOMError(COMResult,OutValue);
    LoginUserIDs:=OutValue;
    if Pos(',', LoginUserIDs)>0 then
    begin
      MsgRecs:=GetUserIDsCommaList(' t.role_id='+ IntToStr(ROLE_ID_MAN) + ' and t.EMAIL_NOTIFICATION_PO=''Y''');
      SendTemplateEmail('PO NOTIFICATION', MsgRecs, OutValue);
      //SendEmail(SystemEmail, MsgRecs,  EmptyStr, 'PO hours', ' has 30 hours above the P. O. for the period of time.' , False, OutValue);
    end;
    OutHTML:=Msg('','send');

 OutHTML:='';
 Exit;  }
  end
    else if (Request.QueryFields.Values['action']='notify') and  (RoID=IntToStr(ROLE_ID_MAN)) then
    begin
      LoginUserIDs:=Request.ContentFields.Values['LoginUserIDs'];
      SendTemplateEmail('SUBMIT REMINDER', LoginUserIDs, OutValue);
      Query:='insert into tts.notification_log nl '+
             '       (mail_template_id, object_id, object_name, send_date, last_updated_by, grid) '+
             ' values((select m.mail_template_id from tts.mail_template m where m.up_name=''SUBMIT REMINDER'' and m.grid=:grid),:user_id, ''WORKED_HOUR'', :TRACK_DATE$1, :last_updated_by, :grid ) '+
              #127'last_updated_by='+UsID +
              #127'user_id='+LoginUserIDs +
              #127'TRACK_DATE$1='+ FormatDateTime('mm/dd/yyyy',PeriodStartDate);
      COMResult:=ARS.Execute(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutParameters,OutValue);
 
      CheckCOMError(COMResult,OutValue);
    end
    else
    begin
      LoginUserIDs:=Request.ContentFields.Values['LoginUserIDs'];
      LoginUserNames:=Request.ContentFields.Values['LoginUserNames'];
      if LoginUserIDs<>EmptyStr then
      begin
        Query:=' update tts.worked_hour_v w '+
          ' set  w.approved=''Y'', w.last_updated_by=:LOGIN_USER_ID'+
          ' where w.login_user_id in ('+LoginUserNames+') '+
          ' and w.TRACK_DATE between :TRACK_DATE$2_bx_1 and :TRACK_DATE$2_bx_2'+
          ' and w.approved<>''Y'' and w.submitted=''Y''';
         if (RoID=IntToStr(ROLE_ID_SPVR)) then  Query:=Query + ' and w.project_id in (select sp.project_id from tts.supervisor_project sp where sp.supervisor_id=:LOGIN_USER_ID)';
        Query:= Query+ ' and w.grid=:GRID'+
          #127'LOGIN_USER_ID='+UsID+
          StringReplace(LoginUserIDs, '@',#127,[rfReplaceAll])+
          #127'TRACK_DATE$2_bx_1='+ FormatDateTime('mm/dd/yyyy',PeriodStartDate)+
          #127'TRACK_DATE$2_bx_2='+ FormatDateTime('mm/dd/yyyy',PeriodEndDate);
        COMResult:=ARS.Update(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutValue);
        CheckCOMError(COMResult,OutValue);
      end;
      ExpStatementIDs:=Request.ContentFields.Values['ExpStatementIDs'];
      ExpStatementNames:=Request.ContentFields.Values['ExpStatementNames'];
      if ExpStatementIDs <> EmptyStr then
      begin
        Query:= ' update tts.exp_statement e ' +
                ' set e.approved=''Y'', e.last_updated_by=:LOGIN_USER_ID '+
                ' where e.exp_statement_id in ('+ExpStatementNames+') '+
                ' and (e.approved<>''Y'' or e.approved is null) and e.submitted=''Y''' +
                ' and e.grid=:GRID '+
                #127'LOGIN_USER_ID='+UsID+
                StringReplace(ExpStatementIDs, '@',#127,[rfReplaceAll]);
        COMResult:=ARS.Update(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutValue);
        CheckCOMError(COMResult,OutValue);
      end;
    end;
    Response.SendRedirect(Request.Referer);
  except
    on E: ENoRowsUpdated do
      OutHTML := ErrorMsg('approve','This payroll period has not approved hours.');
    on E: Exception do
      OutHTML := ErrorMsg('approve', E.Message);
  end;
  Handled:=True;
end; { /approve }


procedure TWM.WMWebActionItemLoginAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
begin { /login }
  try
    if DoLogin(OutValue) then
    begin
      if Request.QueryFields.Values['c']=EmptyStr then
      with Response.Cookies.Add do
      begin
          Name := 'referer';
          Value := Request.Referer;
          Secure := False;
          Expires := Now + 356;
      end;
      if SessionData.Values['role_id']=IntToStr(ROLE_ID_OBS) then
        Response.CustomHeaders.Add('Location=worked_hour@month?sti=1&s='+SessionID)
      else if SessionData.Values['grid']='-1' then
        Response.CustomHeaders.Add('Location=list?o=admin_group&sti=2&s='+SessionID)
      else if SessionData.Values['role_id']=IntToStr(ROLE_ID_HR) then
        Response.CustomHeaders.Add('Location=list?o=accounting_info&sc=t.up_last_name&sti=5&s='+SessionID + '&xml=<w%20null=''''><f%20n=''a.termination_date''/></w>')
      else
        Response.CustomHeaders.Add('Location=worked_hour?sti=1&s='+SessionID);
      Response.StatusCode:=302;
      Response.SendResponse;
    end;
  except
    on E: ENoDataFound do
        OutHTML:='<div id="e">Could not find that user name or password is invalid. Please enter valid user name and password.<br/><br/> <#loginarea></div>';
    on E: Exception do
        OutHTML:=ErrorMsg('login', E.Message);
  end;
end; { /login }


procedure TWM.WMWebActionItemLogoutAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  COMResult:HResult;
  OutValue, Referer:WideString;
  SessionExists:WordBool;
begin { /logout }
  try
    COMResult:=WSS.InvalidateWebSession(SessionID,SessionExists,OutValue);
    CheckCOMError(COMResult,OutValue);
    Referer:=Request.CookieFields.Values['referer'];
    if Referer=EmptyStr then Referer:='home';
    Response.SendRedirect(Referer);  except
  on E: Exception do
  	OutHTML := ErrorMsg('logout', E.Message);
  end;
end; { /logout }

procedure TWM.WMWebActionItemSendAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
 Subject, Content, FromUser, FromEmail,ToUser, CCToUser, ObjectName:String;
begin  {/send}
  try
    ObjectName:= Request.QueryFields.Values['o'];
    Subject:=Request.ContentFields.Values['subject'];
    Content:=Request.ContentFields.Values['content'];
    ToUser:=Request.ContentFields.Values['to_login_user_id'];
    if ObjectName=OBJ_MAIL_TMPL then
    begin
      FromEmail:=Request.ContentFields.Values['from_email'];
      CCToUser:=Request.ContentFields.Values['cc_email'];
      SendEmail(FromEmail, ToUser, CCToUser,  Subject, Content,  (Request.ContentFields.Values['format']='H') , OutValue);
    end
    else if ObjectName=OBJ_MESSAGE then
    begin
      FromUser:=UsID;
      SendMessage(fromUser, ToUser, Subject, Content, (Request.ContentFields.Values['urgent']='Y'),OutValue);
    end;
    Response.SendRedirect(Request.Referer);
  except
    on E: Exception do
      OutHTML := ErrorMsg('send', E.Message);
  end;
  Handled:=True;
end;  {/send}

procedure TWM.WMWebActionItemListAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  XMLList, XMLOrder, XMLWhere, Query: String;
  StartDate, EndDate: TDateTime;
  Period, PageNumber: Integer;
  UsePages: Boolean;
  Year, Month, Day: Word;
begin { /list}
  OutValue:=EmptyStr;
  XMLWhere:=EmptyStr;
  XMLOrder:=EmptyStr;
  try
    XMLList:=Trim(LoadFile('list'+PO.Name+DOCEXT_XML));
    XMLWhere:=GetRequestValue('xml');
    if XMLWhere=EmptyStr then
      XMLWhere:=UrlDecode(Request.CookieFields.Values['xml_list' + PO.Name]);
    
    //calendars
    case PO.ListCalendar.CalendarType of
    1:
      begin  //'p'=1=payroll
        Period:= GroupPeriod-1;
        GetPayrollPeriod(StartDate, EndDate);
        OutHTMLCalendar:=Calendars('list?o=' + PO.Name + '&amp;c=p&amp;f=' + PO.ListCalendar.FieldName + '&amp;', StartDate, Period);
        XMLWhere:=XMLWhere+'<w b="TRACK_DATE"><f n="trunc(' + PO.ListCalendar.FieldName + ')">'+ FormatDateTime('mm/dd/yyyy',StartDate)+ '::' + FormatDateTime('mm/dd/yyyy', EndDate) + '</f></w>';
      end;
    2:  //m='month'=2
      begin
        StartDate:=1;
        Period:=-1;
        OutHTMLCalendar:=Calendars('list?o=' + PO.Name + '&amp;c=m&amp;f=' + PO.ListCalendar.FieldName + '&amp;', StartDate, Period);
        XMLWhere:=XMLWhere+'<w b="CREATION_DATE"><f  n="trunc(' + PO.ListCalendar.FieldName + ')">'+ FormatDateTime('mm/dd/yyyy',StartDate)+ '::' + FormatDateTime('mm/dd/yyyy', StartDate+Period) + '</f></w>';
      end;
    3:   //custom =3='c'
      begin
        Month:=StrToInt(Request.QueryFields.Values['m']);
        Year:=StrToInt(Request.QueryFields.Values['y']);
        Day:=StrToInt(Request.QueryFields.Values['d']);
        StartDate:=EncodeDate(Year, Month, Day);
        try
          Period:=StrToInt(Request.QueryFields.Values['pr'])-1;
        except
          Period:=GroupPeriod-1;
        end;
        OutHTMLCalendar:=Calendars('list?o=' + PO.Name + '&amp;c=c&amp;&amp;pr='+ IntToStr(Period+1)+ '&amp;f=' + PO.ListCalendar.FieldName + '&amp;', StartDate, Period);
        XMLWhere:=XMLWhere+'<w  b="CREATION_DATE"><f n="trunc(' + PO.ListCalendar.FieldName + ')">'+ FormatDateTime('mm/dd/yyyy',StartDate)+ '::' + FormatDateTime('mm/dd/yyyy', StartDate+Period) + '</f></w>';
      end;
    end;
    //end calendar
    XMLList:=StringReplace(XMLList,'<#condition>', XMLWhere,[rfReplaceAll]);

    if (GetRequestValue('sc')<>EmptyStr) then
      XMLOrder:=GetRequestValue('sc')+ ' '+ GetRequestValue('so')
    else
      XMLOrder:=PO.ListOrder;
    XMLList:=StringReplace(XMLList,'<#order>', XMLOrder,[rfReplaceAll]);

    XMLList:= StringReplace(XMLList, '<#usid>', UsID, [rfReplaceAll]);
    XMLList:= StringReplace(XMLList, '<#roid>', RoID, [rfReplaceAll]);
    XMLList:= StringReplace(XMLList, '<#oid>', GetRequestValue('oid'), [rfReplaceAll]);

    UsePages:= (PO.ListUsePages and PO.ListUseFrame);
    DoList(XMLList, PageNumber, UsePages);

    {
    account_info
    account_info_history
    invoice
    ira_contrib
    }
    if  PO.SecurityOn and (GrID<>'-1') then
    begin
      Query:='select upper(f.field_name), r.type '+
           ' from tts.role_field_privilege r, tts.field_privilege f, tts.object_privilege op '+
           ' where op.object_name=:object_name' +
           ' and op.object_privilege_id=f.object_privilege_id '+
           ' and f.field_privilege_id=r.field_privilege_id and r.role_id=:role_id' +
           #127'object_name=' + PO.SecurityName +
           #127'role_id=' + PrID;
      COMResult:=ARS.List(MAX_WAIT,-1,-1,Query,0,OutValue);
      CheckCOMError(COMResult,OutValue);
      XMLList:='<privileges><d>'+OutValue+'</d></privileges>' + XMLList;
    end;

    XMLList:='<d>'+XMLList+'<i>'+
        '<s>'+SessionID+'</s><oa>' +PO.Alias + '</oa>'+
        '<sti>'+Request.QueryFields.Values['sti']  +'</sti>'+
        '<ro>'+RoID+'</ro><ui>'+UsID+'</ui>'+
        '<sc>'+Request.ContentFields.Values['sc']+'</sc><so>'+Request.ContentFields.Values['so']+'</so>'+
        '<oid>'+GetRequestValue('oid')+'</oid><o>'+GetRequestValue('o')+'</o>'+
        '<td>'+GetRequestValue('TrackDate')+'</td><hf>'+GroupHourFormat+'</hf>';
      if PO.ListCalendar.CalendarType>0 then
        XMLList:=XMLList + '<y>'+Request.QueryFields.Values['y']  +'</y><m>'+Request.QueryFields.Values['m']  +'</m><d>'+Request.QueryFields.Values['d']  +'</d>'+
                           '<sd>' + FormatDateTime('mm/dd/yyyy',StartDate) + '</sd><ed>' + FormatDateTime('mm/dd/yyyy',StartDate+Period) + '</ed><pr>'+IntToStr(Period) +'</pr>';

      if UsePages then
        XMLList:= XMLList+'<pages>' + BoolToStr(UsePages)+ '</pages><pn>'+IntToStr(PageNumber)+'</pn><ps>'+IntToStr(LIST_PAGE_SIZE)+'</ps>';
      XMLList:= XMLList +  '<sq><![CDATA['+XMLWhere+']]></sq></i></d>';

    COMResult:=XSLS.Transform(MAX_WAIT,XMLList,LoadFile('list'+PO.Name +DOCEXT_XSL),OutValue);
    CheckCOMError(COMResult,OutValue);
    OutHTML:=OutValue;
  except
    on E:EListError do
      OutHTML := ErrorMsg('list', 'Unable to load a Privilege Object. ' + E.Message);
    on E: Exception do
      OutHTML := ErrorMsg('list'+PO.Name, E.Message);
  end;          
end; { /list}

procedure TWM.WMWebActionItemDetailAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  XML, XSL, Query, ObjectID, ObjectName: String;
  I, PageNumber :Integer;
  UseDefaultSQL: Boolean;
begin { /detail }
  try
    XML:= EmptyStr;
    OutValue:= EmptyStr;
    ObjectName:= GetRequestValue('o');
    ObjectID:= GetRequestValue('oid');
    UseDefaultSQL:= False;

    if (((ObjectName = OBJ_TRAVEL_PROFILE) or (ObjectName=OBJ_W4INFO)) and ((RoID<>IntToStr(ROLE_ID_MAN)))) then ObjectID:=UsID;
    if Request.PathInfo='/detail' then
    begin
      Query:=Trim(PO.DetailSQL.SQL.Text);
      case PO.DetailSQL.Param of
        pObjectID: Query:=Query+#127 + 'oid=' + ObjectID;
        pGrID: Query:=Query+#127 + 'grid=' + GrID;
        pUserID: Query:=Query+#127 + 'usid=' + UsID;
      end;
      try
        COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID),  -1, Query, OutValue);
        CheckComError(COMResult, OutValue);
        XML:=OutValue;
        if PO.Name=OBJ_MESSAGE then MarkAsReadMsg(OutValue);
      except
        on E: ENoDataFound do UseDefaultSQL:=True;
      end;
      
      for I:=0 to PO.OptionLists.Count-1 do
      with PO.OptionLists[I] do
      begin
        if (PO.OptionLists[I].Name <> 'default') or UseDefaultSQL then
        begin
          Query:=Trim(FQueryItem.SQL.Text);
          case FQueryItem.Param of
            pObjectID: Query:=Query+#127 + 'oid=' + ObjectID;
            pGrID: Query:=Query+#127 + 'grid=' + GrID;
            pUserID: Query:=Query+#127 + 'usid=' + UsID;
          end;
          COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
          CheckComError(COMResult, OutValue);
          if PO.OptionLists[I].Name='default' then XML:=XML+OutValue
          else XML:=XML+'<'+PO.OptionLists[I].Name+'>'+ OutValue +'</'+PO.OptionLists[I].Name+'>';
        end;
      end;
      if (Request.QueryFields.Values['p']<>EmptyStr) then XSL:=LoadFile('detail'+ ObjectName + '@print' + DOCEXT_XSL)
      else XSL:=LoadFile('detail'+ ObjectName+DOCEXT_XSL);
      end
      else if Request.PathInfo='/search'  then
      begin
      for I:=0 to PO.OptionListsSearch.Count-1 do
        with PO.OptionListsSearch[I] do
        begin
           Query:=Trim(FQueryItem.SQL.Text);
          case FQueryItem.Param of
            pObjectID: Query:=Query+#127 + 'oid=' + ObjectID;
            pGrID: Query:=Query+#127 + 'grid=' + GrID;
            pUserID: Query:=Query+#127 + 'usid=' + UsID;
          end;
          COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
          CheckComError(COMResult, OutValue);
          if PO.OptionListsSearch[I].Name='default' then XML:=XML+OutValue
          else XML:=XML+'<'+PO.OptionListsSearch[I].Name+'>'+ OutValue +'</'+PO.OptionListsSearch[I].Name+'>';
        end;
        XSL:=LoadFile('search'+ ObjectName+DOCEXT_XSL);
      end;
        
      if PO.SecurityOn and (GrID<>'-1') then
      begin
        Query:= 'select  upper(f.field_name),rf.type, f.field_alias, f.field_type, f.required_field '+
        ' from tts.role_field_privilege rf, tts.field_privilege f , tts.role r '+
        ' where r.role_id=:pr_id and r.role_id=rf.role_id and rf.field_privilege_id=f.field_privilege_id'+
        ' and f.object_name=:object_name and (r.grid=:grid or r.grid is null)'+
        #127'pr_id=' +PrID + #127'object_name=' + PO.SecurityName;
        COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
        CheckComError(COMResult, OutValue);
        XML:=XML+'<privileges>'+ OutValue +'</privileges>';
      end;
   
       GetPageNumber(PageNumber);
       XML:='<?xml version="1.0"?><d><i><s>'+SessionID+'</s><oa>' +PO.Alias + '</oa>'+
         '<sti>'+Request.QueryFields.Values['sti']+'</sti>'+
         '<y>'+Request.QueryFields.Values['y']+'</y><m>'+Request.QueryFields.Values['m']+'</m><d>'+Request.QueryFields.Values['d']+'</d>'+
         '<gd>'+FormatDateTime('mm/dd/yyyy',GroupStartDate)+'</gd>'+
         '<gp>'+IntToStr(GroupPeriod)+'</gp><gn>' + GroupName + '</gn>'+
         '<ghf>'+GroupHourFormat+'</ghf>'+
         '<oid>'+ObjectID+'</oid><o>'+ObjectName+'</o><ui>'+UsID+'</ui>'+
         '<ro>'+RoID+'</ro><lr>'+GetRequestValue('lr')+'</lr>'+
         '<pn>'+IntToStr(PageNumber)+'</pn><ps>'+IntToStr(LIST_PAGE_SIZE)+'</ps>'+
         '<p>' + Request.QueryFields.Values['p'] + '</p><uid>' + Request.QueryFields.Values['uid'] + '</uid><r>' + Request.QueryFields.Values['r'] + '</r></i>'+XML;
         XML:=XML+'</d>';
    COMResult:=XSLS.Transform(MAX_WAIT,XML,XSL,OutValue);
    CheckCOMError(COMResult,OutValue);
    OutHTML:=OutValue;
  except
    on E:EListError do
      OutHTML := ErrorMsg('detail', 'Unable to load a Privilege Object. ' + E.Message);
    on E: Exception do
      OutHTML := ErrorMsg('detail'+ ObjectName, E.Message);
  end;
  Handled:=True;
end; { /detail }


procedure TWM.WMWebActionItemUpdateAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  XML, MsgRecs: String;
  RowCount:Integer;
  UsedGrID: String;
begin { /update }
  if PO.IgnoreGrID then UsedGrID:='-1'
  else UsedGrID:= GrID;
  try
    if(Request.ContentFields.Values['xmll'] <> EmptyStr) then     //update multiple records
    begin
      XML:=GenerateBaseXML(GetRequestValue('ol'),Request.ContentFields.Values['xmll'], PO.IgnoreGrID);
      DoMultipleUpdate(XML, UsedGrID, OutValue);
    end;
    if(Request.ContentFields.Values['xmlm'] <> EmptyStr) then
    begin
      XML:=GenerateBaseXML(GetRequestValue('om'),Request.ContentFields.Values['xmlm'], PO.IgnoreGrID);
      DoMultipleUpdate(XML, UsedGrID, OutValue);
    end;
    if(Request.ContentFields.Values['xmln'] <> EmptyStr) then
    begin
      XML:=GenerateBaseXML(GetRequestValue('on'),Request.ContentFields.Values['xmln'], PO.IgnoreGrID);
      DoMultipleUpdate(XML, UsedGrID, OutValue);
    end;
    if(Request.ContentFields.Values['xmlo'] <> EmptyStr) then
    begin
      XML:=GenerateBaseXML(GetRequestValue('oo'),Request.ContentFields.Values['xmlo'], PO.IgnoreGrID);
      DoMultipleUpdate(XML, UsedGrID, OutValue);
    end;
    if(Request.ContentFields.Values['xmlp'] <> EmptyStr) then
    begin
      XML:=GenerateBaseXML(GetRequestValue('op'),Request.ContentFields.Values['xmlp'], PO.IgnoreGrID);
      DoMultipleUpdate(XML, UsedGrID, OutValue);
    end;
    if(Request.ContentFields.Values['xml'] <> EmptyStr) then
    begin
      XML:=GenerateBaseXML(PO.DBTableName, Request.ContentFields.Values['xml'],  PO.IgnoreGrID);
      DoUpdate(XML , UsedGrID, RowCount, OutValue);
      CheckCOMError(COMResult, OutValue);
    end;

    if (PO.Name=OBJ_W4INFO) and ((RoID=IntToStr(ROLE_ID_USER))) then  //broadcast managers msg for changed w4info
    begin
      MsgRecs:=GetUserIDsCommaList(' t.role_id='+ IntToStr(ROLE_ID_MAN) + ' and t.EMAIL_NOTIFICATION_W4=''Y'' ');
      SendTemplateEmail('W4 NOTIFICATION', MsgRecs, OutValue);
    end
    else if (PO.Name=OBJ_TODOLOG) then UpdateSessionData('checked_out_to_do',EmptyStr)
    else if PO.Name=OBJ_TRAVEL_PROFILE then PostTravelProfile(GetRequestValue('travel_profile_id'));
    if not(PO.Name=OBJ_TRAVEL_PROFILE) then Response.SendRedirect(Request.Referer);
  except
    on E:EListError do
      OutHTML := ErrorMsg('update', 'Unable to load a Privilege Object. ' + E.Message);
    on E:EChildRecordFound do
      OutHTML :=ErrorMsg('update ' + PO.Name, 'Child record found. ' + E.Message);
    on E: Exception do
      OutHTML:= ErrorMsg('update ' + PO.Name, E.Message);
  end;
  Handled:=True;
end; { /update }

procedure TWM.WMWebActionItemInsertAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  XML, ObjectName, UserName, UserPassword, InsertXML, UsedGrID: String;
  ObjectID: WideString;
begin { /insert }
  try
   if PO.IgnoreGrID then UsedGrID:='-1'
    else UsedGrID:= GrID;

    ObjectName:=GetRequestValue('o');
    InsertXML:=Request.ContentFields.Values['xml'];
    if ObjectName=OBJ_LOGIN_USER then
    begin
      Randomize;
      UserName:=Request.ContentFields.Values['user_name'];
      UserPassword:= RandomString(10);
      InsertXML:=InsertXML + '<w c="USER_NAME">'+UserName+'</w><w c="USER_PASS">' + UserPassword + '</w>';
    end;
    if(Request.ContentFields.Values['xmll'] <> EmptyStr) then     //update multiple records
    begin
      XML:=GenerateBaseXML(GetRequestValue('ol'),Request.ContentFields.Values['xmll'], PO.IgnoreGrID);
      DoMultipleUpdate(XML, UsedGrID, OutValue);
    end;

    XML:=GenerateBaseXML(PO.DBTableName, InsertXML, PO.IgnoreGrID);
    DoInsert(XML, UsedGrID,  ObjectID, OutValue);
    CheckCOMError(COMResult,OutValue);

    if ObjectName = OBJ_LOGIN_USER then SendTemplateEmail('LOGIN NOTIFICATION', ObjectID, OutValue)
    else if (PO.Name=OBJ_TODOLOG) then UpdateSessionData('checked_out_to_do',GetRequestValue('oid'))
    else if ObjectName=OBJ_TRAVEL_PROFILE then  PostTravelProfile(ObjectID);

    if not(ObjectName=OBJ_TRAVEL_PROFILE) then Response.SendRedirect(Request.Referer+'&oid='+ObjectID);
  except
    on E:EListError do
      OutHTML := ErrorMsg('update', 'Unable to load a Privilege Object. ' + E.Message);
    on E: EDuplicateLogin do
      OutHTML := Msg('We''re sorry, but the login name you entered is already tied to a different account.', 'stop');
    on E: Exception do
      OutHTML := ErrorMsg('insert '+PO.Name, E.Message);
  end;
  Handled:=True;
end; { /insert }


procedure TWM.WMWebActionItemDeleteAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
   XML, ObjectID, UsedGrID: String;
   RowCount:Integer;
begin { /delete }
  if PO.IgnoreGrID then UsedGrID:='-1'
  else UsedGrID:= GrID;
 try
    ObjectID:=Request.ContentFields.Values['oid'];
    if Request.QueryFields.Values['soft'] <> EmptyStr then
    begin
      XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>';
      if (PO.IgnoreGrID) then XML:=XML+'G';
      XML:=XML+'B</gb></i><tt><t>TTS.'+PO.DBTableName+'</t></tt><ww><w c='''+PO.DBTableName+'_id'' k="">'+ ObjectID+'</w><w c="ACTIVE">N</w></ww></d>';
      DoUpdate(XML, UsedGrID,  RowCount, OutValue);
    end
    else
    begin
      XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>';
      if (PO.IgnoreGrID) then XML:=XML+'G';
      XML:=XML+'B</gb></i><tt><t>TTS.'+PO.DBTableName+'</t></tt><ww><w c='''+PO.DBTableName+'_id''>'+ ObjectID+'</w></ww></d>';
      DoDelete(XML, UsedGrID, RowCount, OutValue);
    end;
    Response.SendRedirect(Request.Referer);
  except
    on E:EListError do
      OutHTML := ErrorMsg('update', 'Unable to load a Privilege Object. ' + E.Message);
    on E:EChildRecordFound do
      OutHTML := Msg('This object cannot be deleted because it contains past data. You can make the object inactive instead.','search');
    on E:Exception do
      OutHTML := ErrorMsg('delete' + PO.Name, E.Message);
  end;
end; { /delete }



procedure TWM.WMWebActionItemFileAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  FileStream: TFileStream;
  Query, XML, FileName, OriginalFileName, ObjectName, ObjectID, RequestReferer, FileContentType,FileExt, FileNotes:String;
  I, RowCount: Integer;
  FileID: WideString;
begin { /file }
  RequestReferer:=Request.Referer;
  try
    if Request.MethodType=mtPost then
    begin
      ObjectName:=ExtractObjectName(Request.QueryFields.Values['o']);
      ObjectID:=Request.QueryFields.Values['oid'];
      with TMsMultipartFormParser.Create do
      try
        Parse(Request);
          for I := 0 to Files.Count -1 do
          begin
              FileName:=ExtractFileName(Files[i].FileName);
              FileContentType:=Files[i].ContentType;
              FileExt:=ExtractFileExt(FileName);
              XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>';
              if PO.IgnoreGrID then XML:=XML+'G';
              XML:=XML+'B</gb><s>'+SessionID+'</s></i><tt><t>TTS.DOCUMENT</t></tt>'+
              '<ww><w c="DOCUMENT_ID"  k=""></w>'+
              '<w c="OBJECT_ID">' +ObjectID+ '</w>'+
              '<w c="OBJECT_NAME">' +ObjectName+ '</w>'+
              '<w c="ORIGINAL_FILE_NAME">'+FileName+'</w>'+
              '<w c="FILE_EXT">'+FileExt+'</w>'+
              '<w c="NOTE">'+ FileNotes +'</w>'+
              '<w c="CONTENT_TYPE">'+ FileContentType +'</w>'+
              '<w c="LAST_UPDATED_BY">'+UsID+'</w></ww></d>';
              DoInsert(XML, GrID, FileID, OutValue);
              CheckCOMError(COMResult,OutValue);
              try
                Files[i].SaveToFile(UploadPath + ObjectName+ '_' + ObjectID + '_' + 'DOC_'+FileID + FileExt);
              except
                XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>GB</gb></i><tt><t>TTS.DOCUMENT</t></tt><ww><w c="DOCUMENT_ID">'+ FileID+'</w></ww></d>';
                DoDelete(XML, GrID, RowCount, OutValue);
              end;
          end;
        finally
          Free;
        end;
        Response.SendRedirect(RequestReferer);
  end
  else if(Request.MethodType=mtGet) then
  begin
    FileID:=Request.QueryFields.Values['oid'];
    Query:=PO.DetailSQL.SQL.Text+  #127' oid='+FileID;
    COMResult:=ARS.Detail(MAX_WAIT, -1,  -1, Query, OutValue);
    CheckCOMError(COMResult,OutValue);
    XML:='<d>'+OutValue+'</d>';
    COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
    CheckCOMError(COMResult,OutValue);
    with TStringList.Create do
    try
      Text:=OutValue;
      FileName:=Values['OBJECT_NAME']+ '_' + Values['OBJECT_ID'] + '_' + 'DOC_'+FileID + Values['FILE_EXT'];
      OriginalFileName:= Values['ORIGINAL_FILE_NAME'];
    finally
      Free;
    end;
    if  (Request.QueryFields.Values['a']='d')  then
    begin
     XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>GB</gb></i><tt><t>TTS.DOCUMENT</t></tt><ww><w c="DOCUMENT_ID">'+ FileID+'</w></ww></d>';
     DoDelete(XML, GrID, RowCount, OutValue);
     DeleteFile (FileName);
     Response.SendRedirect(RequestReferer);
    end
    else
    begin    
      FileStream:=TFileStream.Create(UploadPath+FileName,fmOpenRead OR fmShareDenyNone);
      try
        with Response do
        begin
          if Uppercase(ExtractFileExt(FileName))='.JPG' Then
            ContentType:='image/jpeg'
          else if Uppercase(ExtractFileExt(FileName))='.GIF' Then
            ContentType:='image/gif'
          else if Uppercase(ExtractFileExt(FileName))='.HTM' Then
            ContentType:='text/html'
          else
          begin
            ContentType:='application/x-zip-compressed';
            SetCustomHeader('Content-Disposition', 'attachment; filename="' + OriginalFileName + '"');
          end;
          ContentLength:=FileStream.Size;
          SendResponse;
          SendStream(FileStream);
      end;
      finally
        FileStream.Free;
      end;
    end;
  end;
  except
    on E: Exception do
      OutHTML := ErrorMsg('file', E.Message);
  end;
  Handled := True;
end;

procedure TWM.WMWebActionItemResetPasswordAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  Query, UserEmail, ToUsers: string;
  XML: WideString;
begin { /resetpassword }
  try
    UserEmail:=Request.ContentFields.Values['email'];
    Query:= 'select login_user_id from tts.login_user t where t.up_email=:email and t.active=''Y'''+
          #127'email='+UpperCase(UserEmail);
    COMResult:=ARS.List(MAX_WAIT,-1,-1,Query,0,OutValue);
    CheckComError(COMResult, OutValue);
    XML:='<l>'+OutValue+'</l>';
    COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
    CheckCOMError(COMResult,OutValue);
    ToUsers:=OutValue;
    if Length(ToUsers)=0 then
      raise ENoDataFound.Create('We are sorry, the e-mail address you entered was not found in our records.<br/>Please enter the e-mail address you use in TTS, and try again.');
    SendTemplateEmail('RETRIEVE PASSWORD', ToUsers , OutValue);
    OutHTML:=Msg('Your request for retrieval of your lost password was successful. The password will be e-mailed to you shortly.','send');
  except
    on E: ESendEMail do
    begin
      OutHTML := ErrorMsg('resetpassword', 'There was a problem sending e-mail to ' + UserEmail + '.  Please contact ' +
        ErrorEMail + ' for assistance and reference the following error message: ' +
        E.Message);
    end;
    on E:ENoDataFound do
      OutHTML:=Msg(E.Message, 'stop');
    on E: Exception do
      OutHTML := ErrorMsg('resetpassword', E.Message);
  end;
  Handled:=True;
end; { /resetpassword }


procedure TWM.WMWebActionItemJabberAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  JabberLogin, JabberPassword, Action, Query: String;
  MessageText, ToUser, XML, XSLT:WideString;
begin
  JabberLogin:=XMLEscape(UserLogin + '_' + GroupName);
  JabberPassword:=UserLogin;
  MessageText:=EmptyStr;
  Action:=Request.QueryFields.Values['d'];
  try
      Response.ContentType:='text/xml';
      Response.SetCustomHeader('Cache-Control','no-store, private, no-transform');
      if Action='out' then //send message
      begin
          XML:=StringReplace(Request.ContentFields.Text,'<#from>', JabberLogin, []);
          XSLT:='<?xml version="1.0"?><xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">'+
                '<xsl:output method="text"/>'+
                '<xsl:template match="/mm/m">&#x0A;from=<xsl:value-of select="@from"/>&#x0A;to=<xsl:value-of select="@to"/>&#x0A;text=<xsl:value-of select="@text"/>'+
                '</xsl:template></xsl:stylesheet>';
          COMResult:=XSLS.Transform(MAX_WAIT,XML,XSLT,OutValue);
          CheckCOMError(COMResult,OutValue);
          with TStringList.Create do
          try
            Text:= OutValue;
            ToUser:=Values['to'];
            MessageText:=Values['text'];
          finally
            Free;
          end;
          COMResult:= JS.Send(MAX_WAIT, JabberLogin, ToUser, MessageText, 'CHAT', '');
          CheckCOMError(COMResult, MessageText);
          OutHTML:=XML;
          Handled:=True;
          UpdateChatLog(XML);
      end
      else if Action='in' then //get received messages from server 
      begin
        COMResult:= JS.GetLastMessages(MAX_WAIT, JabberLogin, Request.QueryFields.Values['status'],  XML);
        CheckCOMError(COMResult, XML);
        OutHTML:=XML;
        Handled:=True;
        if(XML <> '<mm session="active"></mm>') and (XML <> '<mm session="inactive"></mm>') then
          UpdateChatLog(XML);
      end
      else if Action='history' then
      begin
        Query:=' select t.chat_log from tts.chat_log t '+
             ' where t.grid=:GRID and t.login_user_id=:LOGIN_USER_ID'+
             ' and t.last_updated_date=(select max(d.last_updated_date) from tts.chat_log d  where d.login_user_id=t.login_user_id) '+
             #127'LOGIN_USER_ID='+UsID;
        COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID), -1, Query, OutValue);
        CheckComError(COMResult, OutValue);
        XML:='<d>'+OutValue+'</d>';
        COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
        with TStringList.Create do
        try
          Text:=OutValue;
          XML:='<log>' + Values['CHAT_LOG']+'</log>';
        finally
          Free;
        end;
        CheckCOMError(COMResult,OutValue);
        OutHTML:=XML;
        Handled:=True;
      end
      else if Action='disconnect' then //disconnect
      begin
        COMResult:=JS.Disconnect(JabberLogin, OutValue);
        CheckCOMError(COMResult,OutValue);
        OutHTML:='<mm session="inactive"></mm>';
      end
      else if Action='connect' then //connect to JabberServer
        JabberServerConnect(JabberLogin,JabberPassword, OutValue);
  except
    on E: ENoJabberSessionFound do
    begin
      ComResult:=JS.Disconnect(JabberLogin, OutValue);
      CheckCOMError(COMResult,OutValue);
      OutHTML:='<mm error=""><m text="ENoJabberSessionFound: ' +E.Message+ '" from="" to="' + JabberLogin + '"/></mm>';
    end;
    on E: EJabberCommunicationError do
       OutHTML:=ErrorMsg('jabber', '<mm error=""><m from="" to="'+ JabberLogin +'" text="EJabberCommunicationError: ' + E.Message + '"/></mm>');
    on E: EJabberAuthenticationError do
    begin
      ComResult:=JS.Disconnect(JabberLogin, OutValue);
      CheckCOMError(COMResult,OutValue);
      OutHTML:=ErrorMsg('jabber', '<mm error=""><m from="" to="'+ JabberLogin +'" text="' + E.Message + '"/></mm>');
    end;
    on E: Exception do
      OutHTML:=ErrorMsg('jabber', '<mm error=""><m from="" to="'+ JabberLogin +'" text="' + E.Message + '"/></mm>');
  end;
  Handled:=True;
end;

procedure TWM.WMWebActionItemMailLoginAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var
  Query, MailLogin, MailPass,XML: String;
begin
  try
  Query:='select mail_login, mail_pass from tts.login_user where login_user_id=:login_user_id and mail_login is not null and grid=:grid and active=''Y'''#127'login_user_id='+UsID;
  COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID),  -1, Query, OutValue);
  CheckCOMError(COMResult,OutValue);
  XML:='<d>'+OutValue+'</d>';
  COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
  CheckCOMError(COMResult,OutValue);
  with TStringList.Create do
  try
    Text:=OutValue;
    MailLogin:=Values['MAIL_LOGIN'];
    MailPass:= Values['MAIL_PASS'];
  finally
    Free;
  end;
  OutHTML:='<form name="frm" action="https://mail.objectsystems.com/webmail/src/redirect.php" target="mail" method="post" style="display:none">'+
           '<input type="hidden" name="login_username" value="' + MailLogin + '" /><input type="hidden" value="' + MailPass + '" name="secretkey" />'+
           '<input type="hidden" name="js_autodetect_results" value="SMPREF_JS_OFF" /><input type="hidden" name="just_logged_in" value="1" /></form>'+
           '<iframe name="mail" src="/tts/wait.htm" id="mail" width="100%" height="100%" frameborder="0"></iframe><script> window.setTimeout(''document.forms["frm"].submit()'', 1000);</script>';
  except
    on E:ENoDataFound do
      OutHTML:= ErrorMsg('maillogin', 'Before checking your e-mail, please make sure that fields "E-mail Login" and "E-mail Password" (<a href="detail?o=login_user&oid=' + UsID + '&s=' + SessionID + '&sti=3">Account/Web Mail Login screen</a>)are not empty. If you do not have Objectsystems/ Atttitude e-mail account ');
    on E:Exception do
      OutHTML := ErrorMsg('maillogin', E.Message);
  End;
end;


{==============================================================================}
{                             DB Utility functions                             }
{==============================================================================}

{
function TWM.DoDetail(const ObjectID: String; const ObjectName:String; out OutValue: WideString):WideString;
var
  SQL, SQLDefault: String;
begin
  with TStringList.Create do
   try
    LoadFromFile(MainDocsPath + 'detail' + ObjectName + DOCEXT_SQL);
    SQL:=Trim(Values['sql']) ;
    SQLDefault:=Trim(Values['sql_default']);
  finally
    Free;
  end;

  try
     SQL:= StringReplace(SQL, '<#oid>',ObjectID, [rfReplaceAll]);
     SQL:= StringReplace(SQL, '<#oname>',GetRequestValue('oname'), [rfReplaceAll]);
     SQL:= StringReplace(SQL, '<#usid>',UsID, [rfReplaceAll]);
     COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID), -1, SQL, OutValue);
     CheckComError(COMResult, OutValue);
  except
    on E:ENoDataFound do
    begin
      SQLDefault:= StringReplace(SQLDefault, '<#oid>',ObjectID, [rfReplaceAll]);
      SQLDefault:= StringReplace(SQLDefault, '<#login_user_id>',UsID, [rfReplaceAll]);
      SQLDefault:= StringReplace(SQLDefault, '<#chosen_user_id>',Request.ContentFields.Values['chosen_user_id'], [rfReplaceAll]);
      COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID), -1, SQLDefault, OutValue);
      CheckComError(COMResult, OutValue);
    end;
  end;
  Result:=OutValue;
  if (ObjectID<>EmptyStr) and (ObjectName=OBJ_MESSAGE) then MarkAsReadMsg(OutValue);
end; }

procedure TWM.DoList(var XML:String;  var PageNumber: Integer; const UsePages:Boolean);
var
  Query: WideString;
begin
  COMResult:=XSLS.Transform(MAX_WAIT, XML, CoreSearchSQLXSLT, OutValue);
  CheckCOMError(COMResult,OutValue);
  Query:=OutValue;
  if UsePages then
  begin  //add pages params
    GetPageNumber(PageNumber);
    if Pos(#127,Query)>0 then 
       Query:=StringReplace(Query,#127,') where ROW_NUMBER between ((:PAGE_NUMBER - 1) * :PAGE_SIZE + 1) and (:PAGE_NUMBER * :PAGE_SIZE + 1)'#127,[])
    else
       Query:=Query+') where ROW_NUMBER between ((:PAGE_NUMBER - 1) * :PAGE_SIZE + 1) and (:PAGE_NUMBER * :PAGE_SIZE + 1)';
    Query:='select * from ('+Query+#127'PAGE_NUMBER='+IntToStr(PageNumber)+#127'PAGE_SIZE='+IntToStr(LIST_PAGE_SIZE);
  end; //end pages
  COMResult:=ARS.List( MAX_WAIT,StrToInt(GrID),-1,Query,0,OutValue);
  CheckCOMError(COMResult,OutValue);
  XML:=(OutValue+XML);
end;

procedure TWM.DoInsert(XML, InsertGrID: String;out ObjID:WideString; out OutValue:WideString);
var
  Query:WideString;
begin
  XML:=StringReplace(XML, '[#LOGIN_USER_ID]', UsID, [rfReplaceAll]);
  COMResult:=XSLS.Transform(MAX_WAIT,XML,CoreInsertSQLXSLT,OutValue);
  CheckCOMError(COMResult,OutValue);
  Query:=OutValue;
  COMResult:=ARS.Insert(MAX_WAIT,StrToInt(InsertGrID),-1,Query,ObjID,False,'','','',OutValue);
  CheckCOMError(COMResult,OutValue);
end;

procedure TWM.DoUpdate(XML, UpdateGrID: String; out RowCount:Integer; out OutValue:WideString);
var Query: WideString;
begin
  XML:=StringReplace(XML, '<#LOGIN_USER_ID>', UsID, [rfReplaceAll]);
  COMResult:=XSLS.Transform(MAX_WAIT,XML,CoreUpdateSQLXSLT,OutValue);
  CheckCOMError(COMResult,OutValue);
  Query:=OutValue;
  COMResult:=ARS.Update(MAX_WAIT,StrToInt(UpdateGrID),-1,Query,False,'','','',OutValue);
  CheckCOMError(COMResult,OutValue);
  RowCount:=StrToInt(OutValue);
end;

procedure TWM.DoMultipleUpdate(XML:String; const UsedGrID: String; out OutValue:WideString);
var Query,OutParameters:WideString;
begin
  XML:=StringReplace(XML, '[#LOGIN_USER_ID]', UsID, [rfReplaceAll]);             //Request.ContentFields.Values['login_user_id']
  XML:=StringReplace(XML, '<#oid>', GetRequestValue('oid'), [rfReplaceAll, rfIgnoreCase]);
  XML:=StringReplace(XML, '</s>', '<w c="LAST_UPDATED_BY">'+UsID+'</w></s>', [rfReplaceAll]);
  COMResult:=XSLS.Transform(MAX_WAIT,XML,CoreMultipleSQLXSLT,OutValue);
  CheckCOMError(COMResult,OutValue);
  Query:=OutValue;
  COMResult:=ARS.Execute(MAX_WAIT,StrToInt(UsedGrID),-1,Query,False,'','','',OutParameters,OutValue);
  CheckCOMError(COMResult,OutValue);
end;

procedure TWM.DoDelete(XML, DeleteGrID: String; out RowCount:Integer; out OutValue:WideString);
var Query:WideString;
begin
  COMResult:=XSLS.Transform(MAX_WAIT,XML,CoreDeleteSQLXSLT,OutValue);
  CheckCOMError(COMResult,OutValue);
  Query:=OutValue;
  COMResult:=ARS.Delete(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutValue);
  CheckCOMError(COMResult,OutValue);
  RowCount:=StrToInt(OutValue);
end;


function TWM.GetUserIDsCommaList(SQLWhere: String): String;
var
  XML:String;
begin
  XML:='select t.login_user_id from tts.login_user t where '+ SQLWhere +' and grid=:GRID and t.active=''Y''';
  COMResult:=ARS.List(MAX_WAIT,StrToInt(GrID),-1,XML,0,OutValue);
  CheckComError(COMResult, OutValue);
  XML:='<l>'+ OutValue +'</l>';
  COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
  CheckCOMError(COMResult,OutValue);
  Result:=OutValue;
end;

{==============================================================================}
{                              Utility functions                               }
{==============================================================================}
procedure TWM.MarkAsReadMsg(const DetailXML:String);
var
  XML, ToLoginUserID,Reviewed,MessageID:String;
  RowCount:Integer;
begin
  COMResult:=XSLS.Transform(MAX_WAIT, '<d>'+DetailXML+'</d>', ExtractFieldsXSLT, OutValue);
  CheckCOMError(COMResult,OutValue);
  with TStringList.Create do
  try
    Text:=OutValue;
    MessageID:=Values['MESSAGE_ID'];
    ToLoginUserID:=Values['TO_LOGIN_USER_ID'];
    Reviewed:=Values['REVIEWED'];
  finally
    Free;
  end;
  if (ToLoginUserID=UsID) and (Reviewed<>'Y') then
  begin
    XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>B</gb></i><tt><t>TTS.MESSAGE</t></tt><ww><w c="MESSAGE_ID"  k="">'+MessageID+'</w><w c="REVIEWED">Y</w></ww></d>';
    DoUpdate(XML, GrID, RowCount, OutValue);
    CheckCOMError(COMResult,OutValue);
    if SessionNewMsgs > 0 then   //update session data
    begin
      UpdateSessionData('newmsg', IntToStr(SessionNewMsgs-1));
      SessionNewMsgs:=SessionNewMsgs-1;
    end
  end;
end;

procedure TWM.GetPageNumber(out PageNumber:Integer);
begin { GetPageNumber }
  if Request.QueryFields.Values['pn']=EmptyStr then
    PageNumber:=1
  else
    PageNumber:=StrToInt(Request.QueryFields.Values['pn']);
end; { GetPageNumber }

function TWM.GetRequestValue(const FieldName:String):String;
begin
  if Trim(Request.ContentFields.Values[FieldName])<>'' then
      Result:= Trim(Request.ContentFields.Values[FieldName])
  else
      Result:= Trim(Request.QueryFields.Values[FieldName]);
end;

function TWM.CompressResponse: Boolean;
var
  OutValue: WideString;
  COMResult: HResult;
begin { CompressResponse }
  Result:=False;
  if (Response.ContentLength>=COMPRESS_MIN_SIZE) then 
    if Pos('/DEFLATE/','/'+Uppercase(StringReplace(StringReplace(Request.GetFieldByName('HTTP_ACCEPT_ENCODING'),',','/',[rfReplaceAll]),' ','',[rfReplaceAll]))+'/')>0 Then //If client can accept deflated responses Then...
    begin
      if Response.ContentType <> 'application/pdf' then
      begin
        COMResult:=ZS.CompressString(MAX_WAIT,Response.Content,1,0,0,OutValue);
        CheckCOMError(COMResult,OutValue);
        Response.Content:=OutValue;
        Response.ContentEncoding:='deflate';
        Result:=True;
      end;
    end;
end; { CompressResponse }


procedure TWM.CheckCOMError(COMResult: HResult;var ErrorMsg: WideString);
const
  COMErrorBase = -$200;
  COMErrorJNoSessionFound = -$401;
  COMErrorJCommunicationError = -$402;
  COMErrorJAuthenticationError = -$403;
begin { CheckCOMError }
  if COMResult<>0 then
  begin
    if COMResult=COMErrorJNoSessionFound then
      raise ENoJabberSessionFound.Create(ErrorMsg)
    else if COMResult=COMErrorJCommunicationError then
      raise EJabberCommunicationError.Create(ErrorMsg)
    else if COMResult=COMErrorJAuthenticationError then
      raise EJabberAuthenticationError.Create(ErrorMsg)
    else if COMResult=COMErrorBase then
    begin
      if (Pos('ORA-00001: unique constraint (TTS.LOGIN_USER_UK1) violated', ErrorMsg) > 0) then
        raise EDuplicateLogin.Create(ErrorMsg)
      else if (Pos('No data found.', ErrorMsg) > 0) then
        raise ENoDataFound.Create(ErrorMsg)
      else if (Pos('violated - child record found', ErrorMsg) > 0) then
        raise EChildRecordFound.Create(ErrorMsg)
      else if (Pos('Update failed (0 rows updated).', ErrorMsg) > 0) then
        raise ENoRowsUpdated.Create(ErrorMsg)
      else if (Pos('unique constraint (TTS.WORKED_HOUR_UK1) violated', ErrorMsg)>0 ) then
        raise EDuplicateEntryFound.Create(ErrorMsg)
      else
         raise Exception.Create(ErrorMsg) //Kick out the error message returned from the COM server.
      end
    else
      OleCheck(COMResult); //Kick out an EOleSysError with { /update } the appropriate message--this is for *actual* COM system errors.
  end;
end; { CheckCOMError }

function TWM.LoadFile(FileName:String):WideString;
begin { LoadFile }
  with TStringList.Create do
  try
    LoadFromFile(MainDocsPath+FileName);
    Text:= StringReplace(Text, '[include_out_helpers_detail]', OutHelpersDetailXSLT,[]);
    Text:= StringReplace(Text, '[include_out_helpers_list]', OutHelpersListXSLT,[]);
    Result:=Text;
  finally
    Free;
  end;
end; { LoadFile }

function TWM.GenerateBaseXML(const TableName, XMLValues: String; const IgnoreGrID: Boolean): String;
begin
    Result:='<d><i><ss>TTS</ss><gb>';
      if (IgnoreGrID) then Result:=Result+'G';
      Result:=Result+'B</gb></i><tt><t gb="">TTS.'+TableName+'</t></tt><ww>'+XMLValues+'<w c="LAST_UPDATED_BY">'+UsID+'</w></ww></d>';
end;

function TWM.ExtractObjectName(S: String): String;
var
  p:Integer;
begin
    p:=Pos('@',s);
    if (p=0) then result:=S
    else result:=Copy(s,1,p-1);
end;

function TWM.XMLEscape(XML: String): String;
var
  EscapedXML:String;
begin
  EscapedXML:=StringReplace(XML, '#', '#23;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, ' ', '#20;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, '"', '#22;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, '&', '#26;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, '''','#27;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, '/', '#2f;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, ':', '#3a;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, '<', '#3c;', [rfReplaceAll]);
  EscapedXML:=StringReplace(EscapedXML, '>', '#3e;', [rfReplaceAll]);
  Result:=StringReplace(EscapedXML, '@', '#40;', [rfReplaceAll]);
end;



function TWM.ErrorMsg(const Location:String;const ErrorText:String): WideString;
begin
  LogWriter.WriteLog('TimeTrackingSystem.dll',Location, Request.PathInfo,'E','Message', ErrorText,
      ThreadID,ThreadRequestNum,SessionID,True,Request,True,Response);
  if Response.ContentType='text/xml' then
    Result:=ErrorText
  else
    Result:='<div id="e"> '+ErrorText+'<br>please contact <a href="mailto:'+ErrorEmail+'?subject=TTSError&amp;body=Exception in TimeTrackngSystem.dll [/'+ Location +']:' + URLEncode(ErrorText)+ #$D#$A'Request.URL= '+UrlEncode(Request.PathInfo+'?'+Request.Query)+'">'+ErrorEmail+'</a> for assistance<br/> or use our '+
            '<a href=''javascript:void(0)'' onclick=''window.open("support?p=y&s='+SessionID+'" ,"newwin402","height=402,width=388,left=300,top=150,toolbar=no,titlebar=no,location=no,directories=no,status=no,menubar=no,screenX=0,screenY=0,scrollbars=no,resizable=yes",false);''>Live Chat</a> support. <br/><br/>'+
            '<button accesskey="b" onclick="javascript:window.history.back()"><u>B</u>ack</button></div>';
end;

function TWM.Calendars(const Path: String; var StartDate: TDateTime; var Period: Integer): String;
var
   XML:String;
   Month, Day, Year: Word;
   EndDate: TDateTime;
   LastPeriodStartDate: TDateTIme;
begin
    if (StartDate=0) or (StartDate=1) then   //if isn`t assigned - set focused period to current/chosen month
    begin
      if Request.QueryFields.Values['y']<>'' then    //get start date from request
      begin
        Month:=StrToInt(Request.QueryFields.Values['m']);
        Year:=StrToInt(Request.QueryFields.Values['y']);
        Day:=StrToInt(Request.QueryFields.Values['d']);
      end
      else DecodeDate(UserDate, Year, Month, Day);   //or set start date to user current date
      if  StartDate=1 then
        StartDate:=EncodeDate(Year, Month, 1)
      else
        StartDate:=EncodeDate(Year, Month, Day);
    end;
    if Period=-1 then Period:=MONTH_LEN[Month]-1;

    EndDate:=StartDate+Period;
    LastPeriodStartDate:=UserDate -(((Math.Floor(UserDate-GroupStartDate)) Mod GroupPeriod) + GroupPeriod);      

    DecodeDate(IncMonth(StartDate, -1), Year, Month, Day);
    XML:='<d><i><s>'+SessionID+'</s><sti>'+Request.QueryFields.Values['sti']+'</sti><sd>'+FormatDateTime('yyyymmdd',StartDate)+'</sd>'+
        '<lpsd>' + FormatDateTime('yyyymmdd',LastPeriodStartDate) + '</lpsd>' +
        '<npsd>' + FormatDateTime('yyyymmdd', StartDate+Period+1) + '</npsd>' +
        '<ppsd>' + FormatDateTime('yyyymmdd', StartDate-Period+1) + '</ppsd>' +
        '<cpsd>' + FormatDateTime('yyyymmdd', UserDate) + '</cpsd>' +
        '<url>'+ Path +'</url></i>';
    XML:=XML+'<date month="'+IntToStr(Month)+'" year="'+IntToStr(Year)+'" color="b">'+ MonthCalendar(Month, Day, Year, StartDate, EndDate) +'</date>';
    DecodeDate(StartDate, Year, Month, Day);
    XML:=XML+'<date month="'+IntToStr(Month)+'" year="'+IntToStr(Year)+'" color="b">'+ MonthCalendar(Month, Day, Year, StartDate, EndDate) +'</date>';
    DecodeDate(IncMonth(StartDate, 1), Year, Month, Day);
    XML:=XML+'<date month="'+IntToStr(Month)+'" year="'+IntToStr(Year)+'" color="b">'+ MonthCalendar(Month, Day, Year, StartDate, EndDate) +'</date></d>';
    COMResult:=XSLS.Transform(MAX_WAIT, XML, GenerateCalendarXSLT, OutValue);           
    CheckCOMError(COMResult,OutValue);
    Result := OutValue;
end;

function TWM.MonthCalendar(iMonth:Integer; iDay:Integer; iYear:Integer; dtFocusStartDate:TDateTime; dtFocusEndDate:TDateTime ): WideString;
var
  dtDate: TDateTime;
  Rows, I, J: Integer;
  bStop:Boolean;
  HTML:String;
  Month, Year, Day:Word;  
begin
  bStop:=False;
  Rows:=6;
  iDay:=1;
  HTML:='';
  for J:=1 to Rows do
  begin
    HTML:=HTML+'<w>';
    for I:=1 to 7 do
    begin
      dtDate:= EncodeDate(iYear, iMonth, iDay);
      if (I=DayOfWeek(dtDate-1)) and not bStop then
      begin
        HTML:=HTML+'<d';
        if dtDate=UserDate then HTML:=HTML+' id="t"'
        else if (I=6) or (I=7) then  HTML:=HTML+' id="s"';
        if (dtDate>=dtFocusStartDate) and (dtDate<=dtFocusEndDate) then HTML:=HTML+' c="f"'
        else if (dtDate>(UserDate)) or (dtDate<GroupStartDate)  then HTML:=HTML+' c="r"';
        HTML:=HTML+'>'+ IntToStr(iDay)+'</d>';
        iDay:=iDay+1;
        DecodeDate((IncMonth(EncodeDate(iYear, iMonth, 1),1)-1), Year, Month, Day);
        if (iDay>Day) then
        begin
          iDay:=1;iMonth:=iMonth+1;
          if iMonth>12 then
          begin
            iMonth:=1; iYear:=iYear+1;
          end;
          bStop:=true;
        end;
      end
      else
        HTML:=HTML+'<d> </d>';
    end;
    HTML:=HTML+'</w>';
  end;
  Result:=HTML;
end;

function TWM.Msg(const Text, Ico: String): WideString;
begin
  Result:='<div id="e">'+Text+'<br/><button accesskey="b" onclick="javascript:window.history.back()"><u>B</u>ack</button></div>';
end;

procedure TWM.SendMessage(FromUser, ToUsers, Subject, Content:String; Urgent: Boolean; out OutValue:WideString);
var
  I: Integer;
  ToUser, sUrgent, XML: String;
  Query, UserContent,  ResultCustomData, ResultSessionPrivs, ResultSessionDataFilters,ResultSessionID, ObjID: WideString;
  SessionExists:WordBool;
    StartDate, EndDate: TDateTime;
begin
  if Urgent then sUrgent:='<w c="urgent">Y</w>' else sUrgent:='<w c="urgent">N</w>';
  with TStringList.Create do
  try
    CommaText:= ToUsers;
    for  I:=0  to Count-1 do
    begin
      ToUser:=Strings[I];
      Query:='select t.first_name, t.middle_name, t.last_name, t.email, t.user_name, t.user_pass, g.group_name from tts.login_user t, tts.login_group g where t.login_user_id = :login_user_id and t.grid=:GRID and t.grid=g.grid and t.active=''Y''' + #127'login_user_id=' + ToUser;
      COMResult:=ARS.Detail(MAX_WAIT,StrToInt(GrID),-1,Query,OutValue);
      CheckCOMError(COMResult,OutValue);
      XML:='<d>'+OutValue+'</d>';
      COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
      CheckCOMError(COMResult,OutValue);
      with TStringList.Create do
      try
        Text:=OutValue;
        UserContent:=StringReplace(Content, '[RECIPIENT:FIRST_NAME]', Values['FIRST_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:MIDDLE_NAME]', Values['MIDDLE_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:LAST_NAME]', Values['LAST_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:EMAIL]', Values['EMAIL'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:LOGIN_NAME]', Values['USER_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:USER_PASSWORD]', Values['USER_PASS'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:GROUP_NAME]', Values['GROUP_NAME'], [rfReplaceAll, rfIgnoreCase]);
        if (SessionID<>EmptyStr) then                            
        begin
          UserContent:=StringReplace(UserContent, '[LOGIN_USER:FIRST_NAME]',  SessionData.Values['first_name'], [rfReplaceAll, rfIgnoreCase]);
          UserContent:=StringReplace(UserContent, '[LOGIN_USER:LAST_NAME]',  SessionData.Values['last_name'], [rfReplaceAll, rfIgnoreCase]);
          GetLastPeriod(StartDate, EndDate);
          UserContent:=StringReplace(UserContent, '[PAYROLL:LAST_PERIOD_DATES]', FormatDateTime('mm/dd/yyyy',StartDate) + ':' + FormatDateTime('mm/dd/yyyy',EndDate) , [rfReplaceAll, rfIgnoreCase]);
        end;
      finally
        Free;
      end;
      XML:='<d><i><ss>TTS</ss><gb>B</gb><s>'+SessionID+'</s></i><tt><t>TTS.Message</t></tt><ww>'+
           '<w c="message_id"  k="">0</w><w c="from_login_user_id" >'+FromUser+'</w>'+
           '<w c="to_active" >Y</w><w c="from_active" >Y</w>'+
           '<w c="to_login_user_id" >'+ToUser+'</w><w c="LAST_UPDATED_BY" >'+UsID+'</w>'+
           '<w c="subject" ><![CDATA['+Subject+']]></w>'+
           '<w c="text"><![CDATA['+UserContent+']]></w>'+sUrgent+'</ww></d>';
      DoInsert(XML, GrID, ObjID, OutValue);
      CheckCOMError(COMResult,OutValue);
      COMResult := WSS.WebSessionLookup(APP_NAME+ToUser, ResultSessionID, OutValue); //TODO diff session id
      CheckCOMError(COMResult, OutValue);
      if ResultSessionID > EmptyStr then
      begin
        COMResult := WSS.WebSession(ResultSessionID, ResultCustomData, ResultSessionPrivs, ResultSessionDataFilters, OutValue);
        CheckCOMError(COMResult,OutValue);
        if ResultSessionID > EmptyStr then
        begin
          with TStringList.Create do
          try
            Text:=ResultCustomData;
            Values['newmsg']:=IntToStr(StrToInt(Values['newmsg'])+1);
            ResultCustomData:=Text;
          finally
            Free;
          end;
          COMResult:=WSS.UpdateSessionData(ResultSessionID,ResultCustomData,SessionExists,OutValue);
          CheckCOMError(COMResult,OutValue);
        end;
      end;
    end;
  finally
    Free;
  end;
end;

procedure TWM.SendTemplateEmail(const MailTemplateName, ToUsers: String; out OutValue: WideString);
var
Query, XML, Content, FromEmail, CCToUsers, Subject: String;
begin
  Query:='select mail_template_id, name, subject, from_email, cc_email, content, format from tts.mail_template  where up_name = :up_name and ((grid=:grid) or  (grid is null)) and active=''Y''' + #127'up_name=' + UpperCase(MailTemplateName)+#127'grid=' + GrID;
  COMResult:=ARS.Detail(MAX_WAIT,-1,-1,Query,OutValue);
  CheckCOMError(COMResult,OutValue);
  XML:='<d>'+StringReplace(OutValue,  '&#13;&#10;', #127, [rfReplaceAll])+'</d>'; 
  COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);                  
  CheckCOMError(COMResult,OutValue);
  with TStringList.Create do
  try
    Text:=OutValue;    
    CCToUsers:=trim(Values['cc_email']);
    Subject:=trim(Values['subject']);
    Content:= StringReplace(Values['content'], #127, #$D#$A, [rfReplaceAll]);
    FromEmail:=trim(Values['from_email']);
  finally                                                            
    Free;
  end;
  SendEmail(FromEmail, ToUsers,CCToUsers, Subject, Content, False,  OutValue);
end;

procedure TWM.SendEmail(FromUserEmail, ToUsers, CCToUsers, Subject, Content: String; const IsHTML: Boolean;  out OutValue: WideString);
var
  XML, Email: String;
  Query,UserContent: WideString;
  I: Integer;
  StartDate, EndDate: TDateTime;
begin
  with TStringList.Create do
  try
    CommaText:= ToUsers;
    for I:=0 to Count-1 do
    begin
      Query:='select t.first_name, t.middle_name, t.last_name, t.email, t.user_name, t.user_pass, g.group_name from tts.login_user t, tts.login_group g where t.login_user_id = :login_user_id and t.grid=g.grid and t.active=''Y''' + #127'login_user_id=' + Strings[I];
      COMResult:=ARS.Detail(MAX_WAIT,-1,-1,Query,OutValue);
      CheckCOMError(COMResult,OutValue);
      XML:='<d>'+OutValue+'</d>';
      COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
      CheckCOMError(COMResult,OutValue);
      with TStringList.Create do
      try
        Text:=OutValue;
        UserContent:=StringReplace(Content, '[RECIPIENT:FIRST_NAME]', Values['FIRST_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:MIDDLE_NAME]', Values['MIDDLE_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:LAST_NAME]', Values['LAST_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:EMAIL]', Values['EMAIL'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:LOGIN_NAME]', Values['USER_NAME'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:USER_PASSWORD]', Values['USER_PASS'], [rfReplaceAll, rfIgnoreCase]);
        UserContent:=StringReplace(UserContent, '[RECIPIENT:GROUP_NAME]', Values['GROUP_NAME'], [rfReplaceAll, rfIgnoreCase]);
        if (SessionID<>EmptyStr) then                            
        begin
          UserContent:=StringReplace(UserContent, '[LOGIN_USER:FIRST_NAME]',  SessionData.Values['first_name'], [rfReplaceAll, rfIgnoreCase]);
          UserContent:=StringReplace(UserContent, '[LOGIN_USER:LAST_NAME]',  SessionData.Values['last_name'], [rfReplaceAll, rfIgnoreCase]);
          GetLastPeriod(StartDate, EndDate);
          UserContent:=StringReplace(UserContent, '[PAYROLL:LAST_PERIOD_DATES]', FormatDateTime('mm/dd/yyyy',StartDate) + ':' + FormatDateTime('mm/dd/yyyy',EndDate) , [rfReplaceAll, rfIgnoreCase]);
        end;
        Email:= Values['EMAIL'];
      finally
        Free;
      end;
      COMResult:=EMS.Send(MAX_WAIT,Email, CCToUsers , '',0, FromUserEmail,'','', MAIL_AGENT, Subject, IsHTML, UserContent, OutValue);
      try
        CheckComError(COMResult, OutValue);
      except
        on E: Exception do
          raise ESendEmail.Create(E.Message);
      end;
    end;
  finally
    Free;
  end;
end;

function TWM.DoLogin(out OutValue:WideString): Boolean;
var
  CustomData: TStringList;
  LogData, SessionLookupKey, UserName, UserPass, UserOffset: String;
  COMResult: HResult;
  Query, XML, CustomPrivileges, s:WideString;
begin { DoLogin }
  CustomData:=TStringList.Create;
  LogData:='';
  UserName:=Request.ContentFields.Values['username'];
  UserPass:=Request.ContentFields.Values['password'];
  UserOffset:=Request.ContentFields.Values['useroffset'];
  if UserOffset=EmptyStr then UserOffset:='0';
  Query:='select t.login_user_id, t.user_name, r.type as role_id, r.role_id as privilege_role_id,  t.first_name, t.last_name, t.email,g.grid, g.group_name,g.start_date, g.period_in_week, g.hour_format, g.email_error,  '+
           ' (select d.to_do_id from tts.TO_DO_LOG d where d.CHECK_IN_TIME is null and d.login_user_id=t.login_user_id ) as checked_out_to_do , '+
           ' (select count(*) from tts.message m where  (m.to_login_user_id=t.login_user_id) and (m.to_active=''Y'') and m.reviewed_date is null) as newmsg ' +
           ' from tts.login_user t, tts.login_group g, tts.role r ' +
           ' where (t.user_name=:user_name) and (t.user_pass=:user_pass) and (t.grid=g.grid)'+
           ' and (g.up_group_name=:group_name) and (t.active=''Y'') and r.role_id=t.role_id '+
           #127'user_name='+UserName+
           #127'user_pass='+UserPass+
           #127'group_name='+ UpperCase(Request.ContentFields.Values['groupname']);    //todo - tts.account info.hire_date (!tts.login_user.hire_date)

   COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID),  StrToInt(BrID), Query, OutValue);
   CheckCOMError(COMResult,OutValue); 
   XML:='<d>'+OutValue+'</d>';
   COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
   CheckCOMError(COMResult,OutValue);
   try
     CustomData.Text:=OutValue;
     CustomData.Add('useroffset='+UserOffset);
     CustomData.Add('appname=TTS');

       //CustomNavigation:='<p>' + OutValue + '</p>' +LoadFile('login_group_navigation_'+ CustomData.Values['grid'] + DOCEXT_XML) ;
     Query:=' select o.object_name, t.show from tts.login_group_object t, tts.object_privilege o'+
            ' where t.object_privilege_id(+)=o.object_privilege_id and t.grid=:grid'+#127'grid='+CustomData.Values['grid'];
     COMResult:=ARS.List(MAX_WAIT,StrToInt('-1'),StrToInt('-1'),Query,0,OutValue);
     CheckCOMError(COMResult,OutValue);
     s:='<?xml version="1.0"?><menu sg="'+CustomData.Values['role_id']+'" sti="'+ Request.QueryFields.Values['sti'] +'"><p>' + OutValue + '</p>' ;
     s:= s + LoadFile('login_group_navigation' + DOCEXT_XML) + '</menu>';
     COMResult:=XSLS.Transform(MAX_WAIT,s,GenerateMenuXSLT,OutValue);
     CheckCOMError(COMResult,OutValue);
     CustomPrivileges:=OutValue;


     SessionLookupKey := APP_NAME + CustomData.Values['login_user_id'];
     COMResult:=WSS.NewWebSession(SessionLookupKey, CustomData.Text, CustomPrivileges, '', SessionID, OutValue);
     CheckCOMError(COMResult,OutValue);
     SessionData.Assign(CustomData);
   finally
     CustomData.Free;
   end;
   Result:=True;
end; { DoLogin }


procedure TWM.JabberServerConnect(const JabberLogin: String;const JabberPassword: String; out OutValue: WideString);
var
  XML, ObjID:WideString;
begin
  COMResult:=JS.Connect(MAX_WAIT, JabberLogin, JabberPassword, APP_NAME, OutValue); 
  CheckCOMError(COMResult,OutValue);
  if OutValue='CONNECTION_NEW' then
  begin
    XML:='<?xml version="1.0"?><d><i><ss>TTS</ss><gb>B</gb></i><tt><t>tts.chat_log</t></tt><ww><w c="chat_log_id"  k=""></w><w c="login_user_id" >'+UsID+'</w><w c="ACTIVE" >Y</w><w c="LAST_UPDATED_BY">'+UsID+'</w></ww></d>';
    DoInsert(XML, GrID, ObjID, OutValue);
  end;
end;

procedure TWM.UpdateChatLog(const MessageText: String);
var
  Query: String;
begin
  Query:='update tts.chat_log t set t.chat_log=concat(t.chat_log,:MESSAGE_TEXT)  '+
             ' where t.login_user_id=:LOGIN_USER_ID and t.LAST_UPDATED_DATE=(select max(d.LAST_UPDATED_DATE) from tts.chat_log d  where d.login_user_id=t.login_user_id) and t.grid=:GRID' + 
             #127'LOGIN_USER_ID='+UsID +
             #127'MESSAGE_TEXT='+MessageText;
  COMResult:=ARS.Update(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutValue);
  CheckCOMError(COMResult, OutValue);
end;

procedure TWM.PostTravelProfile(TravelProfileID: String);
var
  P, R: TStringStream;
  s: WideString;
  Query, XML: String;
  I:Integer;
begin
  try
    Query:= LoadFile('posttravel_profile'+DOCEXT_SQL)+#127'oid='+ TravelProfileID;
    COMResult:=ARS.Detail(MAX_WAIT, StrToInt(GrID), -1, Query, OutValue);
    CheckComError(COMResult, OutValue);
    XML:='<d>'+OutValue+'</d>';
    COMResult:=XSLS.Transform(MAX_WAIT, XML, ExtractFieldsXSLT, OutValue);
    CheckCOMError(COMResult,OutValue);
    s:=(OutValue);
    R := TStringStream.Create('');
    try
      P := TStringStream.Create('');
      try
      with TStringList.Create do
      try
        Text:=s;
        for I:=0 to Count-1 do
          P.WriteString(LowerCase(Names[I])+'=' + URLEncode(Values[Names[I]]) +'&' );
        P.WriteString('agent_id=29' + '&' + 'client_id=OSI');   //https://www.cb-travel.com/business/profile/- AgentID = 29 (John Saxton)
      finally
          Free;
      end;
        IdHTTPS.Request.ContentType := 'application/x-www-form-urlencoded';
        IdHTTPS.Post(TravelProfileServer+TravelProfileURL, P, R);
      finally;
        P.Free;
      end;
      Query:='update tts.travel_profile t set t.last_send_date=:LAST_SEND_DATE where t.travel_profile_id=:TRAVEL_PROFILE_ID and t.grid=:GRID'+
        #127'TRAVEL_PROFILE_ID='+TravelProfileID+
        #127'LAST_SEND_DATE='+FormatDateTime('mm/dd/yyyy',Now);
      COMResult:=ARS.Update(MAX_WAIT,StrToInt(GrID),-1,Query,False,'','','',OutValue);
      CheckCOMError(COMResult,OutValue);
      s:=R.DataString;
      s:=StringReplace(S,' SRC="/images/' , ' SRC="' + TravelProfileServer + 'images/', [rfReplaceAll]);
      S:= STringReplace(S, 'HREF="/', ' HREF="' + TravelProfileServer, [rfReplaceAll]);
      OutHTML:=S;
    finally
      R.Free;
    end;
  except
    on E: Exception do
      OutHTML := ErrorMsg('posttravel_profile', E.Message);
  end;
end;


procedure TWM.GetLastPeriod(out StartDate, EndDate: TDateTime);
begin
  StartDate:=UserDate -(((Math.Floor(UserDate-GroupStartDate)) Mod GroupPeriod) + GroupPeriod);
  EndDate:=StartDate+GroupPeriod;
end;



procedure TWM.GetPayrollPeriod(out StartDate, EndDate: TDateTime);
var
  InDate: TDateTime;
  Month, Day, Year: Word; 
begin
    //Calculate payroll period start date
    if Request.QueryFields.Values['y']<>'' then
    begin
      Month:=StrToInt(Request.QueryFields.Values['m']);
      Day:=StrToInt(Request.QueryFields.Values['d']);
      Year:=StrToInt(Request.QueryFields.Values['y']);
      InDate:=EncodeDate(Year, Month, Day);
    end
    else InDate:=UserDate;
    StartDate:=InDate -((Math.Floor(InDate-GroupStartDate)) Mod GroupPeriod);
    if (Request.QueryFields.Values['y']='') and (RoID <> IntToStr(ROLE_ID_USER))  then
       StartDate:=StartDate - GroupPeriod;
    EndDate:=StartDate+GroupPeriod-1;

end;


procedure TWM.UpdateSessionData(VarName, NewValue: String);
var
  ResultSessionDataFilters,ResultCustomData,ResultSessionPrivs: WideString;
  SessionExists: WordBool;
begin
  COMResult := WSS.WebSession(SessionID, ResultCustomData, ResultSessionPrivs, ResultSessionDataFilters, OutValue);
  CheckCOMError(COMResult,OutValue);
  with TStringList.Create do
  try
    Text:=ResultCustomData;
    Values[VarName]:=NewValue;
    ResultCustomData:=Text;
  finally
     Free;
  end;
  COMResult:=WSS.UpdateSessionData(SessionID,ResultCustomData, SessionExists,OutValue);
  CheckCOMError(COMResult,OutValue);
end;

procedure TWM.WMWebActionItemOMAtestAction(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  var
    XML:String;
    PageNumber: Integer;
  begin
  try
   LogWriter.WriteLog('TimeTrackingSystem.dll','omatest', Request.PathInfo,'E','Message', 'omatest',
      ThreadID,ThreadRequestNum,SessionID,True,Request,True,Response);
    XML := Request.ContentFields.Values['xml'];
    DoList(XML, PageNumber, False);
    CheckCOMError(COMResult, OutValue);
    Response.ContentType := 'text/xml';
    OutHTML :=  '<d>' + XML + '</d>';
  except
    on E:Exception do
     begin
       Response.ContentType := 'text/html';
       OutHTML :=  E.Message;
     end;
   end;
end;

end.
