create or replace trigger "TTS".TO_DO_LOG_BIU
  before insert or update on TTS.TO_DO_LOG
  for each row
declare
 v_project_id number;
 v_track_date date; 
 v_total_hrs number;
 v_tmp_hrs number;
 v_time_from_midnight number;
 v_login_user_id number;
begin
  if (inserting) and (:new.TO_DO_LOG_ID = 0 or :new.TO_DO_LOG_ID is null) then
    select TTS.TO_DO_LOG_ID_SEQ.nextval into :new.TO_DO_LOG_ID from DUAL;
  elsif (updating) and (:new.TO_DO_LOG_ID<>:old.TO_DO_LOG_ID) then
    raise_application_error(-20000,'Cannot update TO_DO_LOG_ID');
  end if;
  :new.last_updated_date:=sysdate;

  if (inserting) then 
    :new.creation_date:=sysdate;
    :new.created_by:=:new.last_updated_by;
    :new.login_user_id:=:new.last_updated_by;
    :new.check_out_time:=sysdate;
  elsif (updating) and (:old.check_in_time is null and :new.check_in_time is not null) then
    :new.check_in_time:=sysdate;
    select t.project_id into v_project_id from tts.to_do t where t.to_do_id=:old.to_do_id;
    v_track_date:=:old.check_out_time;
    v_total_hrs:=  to_number(ROUND((sysdate - v_track_date)*24, 2));
    v_login_user_id := :old.login_user_id;
    while (v_total_hrs > 0) 
    loop
      v_time_from_midnight:=to_number(to_char(v_track_date,'HH24')) + to_number(ROUND(to_number(to_char(v_track_date,'MI'))/60,2));
      if (v_total_hrs + v_time_from_midnight > 24) then
        v_tmp_hrs:=24-v_time_from_midnight;
        v_total_hrs:=v_total_hrs-v_tmp_hrs;
      else
        v_tmp_hrs:=v_total_hrs;
        v_total_hrs:=0;
      end if;   
      
      update TTS.WORKED_HOUR_V 
        set 
          total  = total + v_tmp_hrs,
          last_updated_by=:new.last_updated_by
        where 
          track_date=trunc(v_track_date)
          and project_id=v_project_id
          and login_user_id=v_login_user_id
          and grid=:old.grid;
    
        if SQL%NOTFOUND then
          insert  into TTS.WORKED_HOUR 
            (login_user_id, project_id, track_date, note, total, last_updated_by, grid)      
          values(
            v_login_user_id,v_project_id,trunc(v_track_date),:new.to_do_id, v_tmp_hrs, v_login_user_id, :new.GRID);
        end if;
        -- set next day from 12.00.00 AM
        v_track_date:=trunc(v_track_date)+1; 
    end loop;
  end if;
 end TO_DO_LOG_BIU;
/
