create or replace trigger TTS.WORKED_HOUR_V_IIUD
  instead of insert or update or delete on TTS.WORKED_HOUR_V
  for each row
declare
 v_new TTS.WORKED_HOUR_V%rowtype;
 v_total_tfd number;
 v_approved_vacation_hours number;
 v_used_vacation_hours number;
 v_approved_holiday_hours number;
 v_used_holiday_hours number;
 v_used_po_hours number;
 v_diff number;
begin
    if (inserting) or (updating) then
      v_new.WORKED_HOUR_V_ID := :new.WORKED_HOUR_V_ID;
      v_new.LOGIN_USER_ID := :new.LOGIN_USER_ID;
      v_new.PROJECT_ID := :new.PROJECT_ID;
      v_new.TRACK_DATE := :new.TRACK_DATE;
      v_new.NOTE := :new.NOTE;
      v_new.LAST_UPDATED_BY := :new.LAST_UPDATED_BY;
      v_new.ACTIVE := :new.ACTIVE;
      v_new.TOTAL := :new.TOTAL;
      v_new.CATEGORY := :new.CATEGORY;
      v_new.SUBMITTED := :new.SUBMITTED;
      v_new.APPROVED := :new.APPROVED;
      v_new.PROCESSED := :new.PROCESSED;
      v_new.NOTIFIED:=:new.NOTIFIED;
   end if;

   
  -- check if worked hours  <= 24 
  select sum(total) into v_total_tfd
  from TTS.worked_hour
  where 
    login_user_id=v_new.login_user_id
    and worked_hour_id <> v_new.worked_hour_v_id
    and track_date=v_new.track_date;
  if ((v_total_tfd+v_new.total)>24) then 
    raise_application_error(-20000,'<br/> Number of hours for ' || v_new.track_date || ' must be less than 24. Please, press the browser "Back" button or use "Back" button below and make sure the information you provided was correct.');
  end if;
  
  --check if sum(V, H & S) worked hours >= vacation_hours in login_user_vacation table
  if v_new.CATEGORY = 'S' or v_new.category='V' or v_new.category='H' then
    if (inserting) or (updating and :old.category='R') then
      v_diff := v_new.total;
    elsif (updating)  then
      v_diff := v_new.total-:old.total;   
    end if;
     
    select nvl(sum(h.total), 0) into v_used_vacation_hours
    from tts.worked_hour h, tts.login_user_vacation v
    where h.track_date between v.start_date and v.end_date
    and v_new.track_date between v.start_date and v.end_date
    and h.category in ('V', 'S')
    and v.login_user_id=h.login_user_id
    and h.login_user_id=v_new.login_user_id
    and v.creation_date = (select max(s.creation_date) from tts.login_user_vacation s where s.login_user_id=h.login_user_id  and s.category in ('V', 'S'));
    begin
      select v.hours into v_approved_vacation_hours
      from tts.login_user_vacation v
      where v_new.track_date between v.start_date and v.end_date
      and v.login_user_id=v_new.login_user_id
      and v.creation_date=(select max(s.creation_date) from tts.login_user_vacation s where s.login_user_id=v.login_user_id and v_new.track_date between v.start_date and v.end_date and s.category in ('V', 'S'));
    exception
      when NO_DATA_FOUND then  
        v_approved_vacation_hours:=0; 
    end;
    if(v_used_vacation_hours+v_diff>v_approved_vacation_hours) then
      raise_application_error(-20000, 'You have exceeded the approved number of vacation hours per this pay period. You can not enter more than '||v_approved_vacation_hours||' vacation hours. ');
    end if;

    select nvl(sum(h.total), 0) into v_used_holiday_hours
    from tts.worked_hour h, tts.login_user_vacation v
    where h.track_date between v.start_date and v.end_date
    and v_new.track_date between v.start_date and v.end_date
    and h.category ='H'
    and v.login_user_id=h.login_user_id
    and h.login_user_id=v_new.login_user_id
    and v.creation_date = (select max(s.creation_date) from tts.login_user_vacation s where s.login_user_id=h.login_user_id and s.category='H');
    begin
      select v.hours into v_approved_holiday_hours
      from tts.login_user_vacation v
      where v_new.track_date between v.start_date and v.end_date
      and v.login_user_id=v_new.login_user_id
      and v.creation_date=(select max(s.creation_date) from tts.login_user_vacation s where s.login_user_id=v.login_user_id and v_new.track_date between v.start_date and v.end_date and s.category='H');
    exception
      when NO_DATA_FOUND then  
        v_approved_vacation_hours:=0; 
    end;
    if(v_used_holiday_hours+v_diff>v_approved_holiday_hours) then
      raise_application_error(-20000, 'You have exceeded the approved number of holiday hours per this pay period. You can not enter more than '||v_approved_holiday_hours||' holiday hours. ');
    end if;


  end if;

  -- check if sum processed worked hours > than PO hours for that payroll period
/*if v_new.processed='Y' and (:old.processed ='N' or :old.processed is null) then

select t.hours - sum(h.total+v_new.TOTAL)  into v_used_po_hours from tts.purchase_order t, tts.worked_hour h  where 
t.login_user_id in (:old.login_user_id)
and sysdate>t.start_date and sysdate<t.end_date
and t.creation_date = (
    select max(s.creation_date) 
    from tts.purchase_order s 
    where s.login_user_id=t.login_user_id 
    and sysdate>s.start_date 
    and sysdate<s.end_date
    )
and h.login_user_id=t.login_user_id 
and h.processed_date between t.start_date and t.end_date and t.grid=:old.grid and h.grid=:old.grid
group by t.hours;

end if;*/
  --end PO check
     
   
   
  if (inserting) then
    insert  into TTS.WORKED_HOUR 
      (login_user_id, 
      project_id, 
      track_date, 
      total, 
      category, 
      note, 
      submitted, 
      last_updated_by, 
      grid)      
    values(
      v_new.LOGIN_USER_ID,
      v_new.PROJECT_ID,
      v_new.TRACK_DATE,
      v_new.TOTAL,
      v_new.CATEGORY,
      v_new.NOTE,
      v_new.SUBMITTED,
      v_new.LAST_UPDATED_BY,
      :new.GRID
      ); 

 elsif (updating) then 
    if v_new.WORKED_HOUR_V_ID is null then
      update TTS.worked_hour_v 
      set 
        total  = total + v_new.total,
        last_updated_by=:new.last_updated_by
       where 
        track_date=v_new.track_date
        and project_id=v_new.project_id
        and login_user_id=v_new.login_user_id;
       
      if SQL%NOTFOUND then
        insert  into TTS.WORKED_HOUR 
          (login_user_id, 
          project_id, 
          track_date, 
          total, 
          category, 
          note, 
          submitted, 
          last_updated_by, 
          grid)      
        values(
          v_new.LOGIN_USER_ID,
          v_new.PROJECT_ID,
          v_new.TRACK_DATE,
          v_new.TOTAL,
          v_new.CATEGORY,
          v_new.NOTE,
          v_new.SUBMITTED,
          v_new.LAST_UPDATED_BY,
          :new.GRID
          ); 
      end if;         
    end if;      
 
    update TTS.WORKED_HOUR set
      LOGIN_USER_ID = v_new.LOGIN_USER_ID,
      PROJECT_ID = v_new.PROJECT_ID,
      NOTE = v_new.NOTE,
      APPROVED = v_new.APPROVED,
      LAST_UPDATED_BY = v_new.LAST_UPDATED_BY,
      ACTIVE = v_new.ACTIVE,
      TOTAL=(v_new.TOTAL),
      CATEGORY =v_new.CATEGORY,
      SUBMITTED=v_new.SUBMITTED,
      PROCESSED=v_new.PROCESSED,
      NOTIFIED=v_new.NOTIFIED, 
      INVOICED=v_new.INVOICED
    where
      WORKED_HOUR_ID = v_new.WORKED_HOUR_V_ID;  
  elsif(deleting) then
    delete from TTS.WORKED_HOUR where WORKED_HOUR_ID = :old.WORKED_HOUR_V_ID;
  end if;
end WORKED_HOUR_V_IIUD;
/
